@@ -1,6 +1,5 @@
 import difflib
 
-from bs4 import BeautifulSoup
 from django.core.exceptions import ImproperlyConfigured
 from django.db import models
 from django.utils.encoding import force_str
@@ -13,12 +12,7 @@ from taggit.managers import TaggableManager
 from wagtail import blocks
 from wagtail.fields import RichTextField, StreamField
 from wagtail.utils.registry import ModelFieldRegistry
-
-
-def text_from_html(val):
-    # Return the unescaped text content of an HTML string
-    return BeautifulSoup(force_str(val), "html5lib").getText()
-
+from wagtail.utils.text import text_from_html
 
 comparison_class_registry = ModelFieldRegistry()
 
@@ -115,6 +109,8 @@ def get_comparison_class_for_block(block):
         return StructBlockComparison
     elif isinstance(block, blocks.StreamBlock):
         return StreamBlockComparison
+    elif isinstance(block, blocks.ListBlock):
+        return ListBlockComparison
     else:
         # As all stream field blocks have a HTML representation, fall back to diffing that.
         return RichTextBlockComparison
@@ -210,13 +206,21 @@ class StructBlockComparison(BlockComparison):
         )
 
 
-class StreamBlockComparison(BlockComparison):
-    def get_block_comparisons(self):
-        a_blocks = list(self.val_a) or []
-        b_blocks = list(self.val_b) or []
+class BaseSequenceBlockComparison(BlockComparison):
+    @staticmethod
+    def get_blocks_from_value(val):
+        raise NotImplementedError
 
+    @staticmethod
+    def get_blocks_by_id(a_blocks, b_blocks):
         a_blocks_by_id = {block.id: block for block in a_blocks}
         b_blocks_by_id = {block.id: block for block in b_blocks}
+        return a_blocks_by_id, b_blocks_by_id
+
+    def get_block_comparisons_by_id(self):
+        a_blocks = self.get_blocks_from_value(self.val_a)
+        b_blocks = self.get_blocks_from_value(self.val_b)
+        a_blocks_by_id, b_blocks_by_id = self.get_blocks_by_id(a_blocks, b_blocks)
 
         deleted_ids = a_blocks_by_id.keys() - b_blocks_by_id.keys()
 
@@ -290,13 +294,80 @@ class StreamBlockComparison(BlockComparison):
                 block_rendered = comparison.htmlvalue(comparison.val_a)
 
             classes = " ".join(classes)
-            comparisons_html.append(
-                '<div class="{0}">{1}</div>'.format(classes, block_rendered)
-            )
+            comparisons_html.append(f'<div class="{classes}">{block_rendered}</div>')
 
         return mark_safe("\n".join(comparisons_html))
 
 
+class StreamBlockComparison(BaseSequenceBlockComparison):
+    @staticmethod
+    def get_blocks_from_value(val):
+        blocks = list(val) or []
+        return blocks
+
+    def get_block_comparisons(self):
+        return self.get_block_comparisons_by_id()
+
+
+class ListBlockComparison(BaseSequenceBlockComparison):
+    @staticmethod
+    def get_blocks_from_value(val):
+        blocks = list(val.bound_blocks) or []
+        return blocks
+
+    def get_block_comparisons(self):
+        a_blocks = self.get_blocks_from_value(self.val_a)
+        b_blocks = self.get_blocks_from_value(self.val_b)
+
+        # ListBlock could be in one of two formats.
+        # If IDs in both a and b are all newly created (ie we're loading data in the old format)
+        # there's no point in using id for identifying which blocks are the same vs moved / added / deleted.
+        both_in_new_format = any(block.original_id for block in a_blocks) and any(
+            block.original_id for block in b_blocks
+        )
+
+        if both_in_new_format:
+            # All blocks have ids, so we can use the BaseSequenceBlock comparison logic, which uses ids.
+            return self.get_block_comparisons_by_id()
+
+        # We're dealing with data in the old format
+        # Let's compare blocks by position - it's not perfect, but it's better than rendering to HTML
+        comparisons = []
+        comparison_class = get_comparison_class_for_block(self.block.child_block)
+
+        a_length = len(a_blocks)
+        b_length = len(b_blocks)
+
+        for index, block in enumerate(b_blocks):
+            if index < a_length:
+                # Assume blocks with the same index are the same changed block
+                # (not necessarily true as blocks could be moved)
+                # Changed/existing block
+                comparisons.append(
+                    comparison_class(
+                        block.block,
+                        True,
+                        True,
+                        a_blocks[index].value,
+                        block.value,
+                    )
+                )
+            else:
+                # New block
+                comparisons.append(
+                    comparison_class(block.block, False, True, None, block.value)
+                )
+
+        if a_length > b_length:
+            for block in a_blocks[b_length:]:
+                # Deleted blocks
+                comparisons.append(
+                    comparison_class(block.block, True, False, block.value, None)
+                )
+
+        return comparisons
+
+
 class StreamFieldComparison(FieldComparison):
     def has_block_ids(self, val):
         if not val:
@@ -423,11 +494,12 @@ class ChildRelationComparison:
     is_field = False
     is_child_relation = True
 
-    def __init__(self, field, field_comparisons, obj_a, obj_b):
+    def __init__(self, field, field_comparisons, obj_a, obj_b, label=""):
         self.field = field
         self.field_comparisons = field_comparisons
         self.val_a = getattr(obj_a, field.related_name)
         self.val_b = getattr(obj_b, field.related_name)
+        self.label = label
 
     def field_label(self):
         """
@@ -437,7 +509,12 @@ class ChildRelationComparison:
 
         if verbose_name is None:
             # Relations don't have a verbose_name
-            verbose_name = self.field.name.replace("_", " ")
+            if self.label:
+                # If the panel has a label, we set it instead.
+                # See InlinePanel.get_comparison for usage
+                verbose_name = self.label
+            else:
+                verbose_name = self.field.name.replace("_", " ")
 
         return capfirst(verbose_name)
 
