@@ -1,9 +1,9 @@
 # definitions which are not being deprecated from wagtail.admin.forms
-from .models import (  # NOQA
+from .models import (  # NOQA: F401
     DIRECT_FORM_FIELD_OVERRIDES,
     FORM_FIELD_OVERRIDES,
     WagtailAdminModelForm,
     WagtailAdminModelFormMetaclass,
     formfield_for_dbfield,
 )
-from .pages import WagtailAdminPageForm  # NOQA
+from .pages import WagtailAdminPageForm  # NOQA: F401
