@@ -1,6 +1,5 @@
 from django import forms
 from django.conf import settings
-from django.utils import timezone
 from django.utils.translation import gettext as _
 from django.utils.translation import ngettext
 
@@ -23,12 +22,19 @@ class CopyForm(forms.Form):
         )
         allow_unicode = getattr(settings, "WAGTAIL_ALLOW_UNICODE_SLUGS", True)
         self.fields["new_slug"] = forms.SlugField(
-            initial=self.page.slug, label=_("New slug"), allow_unicode=allow_unicode
+            initial=self.page.slug,
+            label=_("New slug"),
+            allow_unicode=allow_unicode,
+            widget=widgets.SlugInput,
         )
         self.fields["new_parent_page"] = forms.ModelChoiceField(
             initial=self.page.get_parent(),
             queryset=Page.objects.all(),
-            widget=widgets.AdminPageChooser(can_choose_root=True, user_perms="copy_to"),
+            widget=widgets.AdminPageChooser(
+                target_models=self.page.specific_class.allowed_parent_page_models(),
+                can_choose_root=True,
+                user_perms="copy_to",
+            ),
             label=_("New parent page"),
             help_text=_("This copy will be a child of this given parent page."),
         )
@@ -102,9 +108,9 @@ class CopyForm(forms.Form):
             self._errors["new_slug"] = self.error_class(
                 [
                     _(
-                        'This slug is already in use within the context of its parent page "%s"'
+                        'This slug is already in use within the context of its parent page "%(parent_page_title)s"'
                     )
-                    % parent_page
+                    % {"parent_page_title": parent_page}
                 ]
             )
             # The slug is no longer valid, hence remove it from cleaned_data
@@ -122,6 +128,29 @@ class CopyForm(forms.Form):
 
 
 class PageViewRestrictionForm(BaseViewRestrictionForm):
+    def __init__(self, *args, **kwargs):
+        # get the list of private page options from the page
+        private_page_options = kwargs.pop("private_page_options", [])
+
+        super().__init__(*args, **kwargs)
+
+        if not getattr(settings, "WAGTAIL_PRIVATE_PAGE_OPTIONS", {}).get(
+            "SHARED_PASSWORD", True
+        ):
+            self.fields["restriction_type"].choices = [
+                choice
+                for choice in PageViewRestriction.RESTRICTION_CHOICES
+                if choice[0] != PageViewRestriction.PASSWORD
+            ]
+            del self.fields["password"]
+        # Remove the fields that are not allowed for the page
+        self.fields["restriction_type"].choices = [
+            choice
+            for choice in self.fields["restriction_type"].choices
+            if choice[0] in private_page_options
+            or choice[0] == PageViewRestriction.NONE
+        ]
+
     class Meta:
         model = PageViewRestriction
         fields = ("restriction_type", "password", "groups")
@@ -179,38 +208,18 @@ class WagtailAdminPageForm(WagtailAdminModelForm):
     def clean(self):
         cleaned_data = super().clean()
         if "slug" in self.cleaned_data:
-            if not Page._slug_is_available(
-                cleaned_data["slug"], self.parent_page, self.instance
-            ):
+            page_slug = cleaned_data["slug"]
+            if not Page._slug_is_available(page_slug, self.parent_page, self.instance):
                 self.add_error(
-                    "slug", forms.ValidationError(_("This slug is already in use"))
+                    "slug",
+                    forms.ValidationError(
+                        _(
+                            "The slug '%(page_slug)s' is already in use within the parent page"
+                        )
+                        % {"page_slug": page_slug}
+                    ),
                 )
 
-        # Check scheduled publishing fields
-        go_live_at = cleaned_data.get("go_live_at")
-        expire_at = cleaned_data.get("expire_at")
-
-        # Go live must be before expire
-        if go_live_at and expire_at:
-            if go_live_at > expire_at:
-                msg = _("Go live date/time must be before expiry date/time")
-                self.add_error("go_live_at", forms.ValidationError(msg))
-                self.add_error("expire_at", forms.ValidationError(msg))
-
-        # Expire at must be in the future
-        if expire_at and expire_at < timezone.now():
-            self.add_error(
-                "expire_at",
-                forms.ValidationError(_("Expiry date/time must be in the future")),
-            )
-
-        # Don't allow an existing first_published_at to be unset by clearing the field
-        if (
-            "first_published_at" in cleaned_data
-            and not cleaned_data["first_published_at"]
-        ):
-            del cleaned_data["first_published_at"]
-
         return cleaned_data
 
 
@@ -233,3 +242,39 @@ class MoveForm(forms.Form):
             label=_("New parent page"),
             help_text=_("Select a new parent for this page."),
         )
+
+
+class ParentChooserForm(forms.Form):
+    def __init__(self, child_page_type, user, *args, **kwargs):
+        self.child_page_type = child_page_type
+        self.user = user
+        super().__init__(*args, **kwargs)
+        self.fields["parent_page"] = forms.ModelChoiceField(
+            queryset=Page.objects.all(),
+            widget=widgets.AdminPageChooser(
+                target_models=self.child_page_type.allowed_parent_page_models(),
+                can_choose_root=True,
+                user_perms="add_subpage",
+            ),
+            label=_("Parent page"),
+            help_text=_("The new page will be a child of this given parent page."),
+        )
+
+    def clean_parent_page(self):
+        parent_page = self.cleaned_data["parent_page"].specific_deferred
+        if not parent_page.permissions_for_user(self.user).can_add_subpage():
+            raise forms.ValidationError(
+                _('You do not have permission to create a page under "%(page_title)s".')
+                % {"page_title": parent_page.get_admin_display_title()}
+            )
+        if not self.child_page_type.can_create_at(parent_page):
+            raise forms.ValidationError(
+                _(
+                    'You cannot create a page of type "%(page_type)s" under "%(page_title)s".'
+                )
+                % {
+                    "page_type": self.child_page_type.get_verbose_name(),
+                    "page_title": parent_page.get_admin_display_title(),
+                }
+            )
+        return parent_page
