@@ -1,35 +1,30 @@
-from django.template.loader import render_to_string
+from django.forms import Media
 
+from wagtail import hooks
 from wagtail.admin.auth import user_has_any_page_permission
 from wagtail.admin.navigation import get_site_for_user
-from wagtail.core import hooks
-from wagtail.core.models import Page, Site
+from wagtail.admin.ui.components import Component
+from wagtail.models import Page, Site
 
 
-class SummaryItem:
+class SummaryItem(Component):
     order = 100
 
     def __init__(self, request):
         self.request = request
 
-    def get_context(self):
-        return {}
-
-    def render(self):
-        return render_to_string(self.template, self.get_context(), request=self.request)
-
     def is_shown(self):
         return True
 
 
 class PagesSummaryItem(SummaryItem):
     order = 100
-    template = 'wagtailadmin/home/site_summary_pages.html'
+    template_name = "wagtailadmin/home/site_summary_pages.html"
 
-    def get_context(self):
+    def get_context_data(self, parent_context):
         site_details = get_site_for_user(self.request.user)
-        root_page = site_details['root_page']
-        site_name = site_details['site_name']
+        root_page = site_details["root_page"]
+        site_name = site_details["site_name"]
 
         if root_page:
             page_count = Page.objects.descendant_of(root_page, inclusive=True).count()
@@ -50,35 +45,34 @@ class PagesSummaryItem(SummaryItem):
             page_count = 0
 
         return {
-            'root_page': root_page,
-            'total_pages': page_count,
-            'site_name': site_name,
+            "root_page": root_page,
+            "total_pages": page_count,
+            "site_name": site_name,
         }
 
     def is_shown(self):
         return user_has_any_page_permission(self.request.user)
 
 
-@hooks.register('construct_homepage_summary_items')
-def add_pages_summary_item(request, items):
-    items.append(PagesSummaryItem(request))
-
-
-class SiteSummaryPanel:
-    name = 'site_summary'
-    order = 100
+class SiteSummaryPanel(Component):
+    template_name = "wagtailadmin/home/site_summary.html"
 
     def __init__(self, request):
         self.request = request
-        self.summary_items = []
-        for fn in hooks.get_hooks('construct_homepage_summary_items'):
-            fn(request, self.summary_items)
+        summary_items = []
+        for fn in hooks.get_hooks("construct_homepage_summary_items"):
+            fn(request, summary_items)
+        self.summary_items = [s for s in summary_items if s.is_shown()]
+        self.summary_items.sort(key=lambda p: p.order)
 
-    def render(self):
-        summary_items = [s for s in self.summary_items if s.is_shown()]
-        if not summary_items:
-            return ''
+    def get_context_data(self, parent_context):
+        context = super().get_context_data(parent_context)
+        context["summary_items"] = self.summary_items
+        return context
 
-        return render_to_string('wagtailadmin/home/site_summary.html', {
-            'summary_items': sorted(summary_items, key=lambda p: p.order),
-        }, request=self.request)
+    @property
+    def media(self):
+        media = Media()
+        for item in self.summary_items:
+            media += item.media
+        return media
