@@ -4,14 +4,20 @@ import json
 
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group, Permission
+from django.test import TestCase
 from django.test.utils import override_settings
 from django.urls import reverse
 from django.utils import timezone
 
 from wagtail import hooks
-from wagtail.api.v2.tests.test_pages import TestPageDetail, TestPageListing
+from wagtail.api.v2.tests.test_pages import (
+    TestPageDetail,
+    TestPageListing,
+    TestPageListingSearch,
+)
 from wagtail.models import GroupPagePermission, Locale, Page, PageLogEntry
 from wagtail.test.demosite import models
+from wagtail.test.i18n.models import TestPage
 from wagtail.test.testapp.models import (
     EventIndex,
     EventPage,
@@ -650,8 +656,8 @@ class TestAdminPageListing(AdminAPITestCase, TestPageListing):
                 set(page.keys()), {"id", "meta", "title", "admin_display_title"}
             )
 
-        self.assertTrue(blog_page_seen, "No blog pages were found in the items")
-        self.assertTrue(event_page_seen, "No event pages were found in the items")
+        self.assertTrue(blog_page_seen, msg="No blog pages were found in the items")
+        self.assertTrue(event_page_seen, msg="No event pages were found in the items")
 
     # Not applicable to the admin API
     test_site_filter_same_hostname_returns_error = None
@@ -1100,7 +1106,20 @@ class TestAdminPageDetail(AdminAPITestCase, TestPageDetail):
         )
 
 
-class TestAdminPageDetailWithStreamField(AdminAPITestCase):
+class TestAdminPageListingSearch(AdminAPITestCase, TestPageListingSearch):
+    fixtures = ["demosite.json"]
+
+    def get_response(self, **params):
+        return self.client.get(reverse("wagtailadmin_api:pages:listing"), params)
+
+    def get_page_id_list(self, content):
+        return [page["id"] for page in content["items"]]
+
+    def get_homepage(self):
+        return Page.objects.get(slug="home-page")
+
+
+class TestAdminPageDetailWithStreamField(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def setUp(self):
@@ -1143,7 +1162,7 @@ class TestAdminPageDetailWithStreamField(AdminAPITestCase):
         self.assertEqual(content["body"][0]["value"], 1)
 
 
-class TestCustomAdminDisplayTitle(AdminAPITestCase):
+class TestCustomAdminDisplayTitle(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def setUp(self):
@@ -1174,7 +1193,7 @@ class TestCustomAdminDisplayTitle(AdminAPITestCase):
         )
 
 
-class TestCopyPageAction(AdminAPITestCase):
+class TestCopyPageAction(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def get_response(self, page_id, data):
@@ -1291,7 +1310,7 @@ class TestCopyPageAction(AdminAPITestCase):
         self.assertEqual(response.status_code, 403)
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content, {"detail": "You do not have permission to copy this page"}
         )
 
     def test_recursively_copy_into_self(self):
@@ -1329,7 +1348,7 @@ class TestCopyPageAction(AdminAPITestCase):
         self.assertEqual(response.status_code, 403)
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content, {"detail": "You do not have permission to copy this page"}
         )
 
     def test_without_publish_permissions_at_destination_with_keep_live(self):
@@ -1356,7 +1375,12 @@ class TestCopyPageAction(AdminAPITestCase):
         self.assertEqual(response.status_code, 403)
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content,
+            {
+                "detail": (
+                    "You do not have permission to publish a page at the destination"
+                )
+            },
         )
 
     def test_respects_page_creation_rules(self):
@@ -1366,7 +1390,7 @@ class TestCopyPageAction(AdminAPITestCase):
         self.assertEqual(response.status_code, 403)
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content, {"detail": "You do not have permission to copy this page"}
         )
 
     def test_copy_page_slug_in_use(self):
@@ -1379,10 +1403,17 @@ class TestCopyPageAction(AdminAPITestCase):
 
         self.assertEqual(response.status_code, 400)
         content = json.loads(response.content.decode("utf-8"))
-        self.assertEqual(content, {"slug": ["This slug is already in use"]})
+        self.assertEqual(
+            content,
+            {
+                "slug": [
+                    "The slug 'events' is already in use within the parent page at '/'"
+                ]
+            },
+        )
 
 
-class TestConvertAliasPageAction(AdminAPITestCase):
+class TestConvertAliasPageAction(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def setUp(self):
@@ -1455,7 +1486,7 @@ class TestConvertAliasPageAction(AdminAPITestCase):
         self.assertEqual(response.status_code, 403)
 
 
-class TestDeletePageAction(AdminAPITestCase):
+class TestDeletePageAction(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def get_response(self, page_id):
@@ -1486,14 +1517,14 @@ class TestDeletePageAction(AdminAPITestCase):
         self.assertEqual(response.status_code, 403)
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content, {"detail": "You do not have permission to delete this page"}
         )
 
         # Page is still there
         self.assertTrue(Page.objects.filter(id=4).exists())
 
 
-class TestPublishPageAction(AdminAPITestCase):
+class TestPublishPageAction(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def get_response(self, page_id):
@@ -1534,7 +1565,7 @@ class TestPublishPageAction(AdminAPITestCase):
         self.assertEqual(response.status_code, 403)
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content, {"detail": "You do not have permission to publish this page"}
         )
 
     def test_publish_alias_page(self):
@@ -1556,7 +1587,7 @@ class TestPublishPageAction(AdminAPITestCase):
         )
 
 
-class TestUnpublishPageAction(AdminAPITestCase):
+class TestUnpublishPageAction(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def get_response(self, page_id, data):
@@ -1624,11 +1655,11 @@ class TestUnpublishPageAction(AdminAPITestCase):
         self.assertEqual(response.status_code, 403)
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content, {"detail": "You do not have permission to unpublish this page"}
         )
 
 
-class TestMovePageAction(AdminAPITestCase):
+class TestMovePageAction(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def get_response(self, page_id, data):
@@ -1656,7 +1687,13 @@ class TestMovePageAction(AdminAPITestCase):
 
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content,
+            {
+                "detail": (
+                    "You do not have permission to move the page to the "
+                    "target specified."
+                ),
+            },
         )
 
     def test_move_page_without_destination_page_id(self):
@@ -1667,7 +1704,7 @@ class TestMovePageAction(AdminAPITestCase):
         self.assertEqual(content, {"destination_page_id": ["This field is required."]})
 
 
-class TestCopyForTranslationAction(AdminAPITestCase):
+class TestCopyForTranslationAction(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def get_response(self, page_id, data):
@@ -1750,8 +1787,78 @@ class TestCopyForTranslationAction(AdminAPITestCase):
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(content, {"message": "No Locale matches the given query."})
 
+    def test_translating_latest_non_draft_page_revision(self):
+        old_index_title = self.en_eventindex.title
+        old_post_title = self.en_eventpage.title
+        new_index_title = old_index_title + "-77777"
+        new_post_title = old_post_title + "-77777"
+        self.en_eventindex.title = new_index_title
+        self.en_eventindex.save_revision(log_action=True)
+        self.en_eventpage.title = new_post_title
+        self.en_eventpage.save_revision(log_action=True)
 
-class TestCreatePageAliasAction(AdminAPITestCase):
+        response = self.get_response(
+            self.en_eventindex.id,
+            {"locale": "fr", "copy_parents": True, "recursive": True},
+        )
+
+        assert response.status_code == 201
+
+        new_index_page = [
+            trans_page
+            for trans_page in self.en_eventindex.get_translations()
+            if trans_page.locale.language_code == "fr"
+        ][0]
+        assert new_index_page.title == old_index_title
+        new_post_page = [
+            trans_page
+            for trans_page in self.en_eventpage.get_translations()
+            if trans_page.locale.language_code == "fr"
+        ][0]
+        assert new_post_page.title == old_post_title
+
+    def test_translating_latest_draft_page_revision(self):
+        """In case when Page have only draft revisions"""
+
+        draft_index_page = TestPage(title="Draft Blog", slug="draft_blog", live=False)
+        self.en_homepage.add_child(instance=draft_index_page)
+        draft_blog_post = TestPage(
+            title="Draft Blog post", slug="draft_blog-post", live=False
+        )
+        draft_index_page.add_child(instance=draft_blog_post)
+
+        old_index_title = draft_index_page.title
+        new_index_title = old_index_title + "-77777"
+        draft_index_page.title = new_index_title
+        draft_index_page.save_revision(log_action=True)
+
+        old_page_title = draft_blog_post.title
+        new_page_title = old_page_title + "-77777"
+        draft_blog_post.title = new_page_title
+        draft_blog_post.save_revision(log_action=True)
+
+        response = self.get_response(
+            draft_index_page.id,
+            {"locale": "fr", "copy_parents": True, "recursive": True},
+        )
+
+        assert response.status_code == 201
+
+        new_index_page = [
+            trans_page
+            for trans_page in draft_index_page.get_translations()
+            if trans_page.locale.language_code == "fr"
+        ][0]
+        assert new_index_page.title == new_index_title
+        new_post_page = [
+            trans_page
+            for trans_page in draft_blog_post.get_translations()
+            if trans_page.locale.language_code == "fr"
+        ][0]
+        assert new_post_page.title == new_page_title
+
+
+class TestCreatePageAliasAction(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def setUp(self):
@@ -1859,11 +1966,16 @@ class TestCreatePageAliasAction(AdminAPITestCase):
 
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content,
+            {
+                "detail": (
+                    "You do not have permission to publish a page at the destination"
+                ),
+            },
         )
 
 
-class TestRevertToPageRevisionAction(AdminAPITestCase):
+class TestRevertToPageRevisionAction(AdminAPITestCase, TestCase):
     fixtures = ["test.json"]
 
     def setUp(self):
@@ -1916,7 +2028,7 @@ class TestRevertToPageRevisionAction(AdminAPITestCase):
 
         content = json.loads(response.content.decode("utf-8"))
         self.assertEqual(
-            content, {"detail": "You do not have permission to perform this action."}
+            content, {"detail": "You do not have permission to edit this page"}
         )
 
     def test_revert_to_page_revision_without_revision_id(self):
