@@ -13,7 +13,7 @@ from wagtail.test.testapp.models import SimplePage
 from wagtail.test.utils import WagtailTestUtils
 
 
-class TestBulkPublish(TestCase, WagtailTestUtils):
+class TestBulkPublish(WagtailTestUtils, TestCase):
     def setUp(self):
         self.root_page = Page.objects.get(id=2)
 
@@ -71,12 +71,12 @@ class TestBulkPublish(TestCase, WagtailTestUtils):
 
     def test_publish_view(self):
         """
-        This tests that the publish view responds with an publish confirm page
+        This tests that the publish view responds with a publish confirm page
         """
         # Request confirm publish page
         response = self.client.get(self.url)
 
-        # # Check that the user received an publish confirm page
+        # # Check that the user received a publish confirm page
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(
             response, "wagtailadmin/pages/bulk_actions/confirm_bulk_publish.html"
@@ -130,9 +130,7 @@ class TestBulkPublish(TestCase, WagtailTestUtils):
         )
 
         for child_page in self.pages_to_be_published:
-            self.assertInHTML(
-                "<li>{page_title}</li>".format(page_title=child_page.title), html
-            )
+            self.assertInHTML(f"<li>{child_page.title}</li>", html)
 
     def test_publish_view_post(self):
         """
@@ -142,30 +140,33 @@ class TestBulkPublish(TestCase, WagtailTestUtils):
         mock_handler = mock.MagicMock()
         page_published.connect(mock_handler)
 
-        # Post to the publish page
-        response = self.client.post(self.url)
+        try:
+            # Post to the publish page
+            response = self.client.post(self.url)
 
-        # Should be redirected to explorer page
-        self.assertEqual(response.status_code, 302)
+            # Should be redirected to explorer page
+            self.assertEqual(response.status_code, 302)
 
-        # Check that the child pages were published
-        for child_page in self.pages_to_be_published:
-            published_page = SimplePage.objects.get(id=child_page.id)
-            self.assertTrue(published_page.live)
-            self.assertIn("Hello published", published_page.content)
+            # Check that the child pages were published
+            for child_page in self.pages_to_be_published:
+                published_page = SimplePage.objects.get(id=child_page.id)
+                self.assertTrue(published_page.live)
+                self.assertIn("Hello published", published_page.content)
 
-        # Check that the child pages not to be published remain
-        for child_page in self.pages_not_to_be_published:
-            self.assertFalse(Page.objects.get(id=child_page.id).live)
+            # Check that the child pages not to be published remain
+            for child_page in self.pages_not_to_be_published:
+                self.assertFalse(Page.objects.get(id=child_page.id).live)
 
-        # Check that the page_published signal was fired
-        self.assertEqual(mock_handler.call_count, len(self.pages_to_be_published))
+            # Check that the page_published signal was fired
+            self.assertEqual(mock_handler.call_count, len(self.pages_to_be_published))
 
-        for i, child_page in enumerate(self.pages_to_be_published):
-            mock_call = mock_handler.mock_calls[i][2]
-            self.assertEqual(mock_call["sender"], child_page.specific_class)
-            self.assertEqual(mock_call["instance"], child_page)
-            self.assertIsInstance(mock_call["instance"], child_page.specific_class)
+            for i, child_page in enumerate(self.pages_to_be_published):
+                mock_call = mock_handler.mock_calls[i][2]
+                self.assertEqual(mock_call["sender"], child_page.specific_class)
+                self.assertEqual(mock_call["instance"], child_page)
+                self.assertIsInstance(mock_call["instance"], child_page.specific_class)
+        finally:
+            page_published.disconnect(mock_handler)
 
     def test_after_publish_page(self):
         def hook_func(request, action_type, pages, action_class_instance):
@@ -205,12 +206,12 @@ class TestBulkPublish(TestCase, WagtailTestUtils):
 
     def test_publish_descendants_view(self):
         """
-        This tests that the publish view responds with an publish confirm page that does not contain the form field 'include_descendants'
+        This tests that the publish view responds with a publish confirm page that does not contain the form field 'include_descendants'
         """
         # Get publish page for page with no descendants
         response = self.client.get(self.url)
 
-        # Check that the user received an publish confirm page
+        # Check that the user received a publish confirm page
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(
             response, "wagtailadmin/pages/bulk_actions/confirm_bulk_publish.html"
@@ -218,11 +219,11 @@ class TestBulkPublish(TestCase, WagtailTestUtils):
         # Check the form does not contain the checkbox field include_descendants
         self.assertNotContains(
             response,
-            '<input id="id_include_descendants" name="include_descendants" type="checkbox">',
+            'name="include_descendants"',
         )
 
 
-class TestBulkPublishIncludingDescendants(TestCase, WagtailTestUtils):
+class TestBulkPublishIncludingDescendants(WagtailTestUtils, TestCase):
     def setUp(self):
         self.root_page = Page.objects.get(id=2)
 
@@ -314,12 +315,12 @@ class TestBulkPublishIncludingDescendants(TestCase, WagtailTestUtils):
 
     def test_publish_descendants_view(self):
         """
-        This tests that the publish view responds with an publish confirm page that contains the form field 'include_descendants'
+        This tests that the publish view responds with a publish confirm page that contains the form field 'include_descendants'
         """
         # Get publish page
         response = self.client.get(self.url)
 
-        # Check that the user received an publish confirm page
+        # Check that the user received a publish confirm page
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(
             response, "wagtailadmin/pages/bulk_actions/confirm_bulk_publish.html"
@@ -327,7 +328,7 @@ class TestBulkPublishIncludingDescendants(TestCase, WagtailTestUtils):
         # Check the form contains the checkbox field include_descendants
         self.assertContains(
             response,
-            '<input type="checkbox" name="include_descendants" id="id_include_descendants">',
+            'name="include_descendants"',
         )
 
     def test_publish_include_children_view_post(self):
