@@ -1,5 +1,4 @@
 import datetime
-import unittest
 from unittest import mock
 
 from django.contrib.auth.models import Group, Permission
@@ -10,8 +9,13 @@ from django.urls import reverse
 from django.utils import timezone
 from django.utils.translation import gettext_lazy as _
 
-from wagtail.admin.tests.pages.timestamps import submittable_timestamp
-from wagtail.models import GroupPagePermission, Locale, Page, Revision
+from wagtail.models import (
+    GroupPagePermission,
+    Locale,
+    Page,
+    PageViewRestriction,
+    Revision,
+)
 from wagtail.signals import page_published
 from wagtail.test.testapp.models import (
     BusinessChild,
@@ -28,9 +32,15 @@ from wagtail.test.testapp.models import (
     StandardIndex,
 )
 from wagtail.test.utils import WagtailTestUtils
+from wagtail.test.utils.timestamps import submittable_timestamp
 
 
-class TestPageCreation(TestCase, WagtailTestUtils):
+class TestPageCreation(WagtailTestUtils, TestCase):
+    STATUS_TOGGLE_BADGE_REGEX = (
+        r'data-side-panel-toggle="status"[^<]+<svg[^<]+<use[^<]+</use[^<]+</svg[^<]+'
+        r"<div data-side-panel-toggle-counter[^>]+w-bg-critical-200[^>]+>\s*%(num_errors)s\s*</div>"
+    )
+
     def setUp(self):
         # Find root page
         self.root_page = Page.objects.get(id=2)
@@ -417,6 +427,26 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         )
         self.assertRedirects(response, "/admin/")
 
+    def test_create_page_defined_before_admin_load(self):
+        """
+        Test that a page model defined before wagtail.admin is loaded has all fields present
+        """
+        response = self.client.get(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=("earlypage", "earlypage", self.root_page.id),
+            )
+        )
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, "wagtailadmin/pages/create.html")
+        # Title field should be present and have TitleFieldPanel behaviour
+        # including syncing with slug
+        self.assertContains(response, 'data-w-sync-target-value="#id_slug"')
+        # SEO title should be present in promote tab
+        self.assertContains(
+            response, "The name of the page displayed on search engine results"
+        )
+
     def test_create_simplepage_post(self):
         post_data = {
             "title": "New page!",
@@ -449,7 +479,7 @@ class TestPageCreation(TestCase, WagtailTestUtils):
 
         # treebeard should report no consistency problems with the tree
         self.assertFalse(
-            any(Page.find_problems()), "treebeard found consistency problems"
+            any(Page.find_problems()), msg="treebeard found consistency problems"
         )
 
     def test_create_simplepage_scheduled(self):
@@ -480,7 +510,7 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         self.assertEqual(page.go_live_at.date(), go_live_at.date())
         self.assertEqual(page.expire_at.date(), expire_at.date())
         self.assertIs(page.expired, False)
-        self.assertTrue(page.status_string, "draft")
+        self.assertEqual(page.status_string, "draft")
 
         # No revisions with approved_go_live_at
         self.assertFalse(
@@ -513,20 +543,32 @@ class TestPageCreation(TestCase, WagtailTestUtils):
 
         # Check that a form error was raised
         self.assertFormError(
-            response,
-            "form",
+            response.context["form"],
             "go_live_at",
             "Go live date/time must be before expiry date/time",
         )
         self.assertFormError(
-            response,
-            "form",
+            response.context["form"],
             "expire_at",
             "Go live date/time must be before expiry date/time",
         )
 
+        self.assertContains(
+            response,
+            '<div class="w-label-3 w-text-primary">Invalid schedule</div>',
+            html=True,
+        )
+
+        num_errors = 2
+
+        # Should show the correct number on the badge of the toggle button
+        self.assertRegex(
+            response.content.decode(),
+            self.STATUS_TOGGLE_BADGE_REGEX % {"num_errors": num_errors},
+        )
+
         # form should be marked as having unsaved changes for the purposes of the dirty-forms warning
-        self.assertContains(response, "alwaysDirty: true")
+        self.assertContains(response, 'data-w-unsaved-force-value="true"')
 
     def test_create_simplepage_scheduled_expire_in_the_past(self):
         post_data = {
@@ -549,60 +591,79 @@ class TestPageCreation(TestCase, WagtailTestUtils):
 
         # Check that a form error was raised
         self.assertFormError(
-            response, "form", "expire_at", "Expiry date/time must be in the future"
+            response.context["form"],
+            "expire_at",
+            "Expiry date/time must be in the future",
+        )
+
+        self.assertContains(
+            response,
+            '<div class="w-label-3 w-text-primary">Invalid schedule</div>',
+            html=True,
+        )
+
+        num_errors = 1
+
+        # Should show the correct number on the badge of the toggle button
+        self.assertRegex(
+            response.content.decode(),
+            self.STATUS_TOGGLE_BADGE_REGEX % {"num_errors": num_errors},
         )
 
         # form should be marked as having unsaved changes for the purposes of the dirty-forms warning
-        self.assertContains(response, "alwaysDirty: true")
+        self.assertContains(response, 'data-w-unsaved-force-value="true"')
 
     def test_create_simplepage_post_publish(self):
         # Connect a mock signal handler to page_published signal
         mock_handler = mock.MagicMock()
         page_published.connect(mock_handler)
 
-        # Post
-        post_data = {
-            "title": "New page!",
-            "content": "Some content",
-            "slug": "hello-world",
-            "action-publish": "Publish",
-        }
-        response = self.client.post(
-            reverse(
-                "wagtailadmin_pages:add",
-                args=("tests", "simplepage", self.root_page.id),
-            ),
-            post_data,
-        )
+        try:
+            # Post
+            post_data = {
+                "title": "New page!",
+                "content": "Some content",
+                "slug": "hello-world",
+                "action-publish": "Publish",
+            }
+            response = self.client.post(
+                reverse(
+                    "wagtailadmin_pages:add",
+                    args=("tests", "simplepage", self.root_page.id),
+                ),
+                post_data,
+            )
 
-        # Find the page and check it
-        page = Page.objects.get(
-            path__startswith=self.root_page.path, slug="hello-world"
-        ).specific
+            # Find the page and check it
+            page = Page.objects.get(
+                path__startswith=self.root_page.path, slug="hello-world"
+            ).specific
 
-        # Should be redirected to explorer
-        self.assertRedirects(
-            response, reverse("wagtailadmin_explore", args=(self.root_page.id,))
-        )
+            # Should be redirected to explorer
+            self.assertRedirects(
+                response, reverse("wagtailadmin_explore", args=(self.root_page.id,))
+            )
 
-        self.assertEqual(page.title, post_data["title"])
-        self.assertEqual(page.draft_title, post_data["title"])
-        self.assertIsInstance(page, SimplePage)
-        self.assertTrue(page.live)
-        self.assertTrue(page.first_published_at)
+            self.assertEqual(page.title, post_data["title"])
+            self.assertEqual(page.draft_title, post_data["title"])
+            self.assertIsInstance(page, SimplePage)
+            self.assertTrue(page.live)
+            self.assertTrue(page.first_published_at)
 
-        # Check that the page_published signal was fired
-        self.assertEqual(mock_handler.call_count, 1)
-        mock_call = mock_handler.mock_calls[0][2]
+            # Check that the page_published signal was fired
+            self.assertEqual(mock_handler.call_count, 1)
+            mock_call = mock_handler.mock_calls[0][2]
 
-        self.assertEqual(mock_call["sender"], page.specific_class)
-        self.assertEqual(mock_call["instance"], page)
-        self.assertIsInstance(mock_call["instance"], page.specific_class)
+            self.assertEqual(mock_call["sender"], page.specific_class)
+            self.assertEqual(mock_call["instance"], page)
+            self.assertIsInstance(mock_call["instance"], page.specific_class)
 
-        # treebeard should report no consistency problems with the tree
-        self.assertFalse(
-            any(Page.find_problems()), "treebeard found consistency problems"
-        )
+            # treebeard should report no consistency problems with the tree
+            self.assertFalse(
+                any(Page.find_problems()), msg="treebeard found consistency problems"
+            )
+        finally:
+            page_published.disconnect(mock_handler)
 
     def test_create_simplepage_post_publish_scheduled(self):
         go_live_at = timezone.now() + datetime.timedelta(days=1)
@@ -643,7 +704,7 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         # But Page won't be live
         self.assertFalse(page.live)
         self.assertFalse(page.first_published_at)
-        self.assertTrue(page.status_string, "scheduled")
+        self.assertEqual(page.status_string, "scheduled")
 
     def test_create_simplepage_post_submit(self):
         # Create a moderator user for testing email
@@ -713,10 +774,14 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         self.assertEqual(response.status_code, 200)
 
         # Check that a form error was raised
-        self.assertFormError(response, "form", "slug", "This slug is already in use")
+        self.assertFormError(
+            response.context["form"],
+            "slug",
+            "The slug 'hello-world' is already in use within the parent page",
+        )
 
         # form should be marked as having unsaved changes for the purposes of the dirty-forms warning
-        self.assertContains(response, "alwaysDirty: true")
+        self.assertContains(response, 'data-w-unsaved-force-value="true"')
 
     def test_create_nonexistantparent(self):
         response = self.client.get(
@@ -733,6 +798,47 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         )
         self.assertEqual(response.status_code, 404)
 
+    def test_custom_validation(self):
+        response = self.client.post(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=("tests", "validatedpage", self.root_page.id),
+            ),
+            {
+                "title": "New page!",
+                "foo": "not bar",
+                "slug": "hello-world",
+            },
+        )
+
+        self.assertEqual(response.status_code, 200)
+        self.assertFormError(response.context["form"], "foo", "Field foo must be bar")
+        self.assertFalse(
+            Page.objects.filter(
+                path__startswith=self.root_page.path, slug="hello-world"
+            ).exists()
+        )
+
+        response = self.client.post(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=("tests", "validatedpage", self.root_page.id),
+            ),
+            {
+                "title": "New page!",
+                "foo": "superbar",
+                "slug": "hello-world",
+            },
+        )
+
+        page = Page.objects.get(
+            path__startswith=self.root_page.path, slug="hello-world"
+        )
+        # Should be redirected to edit page
+        self.assertRedirects(
+            response, reverse("wagtailadmin_pages:edit", args=(page.id,))
+        )
+
     def test_preview_on_create(self):
         post_data = {
             "title": "New page!",
@@ -765,6 +871,55 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         self.assertTrue(response.context["self"].path.startswith(self.root_page.path))
         self.assertEqual(response.context["self"].get_parent(), self.root_page)
 
+        # Should not show edit link in the userbar
+        self.assertNotContains(response, "Edit this page")
+
+    def test_preview_with_custom_validation(self):
+        post_data = {
+            "title": "New page!",
+            "foo": "not bar",
+            "slug": "hello-world",
+            "action-submit": "Submit",
+        }
+        preview_url = reverse(
+            "wagtailadmin_pages:preview_on_add",
+            args=("tests", "validatedpage", self.root_page.id),
+        )
+        response = self.client.post(preview_url, post_data)
+
+        # Check the JSON response
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(
+            response.content.decode(),
+            {"is_valid": False, "is_available": False},
+        )
+
+        post_data = {
+            "title": "New page!",
+            "foo": "superbar",
+            "slug": "hello-world",
+            "action-submit": "Submit",
+        }
+        preview_url = reverse(
+            "wagtailadmin_pages:preview_on_add",
+            args=("tests", "validatedpage", self.root_page.id),
+        )
+        response = self.client.post(preview_url, post_data)
+
+        # Check the JSON response
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(
+            response.content.decode(),
+            {"is_valid": True, "is_available": True},
+        )
+
+        response = self.client.get(preview_url)
+
+        # Check the HTML response
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, "tests/validated_page.html")
+        self.assertContains(response, "foo = superbar")
+
     def test_whitespace_titles(self):
         post_data = {
             "title": " ",  # Single space on purpose
@@ -781,7 +936,9 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         )
 
         # Check that a form error was raised
-        self.assertFormError(response, "form", "title", "This field is required.")
+        self.assertFormError(
+            response.context["form"], "title", "This field is required."
+        )
 
     def test_whitespace_titles_with_tab(self):
         post_data = {
@@ -799,7 +956,9 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         )
 
         # Check that a form error was raised
-        self.assertFormError(response, "form", "title", "This field is required.")
+        self.assertFormError(
+            response.context["form"], "title", "This field is required."
+        )
 
     def test_whitespace_titles_with_tab_in_seo_title(self):
         post_data = {
@@ -870,12 +1029,44 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         # Check that a form error was raised
         self.assertEqual(response.status_code, 200)
         self.assertFormError(
-            response,
-            "form",
+            response.context["form"],
             "slug",
             "Ensure this value has at most 255 characters (it has 287).",
         )
 
+    def test_title_field_attrs(self):
+        """
+        Should correctly add the sync field and placeholder attributes to the title field.
+        Note: Many test Page models use a FieldPanel for 'title', StandardChild does not
+        override content_panels (uses the default).
+        """
+
+        response = self.client.get(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=("tests", "standardchild", self.root_page.id),
+            )
+        )
+
+        html = self.get_soup(response.content)
+
+        actual_attrs = html.find("input", {"name": "title"}).attrs
+
+        expected_attrs = {
+            "aria-describedby": "panel-child-content-child-title-helptext",
+            "data-action": "focus->w-sync#check blur->w-sync#apply change->w-sync#apply keyup->w-sync#apply",
+            "data-controller": "w-sync",
+            "data-w-sync-target-value": "#id_slug",
+            "id": "id_title",
+            "maxlength": "255",
+            "name": "title",
+            "placeholder": "Page title*",
+            "required": "",
+            "type": "text",
+        }
+
+        self.assertEqual(actual_attrs, expected_attrs)
+
     def test_before_create_page_hook(self):
         def hook_func(request, parent_page, page_class):
             self.assertIsInstance(request, HttpRequest)
@@ -1064,10 +1255,10 @@ class TestPageCreation(TestCase, WagtailTestUtils):
             "Submit for moderation</button>",
         )
 
-    @override_settings(WAGTAIL_MODERATION_ENABLED=False)
+    @override_settings(WAGTAIL_WORKFLOW_ENABLED=False)
     def test_hide_moderation_button(self):
         """
-        Tests that if WAGTAIL_MODERATION_ENABLED is set to False, the "Submit for Moderation" button is not shown.
+        Tests that if WAGTAIL_WORKFLOW_ENABLED is set to False, the "Submit for Moderation" button is not shown.
         """
         response = self.client.get(
             reverse(
@@ -1101,7 +1292,7 @@ class TestPageCreation(TestCase, WagtailTestUtils):
         self.assertEqual(response.context["page"].locale, fr_locale)
 
 
-class TestPermissionedFieldPanels(TestCase, WagtailTestUtils):
+class TestPermissionedFieldPanels(WagtailTestUtils, TestCase):
     fixtures = ["test.json"]
 
     def setUp(self):
@@ -1142,7 +1333,7 @@ class TestPermissionedFieldPanels(TestCase, WagtailTestUtils):
         self.assertContains(response, '"secret_data"')
 
 
-class TestSubpageBusinessRules(TestCase, WagtailTestUtils):
+class TestSubpageBusinessRules(WagtailTestUtils, TestCase):
     def setUp(self):
         # Find root page
         self.root_page = Page.objects.get(id=2)
@@ -1284,7 +1475,7 @@ class TestSubpageBusinessRules(TestCase, WagtailTestUtils):
         )
 
 
-class TestInlinePanelMedia(TestCase, WagtailTestUtils):
+class TestInlinePanelMedia(WagtailTestUtils, TestCase):
     """
     Test that form media required by InlinePanels is correctly pulled in to the edit page
     """
@@ -1311,7 +1502,7 @@ class TestInlinePanelMedia(TestCase, WagtailTestUtils):
         self.assertContains(response, "wagtailadmin/js/draftail.js")
 
 
-class TestInlineStreamField(TestCase, WagtailTestUtils):
+class TestInlineStreamField(WagtailTestUtils, TestCase):
     """
     Test that streamfields inside an inline child work
     """
@@ -1329,10 +1520,16 @@ class TestInlineStreamField(TestCase, WagtailTestUtils):
         self.assertEqual(response.status_code, 200)
 
         # response should include HTML declarations for streamfield child blocks
-        self.assertContains(response, '<div id="sections-__prefix__-body" data-block="')
+        self.assertContains(response, '<div id="sections-__prefix__-body" data-block')
+        soup = self.get_soup(response.content)
+        blockDiv = soup.find("div", {"data-controller": "w-block"})
+        self.assertIsNotNone(blockDiv)
+        # block div should contain this attributes
+        self.assertTrue(blockDiv.has_attr("data-w-block-arguments-value"))
+        self.assertTrue(blockDiv.has_attr("data-w-block-data-value"))
 
 
-class TestIssue2994(TestCase, WagtailTestUtils):
+class TestIssue2994(WagtailTestUtils, TestCase):
     """
     In contrast to most "standard" form fields, StreamField form widgets generally won't
     provide a postdata field with a name exactly matching the field name. To prevent Django
@@ -1368,7 +1565,7 @@ class TestIssue2994(TestCase, WagtailTestUtils):
         self.assertEqual("hello world", new_page.body[0].value)
 
 
-class TestInlinePanelWithTags(TestCase, WagtailTestUtils):
+class TestInlinePanelWithTags(WagtailTestUtils, TestCase):
     # https://github.com/wagtail/wagtail/issues/5414#issuecomment-567080707
 
     def setUp(self):
@@ -1392,6 +1589,10 @@ class TestInlinePanelWithTags(TestCase, WagtailTestUtils):
             "comments-INITIAL_FORMS": 0,
             "comments-MIN_NUM_FORMS": 0,
             "comments-MAX_NUM_FORMS": 1000,
+            "social_links-TOTAL_FORMS": 0,
+            "social_links-INITIAL_FORMS": 0,
+            "social_links-MIN_NUM_FORMS": 0,
+            "social_links-MAX_NUM_FORMS": 1000,
         }
         response = self.client.post(
             reverse(
@@ -1407,7 +1608,50 @@ class TestInlinePanelWithTags(TestCase, WagtailTestUtils):
         self.assertEqual(new_page.addresses.first().tags.count(), 2)
 
 
-class TestInlinePanelNonFieldErrors(TestCase, WagtailTestUtils):
+class TestNonOrderableInlinePanel(WagtailTestUtils, TestCase):
+    # https://github.com/wagtail/wagtail/issues/11887
+
+    def setUp(self):
+        self.root_page = Page.objects.get(id=2)
+        self.user = self.login()
+
+    def test_create(self):
+        post_data = {
+            "title": "Mr Benn",
+            "slug": "mr-benn",
+            "first_name": "William",
+            "last_name": "Benn",
+            "addresses-TOTAL_FORMS": 0,
+            "addresses-INITIAL_FORMS": 0,
+            "addresses-MIN_NUM_FORMS": 0,
+            "addresses-MAX_NUM_FORMS": 1000,
+            "action-publish": "Publish",
+            "comments-TOTAL_FORMS": 0,
+            "comments-INITIAL_FORMS": 0,
+            "comments-MIN_NUM_FORMS": 0,
+            "comments-MAX_NUM_FORMS": 1000,
+            "social_links-TOTAL_FORMS": 1,
+            "social_links-INITIAL_FORMS": 0,
+            "social_links-MIN_NUM_FORMS": 0,
+            "social_links-MAX_NUM_FORMS": 1000,
+            "social_links-0-url": "https://twitter.com/mrbenn",
+            "social_links-0-kind": "twitter",
+        }
+        response = self.client.post(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=("tests", "personpage", self.root_page.id),
+            ),
+            post_data,
+        )
+        self.assertRedirects(
+            response, reverse("wagtailadmin_explore", args=(self.root_page.id,))
+        )
+        new_page = PersonPage.objects.get(slug="mr-benn")
+        self.assertEqual(new_page.social_links.count(), 1)
+
+
+class TestInlinePanelNonFieldErrors(WagtailTestUtils, TestCase):
     """
     Test that non field errors will render for InlinePanels
     https://github.com/wagtail/wagtail/issues/3890
@@ -1461,7 +1705,7 @@ class TestInlinePanelNonFieldErrors(TestCase, WagtailTestUtils):
 
 
 @override_settings(WAGTAIL_I18N_ENABLED=True)
-class TestLocaleSelector(TestCase, WagtailTestUtils):
+class TestLocaleSelector(WagtailTestUtils, TestCase):
     fixtures = ["test.json"]
 
     def setUp(self):
@@ -1472,7 +1716,6 @@ class TestLocaleSelector(TestCase, WagtailTestUtils):
         )
         self.user = self.login()
 
-    @unittest.expectedFailure  # TODO: Page editor header rewrite
     def test_locale_selector(self):
         response = self.client.get(
             reverse(
@@ -1541,7 +1784,7 @@ class TestLocaleSelector(TestCase, WagtailTestUtils):
 
 
 @override_settings(WAGTAIL_I18N_ENABLED=True)
-class TestLocaleSelectorOnRootPage(TestCase, WagtailTestUtils):
+class TestLocaleSelectorOnRootPage(WagtailTestUtils, TestCase):
     fixtures = ["test.json"]
 
     def setUp(self):
@@ -1549,7 +1792,6 @@ class TestLocaleSelectorOnRootPage(TestCase, WagtailTestUtils):
         self.fr_locale = Locale.objects.create(language_code="fr")
         self.user = self.login()
 
-    @unittest.expectedFailure  # TODO: Page editor header rewrite
     def test_locale_selector(self):
         response = self.client.get(
             reverse(
@@ -1560,6 +1802,7 @@ class TestLocaleSelectorOnRootPage(TestCase, WagtailTestUtils):
 
         self.assertContains(response, 'id="status-sidebar-english"')
 
+        # Should show a link to switch to another locale
         add_translation_url = (
             reverse(
                 "wagtailadmin_pages:add",
@@ -1569,6 +1812,50 @@ class TestLocaleSelectorOnRootPage(TestCase, WagtailTestUtils):
         )
         self.assertContains(response, f'href="{add_translation_url}"')
 
+        # Should not show a link to switch to the current locale
+        self_translation_url = (
+            reverse(
+                "wagtailadmin_pages:add",
+                args=["demosite", "homepage", self.root_page.id],
+            )
+            + "?locale=en"
+        )
+        self.assertNotContains(response, f'href="{self_translation_url}"')
+
+    def test_locale_selector_selected(self):
+        response = self.client.get(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=["demosite", "homepage", self.root_page.id],
+            )
+            + "?locale=fr"
+        )
+
+        self.assertContains(response, 'id="status-sidebar-french"')
+
+        # Should render the locale input with the currently selected locale
+        self.assertContains(response, '<input type="hidden" name="locale" value="fr">')
+
+        # Should show a link to switch to another locale
+        add_translation_url = (
+            reverse(
+                "wagtailadmin_pages:add",
+                args=["demosite", "homepage", self.root_page.id],
+            )
+            + "?locale=en"
+        )
+        self.assertContains(response, f'href="{add_translation_url}"')
+
+        # Should not show a link to switch to the current locale
+        self_translation_url = (
+            reverse(
+                "wagtailadmin_pages:add",
+                args=["demosite", "homepage", self.root_page.id],
+            )
+            + "?locale=fr"
+        )
+        self.assertNotContains(response, f'href="{self_translation_url}"')
+
     @override_settings(WAGTAIL_I18N_ENABLED=False)
     def test_locale_selector_not_present_when_i18n_disabled(self):
         response = self.client.get(
@@ -1590,7 +1877,7 @@ class TestLocaleSelectorOnRootPage(TestCase, WagtailTestUtils):
         self.assertNotContains(response, f'href="{add_translation_url}"')
 
 
-class TestPageSubscriptionSettings(TestCase, WagtailTestUtils):
+class TestPageSubscriptionSettings(WagtailTestUtils, TestCase):
     def setUp(self):
         # Find root page
         self.root_page = Page.objects.get(id=2)
@@ -1598,7 +1885,7 @@ class TestPageSubscriptionSettings(TestCase, WagtailTestUtils):
         # Login
         self.user = self.login()
 
-    def test_commment_notifications_switched_on_by_default(self):
+    def test_comment_notifications_switched_on_by_default(self):
         response = self.client.get(
             reverse(
                 "wagtailadmin_pages:add",
@@ -1668,3 +1955,132 @@ class TestPageSubscriptionSettings(TestCase, WagtailTestUtils):
 
         self.assertEqual(subscription.user, self.user)
         self.assertFalse(subscription.comment_notifications)
+
+
+class TestCommenting(WagtailTestUtils, TestCase):
+    """
+    Tests the commenting related logic of the create page view.
+    """
+
+    def setUp(self):
+        # Find root page
+        self.root_page = Page.objects.get(id=2)
+
+        # Login
+        self.user = self.login()
+
+    def test_comments_enabled_by_default(self):
+        response = self.client.get(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=["tests", "simplepage", self.root_page.id],
+            )
+        )
+
+        self.assertEqual(response.status_code, 200)
+
+        soup = self.get_soup(response.content)
+        form = soup.select_one("[data-edit-form]")
+        self.assertEqual("page-edit-form", form["id"])
+        self.assertIn("w-init", form["data-controller"])
+        self.assertEqual("w-comments:init", form["data-w-init-event-value"])
+
+    @override_settings(WAGTAILADMIN_COMMENTS_ENABLED=False)
+    def test_comments_disabled(self):
+        response = self.client.get(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=["tests", "simplepage", self.root_page.id],
+            )
+        )
+
+        self.assertEqual(response.status_code, 200)
+
+        soup = self.get_soup(response.content)
+        form = soup.select_one("[data-edit-form]")
+        self.assertEqual("page-edit-form", form["id"])
+        self.assertIn("w-init", form["data-controller"])
+        self.assertEqual("", form["data-w-init-event-value"])
+
+
+class TestCreateViewChildPagePrivacy(WagtailTestUtils, TestCase):
+    def setUp(self):
+        self.login()
+
+        self.homepage = Page.objects.get(id=2)
+
+        self.private_parent_page = self.homepage.add_child(
+            instance=SimplePage(
+                title="Private Parent page",
+                content="hello",
+                live=True,
+            )
+        )
+
+        PageViewRestriction.objects.create(
+            page=self.private_parent_page,
+            restriction_type="password",
+            password="password123",
+        )
+
+        self.private_child_page = self.private_parent_page.add_child(
+            instance=SimplePage(
+                title="child page",
+                content="hello",
+                live=True,
+            )
+        )
+
+        self.public_parent_page = self.homepage.add_child(
+            instance=SimplePage(
+                title="Public Parent page",
+                content="hello",
+                live=True,
+            )
+        )
+
+        self.public_child_page = self.public_parent_page.add_child(
+            instance=SimplePage(
+                title="public page",
+                content="hello",
+                live=True,
+            )
+        )
+
+    def test_sidebar_private(self):
+        response = self.client.get(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=("tests", "simplepage", self.private_child_page.id),
+            )
+        )
+
+        self.assertEqual(response.status_code, 200)
+
+        soup = self.get_soup(response.content)
+
+        public_div = soup.select_one('[data-w-zone-switch-key-value="isPublic"]')
+        private_div = soup.select_one('[data-w-zone-switch-key-value="!isPublic"]')
+
+        self.assertEqual(private_div["class"], ["!w-my-0"])
+
+        self.assertEqual(public_div["class"], ["w-hidden"])
+
+    def test_sidebar_public(self):
+        response = self.client.get(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=("tests", "simplepage", self.public_child_page.id),
+            )
+        )
+
+        self.assertEqual(response.status_code, 200)
+
+        soup = self.get_soup(response.content)
+
+        public_div = soup.select_one('[data-w-zone-switch-key-value="isPublic"]')
+        private_div = soup.select_one('[data-w-zone-switch-key-value="!isPublic"]')
+
+        self.assertEqual(public_div["class"], [])
+
+        self.assertEqual(private_div["class"], ["!w-my-0", "w-hidden"])
