@@ -1,17 +1,20 @@
 import unittest
 
-from bs4 import BeautifulSoup
 from django.conf import settings
 from django.test import SimpleTestCase, TestCase
 from django.test.utils import override_settings
-from django.urls import reverse
+from django.urls import reverse, reverse_lazy
 
 from wagtail.admin.rich_text import DraftailRichTextArea, get_rich_text_editor_widget
-from wagtail.admin.rich_text.converters.editor_html import PageLinkHandler
+from wagtail.admin.rich_text.converters.editor_html import (
+    EditorHTMLConverter,
+    PageLinkHandler,
+)
 from wagtail.admin.rich_text.editors.draftail.features import Feature
 from wagtail.blocks import RichTextBlock
 from wagtail.models import Page, get_page_models
 from wagtail.rich_text import RichText
+from wagtail.rich_text.feature_registry import FeatureRegistry
 from wagtail.test.testapp.models import SingleEventPage
 from wagtail.test.testapp.rich_text import CustomRichTextArea, LegacyRichTextArea
 from wagtail.test.utils import WagtailTestUtils
@@ -84,7 +87,7 @@ class TestGetRichTextEditorWidget(TestCase):
         self.assertIsInstance(get_rich_text_editor_widget("custom"), CustomRichTextArea)
 
 
-class TestDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTestUtils):
+class TestDefaultRichText(WagtailTestUtils, BaseRichTextEditHandlerTestCase):
     def setUp(self):
         super().setUp()
         # Find root page
@@ -109,7 +112,9 @@ class TestDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTestUtils):
         self.assertEqual(response.status_code, 200)
 
         # Check that draftail (default editor) initialisation is applied
-        self.assertContains(response, "window.draftail.initEditor('#id_body',")
+        # Check that data-controller and data-w-init-event-value were added after initialization
+        self.assertContains(response, 'data-controller="w-init"')
+        self.assertContains(response, 'data-w-init-event-value="w-draftail:init"')
 
         # check that media for draftail is being imported
         self.assertContains(response, "wagtailadmin/js/draftail.js")
@@ -156,7 +161,7 @@ class TestDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTestUtils):
         },
     }
 )
-class TestDraftailFeatureMedia(BaseRichTextEditHandlerTestCase, WagtailTestUtils):
+class TestDraftailFeatureMedia(WagtailTestUtils, BaseRichTextEditHandlerTestCase):
     """
     Features that define additional js/css imports (blockquote, in this case) should
     have those loaded on the page
@@ -199,7 +204,7 @@ class TestDraftailFeatureMedia(BaseRichTextEditHandlerTestCase, WagtailTestUtils
         "default": {"WIDGET": "wagtail.test.testapp.rich_text.LegacyRichTextArea"},
     }
 )
-class TestOverriddenDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTestUtils):
+class TestOverriddenDefaultRichText(WagtailTestUtils, BaseRichTextEditHandlerTestCase):
     def setUp(self):
         super().setUp()
 
@@ -220,8 +225,14 @@ class TestOverriddenDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTest
         self.assertEqual(response.status_code, 200)
 
         # Check that custom editor (default editor by now) was replaced with fake editor
-        self.assertNotContains(response, 'customEditorInitScript("id_body");')
-        self.assertContains(response, 'legacyEditorInitScript("id_body");')
+        self.assertNotContains(
+            response,
+            '<template data-controller="custom-editor" data-id="id_body"',
+        )
+        self.assertContains(
+            response,
+            '<template data-controller="legacy-editor" data-id="id_body"',
+        )
 
     @unittest.expectedFailure  # TODO(telepath)
     def test_overridden_default_editor_in_rich_text_block(self):
@@ -236,8 +247,14 @@ class TestOverriddenDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTest
         self.assertEqual(response.status_code, 200)
 
         # Check that custom editor (default editor by now) was replaced with fake editor
-        self.assertNotContains(response, 'customEditorInitScript("__PREFIX__-value");')
-        self.assertContains(response, 'legacyEditorInitScript("__PREFIX__-value");')
+        self.assertNotContains(
+            response,
+            '<template data-controller="custom-editor" data-id="__PREFIX__-value"',
+        )
+        self.assertContains(
+            response,
+            '<template data-controller="legacy-editor" data-id="__PREFIX__-value"',
+        )
 
 
 @override_settings(
@@ -248,7 +265,7 @@ class TestOverriddenDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTest
         "custom": {"WIDGET": "wagtail.test.testapp.rich_text.CustomRichTextArea"},
     }
 )
-class TestCustomDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTestUtils):
+class TestCustomDefaultRichText(WagtailTestUtils, BaseRichTextEditHandlerTestCase):
     def setUp(self):
         super().setUp()
 
@@ -269,8 +286,14 @@ class TestCustomDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTestUtil
         self.assertEqual(response.status_code, 200)
 
         # Check that custom editor (default editor by now) was replaced with fake editor
-        self.assertNotContains(response, 'legacyEditorInitScript("id_body");')
-        self.assertContains(response, 'customEditorInitScript("id_body");')
+        self.assertNotContains(
+            response,
+            '<template data-controller="legacy-editor" data-id="id_body"',
+        )
+        self.assertContains(
+            response,
+            '<template data-controller="custom-editor" data-id="id_body"',
+        )
 
     @unittest.expectedFailure  # TODO(telepath)
     def test_custom_editor_in_rich_text_block(self):
@@ -284,7 +307,9 @@ class TestCustomDefaultRichText(BaseRichTextEditHandlerTestCase, WagtailTestUtil
         # Check status code
         self.assertEqual(response.status_code, 200)
 
-        self.assertContains(response, 'customEditorInitScript("__PREFIX__-value");')
+        self.assertContains(
+            response, '<template data-controller="custom-editor" data-id="id_body"'
+        )
 
 
 class TestRichTextValue(TestCase):
@@ -318,7 +343,7 @@ class TestRichTextValue(TestCase):
         },
     }
 )
-class TestDraftailWithFeatureOptions(BaseRichTextEditHandlerTestCase, WagtailTestUtils):
+class TestDraftailWithFeatureOptions(WagtailTestUtils, BaseRichTextEditHandlerTestCase):
     def setUp(self):
         super().setUp()
 
@@ -334,10 +359,20 @@ class TestDraftailWithFeatureOptions(BaseRichTextEditHandlerTestCase, WagtailTes
                 args=("tests", "defaultrichtextfieldpage", self.root_page.id),
             )
         )
-        self.assertEqual(response.status_code, 200)
-        self.assertContains(response, '"type": "header-two"')
-        self.assertContains(response, '"type": "IMAGE"')
-        self.assertNotContains(response, '"type": "ordered-list-item"')
+        soup = self.get_soup(response.content)
+        input = soup.find(
+            "input",
+            {
+                "data-draftail-input": "",
+                "data-controller": "w-init",
+                "data-w-init-event-value": "w-draftail:init",
+            },
+        )
+        data = input["data-w-init-detail-value"]
+
+        self.assertIn('"type": "header-two"', data)
+        self.assertIn('"type": "IMAGE"', data)
+        self.assertNotIn('"type": "ordered-list-item"', data)
 
     @unittest.expectedFailure  # TODO(telepath)
     def test_features_option_on_rich_text_block(self):
@@ -354,7 +389,7 @@ class TestDraftailWithFeatureOptions(BaseRichTextEditHandlerTestCase, WagtailTes
 
 
 class TestDraftailWithAdditionalFeatures(
-    BaseRichTextEditHandlerTestCase, WagtailTestUtils
+    WagtailTestUtils, BaseRichTextEditHandlerTestCase
 ):
     def setUp(self):
         super().setUp()
@@ -381,16 +416,28 @@ class TestDraftailWithAdditionalFeatures(
 
         self.assertEqual(response.status_code, 200)
         # default ones are there
-        self.assertContains(response, '"type": "header-two"')
-        self.assertContains(response, '"type": "LINK"')
-        self.assertContains(response, '"type": "ITALIC"')
+
+        soup = self.get_soup(response.content)
+        input = soup.find(
+            "input",
+            {
+                "data-draftail-input": "",
+                "data-controller": "w-init",
+                "data-w-init-event-value": "w-draftail:init",
+            },
+        )
+        data = input["data-w-init-detail-value"]
+
+        self.assertIn('"type": "header-two"', data)
+        self.assertIn('"type": "LINK"', data)
+        self.assertIn('"type": "ITALIC"', data)
 
         # not the additional ones.
-        self.assertNotContains(response, '"type": "CODE"')
-        self.assertNotContains(response, '"type": "blockquote"')
-        self.assertNotContains(response, '"type": "SUPERSCRIPT"')
-        self.assertNotContains(response, '"type": "SUBSCRIPT"')
-        self.assertNotContains(response, '"type": "STRIKETHROUGH"')
+        self.assertNotIn('"type": "CODE"', data)
+        self.assertNotIn('"type": "blockquote"', data)
+        self.assertNotIn('"type": "SUPERSCRIPT"', data)
+        self.assertNotIn('"type": "SUBSCRIPT"', data)
+        self.assertNotIn('"type": "STRIKETHROUGH"', data)
 
     @override_settings(
         WAGTAILADMIN_RICH_TEXT_EDITORS={
@@ -418,24 +465,36 @@ class TestDraftailWithAdditionalFeatures(
         )
 
         self.assertEqual(response.status_code, 200)
+
+        soup = self.get_soup(response.content)
+        input = soup.find(
+            "input",
+            {
+                "data-draftail-input": "",
+                "data-controller": "w-init",
+                "data-w-init-event-value": "w-draftail:init",
+            },
+        )
+
+        data = input["data-w-init-detail-value"]
         # Added features are there
-        self.assertContains(response, '"type": "header-two"')
-        self.assertContains(response, '"type": "CODE"')
-        self.assertContains(response, '"type": "blockquote"')
-        self.assertContains(response, '"type": "SUPERSCRIPT"')
-        self.assertContains(response, '"type": "SUBSCRIPT"')
-        self.assertContains(response, '"type": "STRIKETHROUGH"')
+        self.assertIn('"type": "header-two"', data)
+        self.assertIn('"type": "CODE"', data)
+        self.assertIn('"type": "blockquote"', data)
+        self.assertIn('"type": "SUPERSCRIPT"', data)
+        self.assertIn('"type": "SUBSCRIPT"', data)
+        self.assertIn('"type": "STRIKETHROUGH"', data)
 
         # But not the unprovided default ones.
-        self.assertNotContains(response, '"type": "LINK"')
-        self.assertNotContains(response, '"type": "ITALIC"')
+        self.assertNotIn('"type": "LINK"', data)
+        self.assertNotIn('"type": "ITALIC"', data)
 
 
-class TestPageLinkHandler(TestCase):
+class TestPageLinkHandler(WagtailTestUtils, TestCase):
     fixtures = ["test.json"]
 
     def test_get_db_attributes(self):
-        soup = BeautifulSoup('<a data-id="test-id">foo</a>', "html5lib")
+        soup = self.get_soup('<a data-id="test-id">foo</a>')
         tag = soup.a
         result = PageLinkHandler.get_db_attributes(tag)
         self.assertEqual(result, {"id": "test-id"})
@@ -452,6 +511,17 @@ class TestPageLinkHandler(TestCase):
             % events_page_id,
         )
 
+    def test_editorhtmlconverter_from_database_format(self):
+        events_page_id = Page.objects.get(url_path="/home/events/").pk
+        db_html = '<a linktype="page" id="%d">foo</a>' % events_page_id
+        converter = EditorHTMLConverter(features=["link"])
+        editor_html = converter.from_database_format(db_html)
+        self.assertEqual(
+            editor_html,
+            '<a data-linktype="page" data-id="%d" data-parent-id="2" href="/events/">foo</a>'
+            % events_page_id,
+        )
+
 
 class TestWidgetNotHidden(SimpleTestCase):
     def test_draftail(self):
@@ -472,3 +542,97 @@ class TestDraftailFeature(SimpleTestCase):
         media_html = str(feature.media)
         self.assertRegex(media_html, r"feature.js\?v=(\w+)")
         self.assertRegex(media_html, r"feature.css\?v=(\w+)")
+
+
+class TestRichTextChooserUrls(WagtailTestUtils, BaseRichTextEditHandlerTestCase):
+    def setUp(self):
+        super().setUp()
+
+        # Find root page
+        self.root_page = Page.objects.get(id=2)
+
+        self.login()
+
+    @override_settings(
+        WAGTAILADMIN_RICH_TEXT_EDITORS={
+            "default": {
+                "WIDGET": "wagtail.admin.rich_text.DraftailRichTextArea",
+            },
+        }
+    )
+    def test_chooser_urls_exist(self):
+        features = FeatureRegistry()
+        link = features.get_editor_plugin("draftail", "link")
+
+        self.assertIsNotNone(link.data.get("chooserUrls"))
+        self.assertEqual(
+            link.data["chooserUrls"]["pageChooser"],
+            reverse_lazy("wagtailadmin_choose_page"),
+        )
+        self.assertEqual(
+            link.data["chooserUrls"]["externalLinkChooser"],
+            reverse_lazy("wagtailadmin_choose_page_external_link"),
+        )
+        self.assertEqual(
+            link.data["chooserUrls"]["emailLinkChooser"],
+            reverse_lazy("wagtailadmin_choose_page_email_link"),
+        )
+        self.assertEqual(
+            link.data["chooserUrls"]["phoneLinkChooser"],
+            reverse_lazy("wagtailadmin_choose_page_phone_link"),
+        )
+        self.assertEqual(
+            link.data["chooserUrls"]["anchorLinkChooser"],
+            reverse_lazy("wagtailadmin_choose_page_anchor_link"),
+        )
+
+    def test_lazy_chooser_urls_resolved_correctly(self):
+        response = self.client.get(
+            reverse(
+                "wagtailadmin_pages:add",
+                args=("tests", "defaultrichtextfieldpage", self.root_page.id),
+            )
+        )
+
+        soup = self.get_soup(response.content)
+        input = soup.find(
+            "input",
+            {
+                "data-draftail-input": "",
+                "data-controller": "w-init",
+                "data-w-init-event-value": "w-draftail:init",
+            },
+        )
+
+        data = input["data-w-init-detail-value"]
+
+        self.assertIn(
+            '"chooserUrls": {"imageChooser": "/admin/images/chooser/"}',
+            data,
+        )
+        self.assertIn(
+            '"chooserUrls": {"embedsChooser": "/admin/embeds/chooser/"}',
+            data,
+        )
+        self.assertIn(
+            '"chooserUrls": {"documentChooser": "/admin/documents/chooser/"}',
+            data,
+        )
+
+        self.assertIn(
+            '"chooserUrls": {"pageChooser": "/admin/choose-page/", "externalLinkChooser": "/admin/choose-external-link/", "emailLinkChooser": "/admin/choose-email-link/", "phoneLinkChooser": "/admin/choose-phone-link/", "anchorLinkChooser": "/admin/choose-anchor-link/"}',
+            data,
+        )
+
+    def test_lazy_urls_resolution(self):
+        """
+        Check that the lazy URLs have been resolved correctly in the rendered widget HTML data attributes.
+        """
+
+        widget = DraftailRichTextArea()
+        html = widget.render("test_chooserUrls", "", {})
+
+        self.assertIn("/admin/choose-page/", html)
+        self.assertIn("/admin/images/chooser/", html)
+        self.assertIn("/admin/embeds/chooser/", html)
+        self.assertIn("/admin/documents/chooser/", html)
