@@ -1,8 +1,11 @@
 import json
+import re
+from html import unescape
 
 from django import forms
 from django.test import TestCase
 from django.test.utils import override_settings
+from django.utils.html import escape
 
 from wagtail.admin import widgets
 from wagtail.admin.forms.tags import TagField
@@ -12,15 +15,16 @@ from wagtail.test.testapp.models import EventPage, RestaurantTag, SimplePage
 
 
 class TestAdminPageChooserWidget(TestCase):
-    def setUp(self):
-        self.root_page = Page.objects.get(id=2)
+    @classmethod
+    def setUpTestData(cls):
+        cls.root_page = Page.objects.get(id=2)
 
         # Add child page
-        self.child_page = SimplePage(
+        cls.child_page = SimplePage(
             title="foobarbaz",
             content="hello",
         )
-        self.root_page.add_child(instance=self.child_page)
+        cls.root_page.add_child(instance=cls.child_page)
 
     def test_not_hidden(self):
         widget = widgets.AdminPageChooser()
@@ -33,14 +37,15 @@ class TestAdminPageChooserWidget(TestCase):
         self.assertInHTML(
             """<input id="__ID__" name="__NAME__" type="hidden" />""", js_args[0]
         )
-        self.assertIn(">Choose a page<", js_args[0])
+        self.assertIn("Choose a page", js_args[0])
         self.assertEqual(js_args[1], "__ID__")
         self.assertEqual(
             js_args[2],
             {
-                "can_choose_root": False,
-                "model_names": ["wagtailcore.page"],
-                "user_perms": None,
+                "canChooseRoot": False,
+                "modelNames": ["wagtailcore.page"],
+                "userPerms": None,
+                "modalUrl": "/admin/choose-page/",
             },
         )
 
@@ -49,14 +54,14 @@ class TestAdminPageChooserWidget(TestCase):
 
         js_args = widgets.PageChooserAdapter().js_args(widget)
         self.assertEqual(
-            js_args[2]["model_names"], ["tests.simplepage", "tests.eventpage"]
+            js_args[2]["modelNames"], ["tests.simplepage", "tests.eventpage"]
         )
 
     def test_adapt_with_can_choose_root(self):
         widget = widgets.AdminPageChooser(can_choose_root=True)
 
         js_args = widgets.PageChooserAdapter().js_args(widget)
-        self.assertTrue(js_args[2]["can_choose_root"])
+        self.assertTrue(js_args[2]["canChooseRoot"])
 
     def test_render_html(self):
         # render_html is mostly an internal API, but we do want to support calling it with None as
@@ -66,14 +71,14 @@ class TestAdminPageChooserWidget(TestCase):
 
         html = widget.render_html("test", None, {})
         self.assertInHTML("""<input name="test" type="hidden" />""", html)
-        self.assertIn(">Choose a page<", html)
+        self.assertIn("Choose a page", html)
 
     def test_render_js_init(self):
         widget = widgets.AdminPageChooser()
 
         html = widget.render("test", None, {"id": "test-id"})
         self.assertIn(
-            'new PageChooser("test-id", null, {"model_names": ["wagtailcore.page"], "can_choose_root": false, "user_perms": null});',
+            'new PageChooser("test-id", {"modelNames": ["wagtailcore.page"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
             html,
         )
 
@@ -82,7 +87,7 @@ class TestAdminPageChooserWidget(TestCase):
 
         html = widget.render("test", None, {"id": "test-id"})
         self.assertIn(
-            'new PageChooser("test-id", null, {"model_names": ["wagtailcore.page"], "can_choose_root": false, "user_perms": "copy_to"});',
+            'new PageChooser("test-id", {"modelNames": ["wagtailcore.page"], "canChooseRoot": false, "userPerms": "copy_to", "modalUrl": "/admin/choose-page/"});',
             html,
         )
 
@@ -99,7 +104,7 @@ class TestAdminPageChooserWidget(TestCase):
         self.assertInHTML("foobarbaz (simple page)", html)
 
         self.assertIn(
-            'new PageChooser("test-id", %d, {"model_names": ["wagtailcore.page"], "can_choose_root": false, "user_perms": null});'
+            'new PageChooser("test-id", {"modelNames": ["wagtailcore.page"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/", "parentId": %d});'
             % self.root_page.id,
             html,
         )
@@ -109,36 +114,36 @@ class TestAdminPageChooserWidget(TestCase):
 
         html = widget.render("test", None, {"id": "test-id"})
         self.assertIn(
-            'new PageChooser("test-id", null, {"model_names": ["tests.simplepage"], "can_choose_root": false, "user_perms": null});',
+            'new PageChooser("test-id", {"modelNames": ["tests.simplepage"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
             html,
         )
 
         html = widget.render("test", self.child_page, {"id": "test-id"})
-        self.assertIn(">Choose a page (Simple Page)<", html)
+        self.assertIn("Choose a page (Simple page)", html)
 
     def test_render_with_target_model_as_single_instance(self):
         widget = widgets.AdminPageChooser(target_models=SimplePage)
 
         html = widget.render("test", None, {"id": "test-id"})
         self.assertIn(
-            'new PageChooser("test-id", null, {"model_names": ["tests.simplepage"], "can_choose_root": false, "user_perms": null});',
+            'new PageChooser("test-id", {"modelNames": ["tests.simplepage"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
             html,
         )
 
         html = widget.render("test", self.child_page, {"id": "test-id"})
-        self.assertIn(">Choose a page (Simple Page)<", html)
+        self.assertIn("Choose a page (Simple page)", html)
 
     def test_render_with_target_model_as_single_string(self):
         widget = widgets.AdminPageChooser(target_models="tests.SimplePage")
 
         html = widget.render("test", None, {"id": "test-id"})
         self.assertIn(
-            'new PageChooser("test-id", null, {"model_names": ["tests.simplepage"], "can_choose_root": false, "user_perms": null});',
+            'new PageChooser("test-id", {"modelNames": ["tests.simplepage"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
             html,
         )
 
         html = widget.render("test", self.child_page, {"id": "test-id"})
-        self.assertIn(">Choose a page (Simple Page)<", html)
+        self.assertIn("Choose a page (Simple page)", html)
 
     def test_render_with_multiple_target_models(self):
         target_models = [SimplePage, "tests.eventpage"]
@@ -146,23 +151,31 @@ class TestAdminPageChooserWidget(TestCase):
 
         html = widget.render("test", None, {"id": "test-id"})
         self.assertIn(
-            'new PageChooser("test-id", null, {"model_names": ["tests.simplepage", "tests.eventpage"], "can_choose_root": false, "user_perms": null});',
+            'new PageChooser("test-id", {"modelNames": ["tests.simplepage", "tests.eventpage"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
             html,
         )
 
         html = widget.render("test", self.child_page, {"id": "test-id"})
-        self.assertIn(">Choose a page<", html)
+        self.assertIn("Choose a page", html)
 
     def test_render_js_init_with_can_choose_root(self):
         widget = widgets.AdminPageChooser(can_choose_root=True)
 
         html = widget.render("test", self.child_page, {"id": "test-id"})
         self.assertIn(
-            'new PageChooser("test-id", %d, {"model_names": ["wagtailcore.page"], "can_choose_root": true, "user_perms": null});'
+            'new PageChooser("test-id", {"modelNames": ["wagtailcore.page"], "canChooseRoot": true, "userPerms": null, "modalUrl": "/admin/choose-page/", "parentId": %d});'
             % self.root_page.id,
             html,
         )
 
+    def test_get_instance(self):
+        widget = widgets.AdminPageChooser(target_models=[SimplePage])
+        self.assertIsNone(widget.get_instance(None))
+        self.assertIsNone(widget.get_instance(self.root_page.id))
+        self.assertIsNone(widget.get_instance(self.child_page.id + 100))
+        self.assertEqual(widget.get_instance(self.child_page), self.child_page)
+        self.assertEqual(widget.get_instance(self.child_page.id), self.child_page)
+
 
 class TestAdminDateInput(TestCase):
     def test_adapt(self):
@@ -283,7 +296,12 @@ class TestAdminDateTimeInput(TestCase):
 
         self.assertEqual(
             js_args[0],
-            {"dayOfWeekStart": 0, "format": "Y-m-d H:i", "formatTime": "H:i"},
+            {
+                "dayOfWeekStart": 0,
+                "format": "Y-m-d H:i",
+                "formatTime": "H:i",
+                "parentID": "body",
+            },
         )
 
     def test_adapt_with_custom_format(self):
@@ -295,7 +313,29 @@ class TestAdminDateTimeInput(TestCase):
 
         self.assertEqual(
             js_args[0],
-            {"dayOfWeekStart": 0, "format": "d.m.Y. H:i", "formatTime": "H:i A"},
+            {
+                "dayOfWeekStart": 0,
+                "format": "d.m.Y. H:i",
+                "formatTime": "H:i A",
+                "parentID": "body",
+            },
+        )
+
+    def test_adapt_with_custom_parent_selector(self):
+        widget = widgets.AdminDateTimeInput(
+            js_overlay_parent_selector="#test-parent-id"
+        )
+
+        js_args = widgets.AdminDateTimeInputAdapter().js_args(widget)
+
+        self.assertEqual(
+            js_args[0],
+            {
+                "dayOfWeekStart": 0,
+                "format": "Y-m-d H:i",
+                "formatTime": "H:i",
+                "parentID": "#test-parent-id",
+            },
         )
 
     def test_render_js_init(self):
@@ -314,6 +354,7 @@ class TestAdminDateTimeInput(TestCase):
         self.assertIn('"dayOfWeekStart": 0', html)
         self.assertIn('"format": "Y-m-d H:i"', html)
         self.assertIn('"formatTime": "H:i"', html)
+        self.assertIn('"parentID": "body"', html)
 
     def test_render_js_init_with_format(self):
         widget = widgets.AdminDateTimeInput(
@@ -349,30 +390,32 @@ class TestAdminDateTimeInput(TestCase):
 
 class TestAdminTagWidget(TestCase):
     def get_js_init_params(self, html):
-        """Returns a list of the params passed in to initTagField from the supplied HTML"""
-        # example - ["test_id", "/admin/tag-autocomplete/", {'allowSpaces': True}]
-        start = "initTagField("
-        end = ");"
-        items_after_init = html.split(start)[1]
-        if items_after_init:
-            params_raw = items_after_init.split(end)[0]
-            if params_raw:
-                # stuff parameter string into an array so that we can unpack it as JSON
-                return json.loads("[%s]" % params_raw)
-        return []
+        """
+        Returns a list of the key parts of data needed for the w-tag controlled element
+        An id for the element with the 'w-tag' controller, the autocomplete url & tag options
 
-    def get_help_text_html_element(self, html):
-        """Return a help text html element with content as string"""
-        start = """<input type="text" name="tags">"""
-        end = "<script>"
-        items_after_input_tag = html.split(start)[1]
-        if items_after_input_tag:
-            help_text_element = items_after_input_tag.split(end)[0].strip()
-            return help_text_element
-        return []
+        example element <input data-controller="w-tag" id="test_id" data-w-tag-url-value="/admin/tag-autocomplete/" data-w-tag-options-value="{...encoded json opts}" />
+        example result - ["test_id", "/admin/tag-autocomplete/", {'allowSpaces': True}]
+        """
+
+        element_id = re.search(
+            r'data-controller=\"w-tag\" id=\"((?:\\.|[^"\\])*)\"\s+', html
+        ).group(1)
+        autocomplete_url = re.search(
+            r'data-w-tag-url-value=\"((?:\\.|[^"\\])*)"', html
+        ).group(1)
+        options = re.search(
+            r'data-w-tag-options-value=\"((?:\\.|[^"\\])*)"', html
+        ).group(1)
+
+        return [
+            element_id,
+            autocomplete_url,
+            json.loads(unescape(options)),
+        ]
 
     def test_render_js_init_basic(self):
-        """Checks that the 'initTagField' is correctly added to the inline script for tag widgets"""
+        """Checks that the 'w-tag' controller attributes are correctly added to the tag widgets"""
         widget = widgets.AdminTagWidget()
 
         html = widget.render("tags", None, attrs={"id": "alpha"})
@@ -389,7 +432,7 @@ class TestAdminTagWidget(TestCase):
 
     @override_settings(TAG_SPACES_ALLOWED=False)
     def test_render_js_init_no_spaces_allowed(self):
-        """Checks that the 'initTagField' includes the correct value based on TAG_SPACES_ALLOWED in settings"""
+        """Checks that the 'w-tag' controller attributes are correctly added to the tag widgets based  on TAG_SPACES_ALLOWED in settings"""
         widget = widgets.AdminTagWidget()
 
         html = widget.render("tags", None, attrs={"id": "alpha"})
@@ -406,7 +449,8 @@ class TestAdminTagWidget(TestCase):
 
     @override_settings(TAG_LIMIT=5)
     def test_render_js_init_with_tag_limit(self):
-        """Checks that the 'initTagField' includes the correct value based on TAG_LIMIT in settings"""
+        """Checks that the 'w-tag' controller attributes are correctly added to the tag widget using options based on TAG_LIMIT in settings"""
+
         widget = widgets.AdminTagWidget()
 
         html = widget.render("tags", None, attrs={"id": "alpha"})
@@ -423,7 +467,8 @@ class TestAdminTagWidget(TestCase):
 
     def test_render_js_init_with_tag_model(self):
         """
-        Checks that 'initTagField' is passed the correct autocomplete URL for the custom model,
+        Checks that the 'w-tag' controller attributes are correctly added to the tag widget using
+        the correct autocomplete URL for the custom model,
         and sets autocompleteOnly according to that model's free_tagging attribute
         """
         widget = widgets.AdminTagWidget(tag_model=RestaurantTag)
@@ -479,16 +524,15 @@ class TestAdminTagWidget(TestCase):
         help_text = widget.get_context(None, None, {})["widget"]["help_text"]
 
         html = widget.render("tags", None, {})
-        help_text_html_element = self.get_help_text_html_element(html)
 
         self.assertEqual(
             help_text,
             'Multi-word tags with spaces will automatically be enclosed in double quotes (").',
         )
 
-        self.assertHTMLEqual(
-            help_text_html_element,
-            """<p class="help">%s</p>""" % help_text,
+        self.assertIn(
+            """<p class="help">%s</p>""" % escape(help_text),
+            html,
         )
 
     @override_settings(TAG_SPACES_ALLOWED=False)
@@ -498,15 +542,14 @@ class TestAdminTagWidget(TestCase):
         help_text = widget.get_context(None, None, {})["widget"]["help_text"]
 
         html = widget.render("tags", None, {})
-        help_text_html_element = self.get_help_text_html_element(html)
 
         self.assertEqual(
             help_text, "Tags can only consist of a single word, no spaces allowed."
         )
 
-        self.assertHTMLEqual(
-            help_text_html_element,
-            """<p class="help">%s</p>""" % help_text,
+        self.assertIn(
+            """<p class="help">%s</p>""" % escape(help_text),
+            html,
         )
 
 
@@ -613,3 +656,23 @@ class TestFilteredSelect(TestCase):
             </select>
         """,
         )
+
+
+class TestSlugInput(TestCase):
+    def test_has_data_attr(self):
+        widget = widgets.slug.SlugInput()
+
+        html = widget.render("test", None, attrs={"id": "test-id"})
+
+        self.assertInHTML(
+            '<input type="text" name="test" data-controller="w-slug" data-action="blur-&gt;w-slug#slugify w-sync:check-&gt;w-slug#compare w-sync:apply-&gt;w-slug#urlify:prevent" data-w-slug-allow-unicode-value data-w-slug-compare-as-param="urlify" id="test-id">',
+            html,
+        )
+
+    @override_settings(WAGTAIL_ALLOW_UNICODE_SLUGS=False)
+    def test_render_data_atrrs_from_settings(self):
+        widget = widgets.slug.SlugInput()
+
+        html = widget.render("test", None, attrs={"id": "test-id"})
+
+        self.assertNotIn("data-w-slug-allow-unicode-value", html)
