@@ -1,13 +1,14 @@
 import re
-import urllib.parse as urlparse
+from collections import defaultdict
+from urllib.parse import parse_qs, quote, urlencode, urlsplit
 
 from django.conf import settings
 from django.core.paginator import InvalidPage, Paginator
 from django.http import Http404
 from django.shortcuts import get_object_or_404
 from django.template.response import TemplateResponse
+from django.urls import NoReverseMatch
 from django.urls.base import reverse
-from django.utils.http import urlencode
 from django.utils.translation import gettext_lazy as _
 from django.views.generic.base import View
 
@@ -93,7 +94,7 @@ def can_choose_page(
 
             if user_perm == "move_to":
                 return page_to_move.permissions_for_user(user).can_move_to(page)
-    if user_perm == "copy_to":
+    if user_perm in {"add_subpage", "copy_to"}:
         return page.permissions_for_user(user).can_add_subpage()
 
     return True
@@ -286,6 +287,11 @@ class BrowseView(View):
         selected_locale = None
         locale_options = []
         if self.i18n_enabled:
+            # Ensure query parameters (e.g. `page_type`, `user_perms`, etc.) are
+            # preserved when switching locales, but reset the pagination as the
+            # number of pages might be different.
+            new_params = request.GET.copy()
+            new_params.pop("p", None)
             if self.parent_page.is_root():
                 # 'locale' is the current value of the "Locale" selector in the UI
                 if request.GET.get("locale"):
@@ -298,25 +304,20 @@ class BrowseView(View):
 
                 # we are at the Root level, so get the locales from the current pages
                 choose_url = reverse("wagtailadmin_choose_page")
-                locale_options = [
-                    {
-                        "locale": locale,
-                        "url": choose_url
-                        + "?"
-                        + urlencode(
-                            {
-                                "page_type": page_type_string,
-                                "locale": locale.language_code,
-                            }
-                        ),
-                    }
-                    for locale in Locale.objects.filter(
-                        pk__in=pages.values_list("locale_id")
-                    ).exclude(pk=active_locale_id)
-                ]
+                for locale in Locale.objects.filter(
+                    pk__in=pages.values_list("locale_id")
+                ).exclude(pk=active_locale_id):
+                    new_params["locale"] = locale.language_code
+                    locale_options.append(
+                        {
+                            "locale": locale,
+                            "url": choose_url + "?" + new_params.urlencode(),
+                        }
+                    )
             else:
                 # We have a parent page (that is not the root page). Use its locale as the selected localer
                 selected_locale = self.parent_page.locale
+                new_params.pop("locale", None)
                 # and get the locales based on its available translations
                 locales_and_parent_pages = {
                     item["locale"]: item["pk"]
@@ -332,17 +333,14 @@ class BrowseView(View):
                         "wagtailadmin_choose_page_child",
                         args=[locales_and_parent_pages[locale.pk]],
                     )
-
                     locale_options.append(
                         {
                             "locale": locale,
-                            "url": choose_child_url
-                            + "?"
-                            + urlencode({"page_type": page_type_string}),
+                            "url": choose_child_url + "?" + new_params.urlencode(),
                         }
                     )
 
-            # finally, filter the browseable pages on the selected locale
+            # finally, filter the browsable pages on the selected locale
             if selected_locale:
                 pages = pages.filter(locale=selected_locale)
 
@@ -355,7 +353,7 @@ class BrowseView(View):
         except InvalidPage:
             raise Http404
 
-        # Annotate each page with can_choose/can_decend flags
+        # Annotate each page with can_choose/can_descend flags
         for page in pages:
             page.can_choose = can_choose_page(
                 page,
@@ -410,7 +408,11 @@ class SearchView(View):
     @property
     def columns(self):
         cols = [
-            PageTitleColumn("title", label=_("Title")),
+            PageTitleColumn(
+                "title",
+                label=_("Title"),
+                is_multiple_choice=self.is_multiple_choice,
+            ),
             ParentPageColumn("parent", label=_("Parent")),
             DateColumn(
                 "updated",
@@ -633,29 +635,57 @@ class ExternalLinkView(BaseLinkFormView):
             if sites is None:
                 sites = Site.get_site_root_paths()
 
+            try:
+                # The serve view might not be routed to the root path of the domain,
+                # e.g. /pages/, so we need to account for the path to the serve view
+                serve_path = reverse("wagtail_serve", args=("",))
+            except NoReverseMatch:
+                serve_path = None
+
             match_relative_paths = submitted_url.startswith("/") and len(sites) == 1
             # We should only match relative urls if there's only a single site
             # Otherwise this could get very annoying accidentally matching coincidentally
             # named pages on different sites
 
+            possible_sites = defaultdict(list)
+
             if match_relative_paths:
-                possible_sites = [
-                    (pk, url_without_query) for pk, path, url, language_code in sites
-                ]
+                for pk, path, url, language_code in sites:
+                    possible_sites[pk].append(url_without_query)
+
+                    # If the submitted URL is prefixed with the serve path,
+                    # also consider it without the serve path so we can match
+                    # the page using Page.route()
+                    if serve_path and url_without_query.startswith(serve_path):
+                        possible_sites[pk].append(
+                            url_without_query[len(serve_path) - 1 :]
+                        )
             else:
-                possible_sites = [
-                    (pk, url_without_query[len(url) :])
-                    for pk, path, url, language_code in sites
-                    if submitted_url.startswith(url)
-                ]
+                for pk, path, url, language_code in sites:
+                    if not submitted_url.startswith(url):
+                        continue
+                    possible_sites[pk].append(url_without_query[len(url) :])
+
+                    # If the submitted URL is prefixed with the serve path,
+                    # also consider it without the serve path so we can match
+                    # the page using Page.route()
+                    if serve_path and url_without_query.startswith(url + serve_path):
+                        possible_sites[pk].append(
+                            url_without_query[len(url) + len(serve_path) - 1 :]
+                        )
 
             # Loop over possible sites to identify a page match
-            for pk, url in possible_sites:
-                try:
-                    route = Site.objects.get(pk=pk).root_page.specific.route(
-                        request,
-                        [component for component in url.split("/") if component],
-                    )
+            for pk, possible_urls in possible_sites.items():
+                site = Site.objects.select_related("root_page").get(pk=pk)
+                root_page = site.root_page.specific
+                for url in possible_urls:
+                    try:
+                        route = root_page.route(
+                            request,
+                            [component for component in url.split("/") if component],
+                        )
+                    except Http404:
+                        continue
 
                     matched_page = route.page.specific
 
@@ -708,9 +738,6 @@ class ExternalLinkView(BaseLinkFormView):
                             },
                         )
 
-                except Http404:
-                    continue
-
             # Otherwise, with no internal matches, fall back to an external url
             return self.render_chosen_response(result)
         else:  # form invalid
@@ -752,9 +779,9 @@ class EmailLinkView(BaseLinkFormView):
             "subject": self.form.cleaned_data["subject"],
             "body": self.form.cleaned_data["body"],
         }
-        encoded_params = urlparse.urlencode(
+        encoded_params = urlencode(
             {k: v for k, v in params.items() if v is not None and v != ""},
-            quote_via=urlparse.quote,
+            quote_via=quote,
         )
 
         url = "mailto:" + self.form.cleaned_data["email_address"]
@@ -785,11 +812,11 @@ class EmailLinkView(BaseLinkFormView):
     def parse_email_link(self, mailto):
         result = {}
 
-        mail_result = urlparse.urlparse(mailto)
+        mail_result = urlsplit(mailto)
 
         result["email"] = mail_result.path
 
-        query = urlparse.parse_qs(mail_result.query)
+        query = parse_qs(mail_result.query)
         result["subject"] = query["subject"][0] if "subject" in query else ""
         result["body"] = query["body"][0] if "body" in query else ""
 
