@@ -1,6 +1,6 @@
 import django_filters
 from django.contrib.auth import get_user_model
-from django.core.exceptions import PermissionDenied, ValidationError
+from django.core.exceptions import ValidationError
 from django.db.models import OuterRef, Subquery
 from django.utils.translation import gettext_lazy as _
 
@@ -29,10 +29,12 @@ class AgingPagesReportFilterSet(WagtailFilterSet):
 
 
 class AgingPagesView(PageReportView):
-    template_name = "wagtailadmin/reports/aging_pages.html"
-    title = _("Aging pages")
+    results_template_name = "wagtailadmin/reports/aging_pages_results.html"
+    page_title = _("Aging pages")
     header_icon = "time"
     filterset_class = AgingPagesReportFilterSet
+    index_url_name = "wagtailadmin_reports:aging_pages"
+    index_results_url_name = "wagtailadmin_reports:aging_pages_results"
     export_headings = {
         "status_string": _("Status"),
         "last_published_at": _("Last published at"),
@@ -46,6 +48,7 @@ class AgingPagesView(PageReportView):
         "last_published_by_user",
         "content_type",
     ]
+    any_permission_required = ["add", "change", "publish"]
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
@@ -110,10 +113,3 @@ class AgingPagesView(PageReportView):
         )
 
         return super().get_queryset()
-
-    def dispatch(self, request, *args, **kwargs):
-        if not page_permission_policy.user_has_any_permission(
-            request.user, ["add", "change", "publish"]
-        ):
-            raise PermissionDenied
-        return super().dispatch(request, *args, **kwargs)
