@@ -1,6 +1,8 @@
 import collections
+import itertools
 import json
 import re
+import warnings
 from functools import lru_cache
 from importlib import import_module
 
@@ -15,7 +17,10 @@ from django.utils.safestring import mark_safe
 from django.utils.text import capfirst
 
 from wagtail.admin.staticfiles import versioned_static
+from wagtail.coreutils import accepts_kwarg
 from wagtail.telepath import JSContext
+from wagtail.utils.deprecation import RemovedInWagtail70Warning
+from wagtail.utils.templates import template_is_overridden
 
 __all__ = [
     "BaseBlock",
@@ -36,7 +41,7 @@ class BaseBlock(type):
     def __new__(mcs, name, bases, attrs):
         meta_class = attrs.pop("Meta", None)
 
-        cls = super(BaseBlock, mcs).__new__(mcs, name, bases, attrs)
+        cls = super().__new__(mcs, name, bases, attrs)
 
         # Get all the Meta classes from all the bases
         meta_class_bases = [meta_class] + [
@@ -51,8 +56,10 @@ class BaseBlock(type):
 class Block(metaclass=BaseBlock):
     name = ""
     creation_counter = 0
+    definition_registry = {}
 
     TEMPLATE_VAR = "value"
+    DEFAULT_PREVIEW_TEMPLATE = "wagtailcore/shared/block_preview.html"
 
     class Meta:
         label = None
@@ -69,7 +76,7 @@ class Block(metaclass=BaseBlock):
     def __new__(cls, *args, **kwargs):
         # adapted from django.utils.deconstruct.deconstructible; capture the arguments
         # so that we can return them in the 'deconstruct' method
-        obj = super(Block, cls).__new__(cls)
+        obj = super().__new__(cls)
         obj._constructor_args = (args, kwargs)
         return obj
 
@@ -90,9 +97,21 @@ class Block(metaclass=BaseBlock):
         self.creation_counter = Block.creation_counter
         Block.creation_counter += 1
         self.definition_prefix = "blockdef-%d" % self.creation_counter
+        Block.definition_registry[self.definition_prefix] = self
 
         self.label = self.meta.label or ""
 
+    @classmethod
+    def construct_from_lookup(cls, lookup, *args, **kwargs):
+        """
+        See `wagtail.blocks.definition_lookup.BlockDefinitionLookup`.
+        Construct a block instance from the provided arguments, using the given BlockDefinitionLookup
+        object to perform any necessary lookups.
+        """
+        # In the base implementation, no lookups take place - args / kwargs are passed
+        # on to the constructor as-is
+        return cls(*args, **kwargs)
+
     def set_name(self, name):
         self.name = name
         if not self.meta.label:
@@ -139,10 +158,10 @@ class Block(metaclass=BaseBlock):
         Return this block's default value (conventionally found in self.meta.default),
         converted to the value type expected by this block. This caters for the case
         where that value type is not something that can be expressed statically at
-        model definition type (e.g. something like StructValue which incorporates a
-        pointer back to the block definion object).
+        model definition time (e.g. something like StructValue which incorporates a
+        pointer back to the block definition object).
         """
-        return self.meta.default
+        return self.normalize(getattr(self.meta, "default", None))
 
     def clean(self, value):
         """
@@ -155,6 +174,15 @@ class Block(metaclass=BaseBlock):
         """
         return value
 
+    def normalize(self, value):
+        """
+        Given a value for any acceptable type for this block (e.g. string or RichText for a RichTextBlock;
+        dict or StructValue for a StructBlock), return a value of the block's native type (e.g. RichText
+        for RichTextBlock, StructValue for StructBlock). In simple cases this will return the value
+        unchanged.
+        """
+        return value
+
     def to_python(self, value):
         """
         Convert 'value' from a simple (JSON-serialisable) value to a (possibly complex) Python value to be
@@ -163,6 +191,9 @@ class Block(metaclass=BaseBlock):
         like the original value but provides a native HTML rendering when inserted into a template; or it
         might be something totally different (e.g. an image chooser will use the image ID as the clean
         value, and turn this back into an actual image object here).
+
+        For blocks that are usable at the top level of a StreamField, this must also accept any type accepted
+        by normalize. (This is because Django calls `Field.to_python` from `Field.clean`.)
         """
         return value
 
@@ -193,8 +224,9 @@ class Block(metaclass=BaseBlock):
 
     def get_context(self, value, parent_context=None):
         """
-        Return a dict of context variables (derived from the block value and combined with the parent_context)
-        to be used as the template context when rendering this value through a template.
+        Return a dict of context variables (derived from the block ``value`` and combined with the
+        ``parent_context``) to be used as the template context when rendering this value through a
+        template. See :ref:`the usage example <streamfield_get_context>` for more details.
         """
 
         context = parent_context or {}
@@ -206,10 +238,11 @@ class Block(metaclass=BaseBlock):
         )
         return context
 
-    def get_template(self, context=None):
+    def get_template(self, value=None, context=None):
         """
-        Return the template to use for rendering the block if specified on meta class.
-        This extraction was added to make dynamic templates possible if you override this method
+        Return the template to use for rendering the block if specified.
+        This method allows for dynamic templates based on the block instance and a given ``value``.
+        See :ref:`the usage example <streamfield_get_template>` for more details.
         """
         return getattr(self.meta, "template", None)
 
@@ -219,7 +252,16 @@ class Block(metaclass=BaseBlock):
         use a template (with the passed context, supplemented by the result of get_context) if a
         'template' property is specified on the block, and fall back on render_basic otherwise.
         """
-        template = self.get_template(context=context)
+        args = {"context": context}
+        if accepts_kwarg(self.get_template, "value"):
+            args["value"] = value
+        else:
+            warnings.warn(
+                f"{self.__class__.__name__}.get_template should accept a 'value' argument as first argument",
+                RemovedInWagtail70Warning,
+            )
+
+        template = self.get_template(**args)
         if not template:
             return self.render_basic(value, context=context)
 
@@ -230,6 +272,102 @@ class Block(metaclass=BaseBlock):
 
         return mark_safe(render_to_string(template, new_context))
 
+    def get_preview_context(self, value, parent_context=None):
+        """
+        Return a dict of context variables to be used as the template context
+        when rendering the block's preview. The ``value`` argument is the value
+        returned by :meth:`get_preview_value`. The ``parent_context`` argument
+        contains the following variables:
+
+        - ``request``: The current request object.
+        - ``block_def``: The block instance.
+        - ``block_class``: The block class.
+        - ``bound_block``: A ``BoundBlock`` instance representing the block and its value.
+
+        If :ref:`the global preview template <streamfield_global_preview_template>`
+        is used, the block will be rendered as the main content using
+        ``{% include_block %}``, which in turn uses :meth:`get_context`. As a
+        result, the context returned by this method will be available as the
+        ``parent_context`` for ``get_context()`` when the preview is rendered.
+        """
+        # NOTE: see StreamFieldBlockPreview.base_context for the context variables
+        # that can be documented.
+        return parent_context or {}
+
+    def get_preview_template(self, value, context=None):
+        """
+        Return the template to use for rendering the block's preview. The ``value``
+        argument is the value returned by :meth:`get_preview_value`. The ``context``
+        argument contains the variables listed for the ``parent_context`` argument
+        of :meth:`get_preview_context` above (and not the context returned by that
+        method itself).
+
+        Note that the preview template is used to render a complete HTML page of
+        the preview, not just an HTML fragment for the block. The method returns
+        the ``preview_template`` attribute from the block's options if provided,
+        and falls back to
+        :ref:`the global preview template <streamfield_global_preview_template>`
+        otherwise.
+
+        If the global preview template is used, the block will be rendered as the
+        main content using ``{% include_block %}``, which in turn uses
+        :meth:`get_template`.
+        """
+        return (
+            getattr(self.meta, "preview_template", None)
+            or self.DEFAULT_PREVIEW_TEMPLATE
+        )
+
+    def get_preview_value(self):
+        """
+        Return the placeholder value that will be used for rendering the block's
+        preview. By default, the value is the ``preview_value`` from the block's
+        options if provided, otherwise the ``default`` is used as fallback. This
+        method can be overridden to provide a dynamic preview value, such as
+        from the database.
+        """
+        if hasattr(self.meta, "preview_value"):
+            return self.normalize(self.meta.preview_value)
+        return self.get_default()
+
+    @cached_property
+    def _has_default(self):
+        return getattr(self.meta, "default", None) is not None
+
+    @cached_property
+    def is_previewable(self):
+        # To prevent showing a broken preview if the block preview has not been
+        # configured, consider the block to be previewable if either:
+        # - a specific preview template is configured for the block
+        # - a preview value is provided and the global template has been overridden
+        # which are the intended ways to configure block previews.
+        #
+        # If a block is made previewable by other means, the `is_previewable`
+        # property should be overridden to return `True`.
+        has_specific_template = (
+            hasattr(self.meta, "preview_template")
+            or self.__class__.get_preview_template is not Block.get_preview_template
+        )
+        has_preview_value = (
+            hasattr(self.meta, "preview_value")
+            or self._has_default
+            or self.__class__.get_preview_context is not Block.get_preview_context
+            or self.__class__.get_preview_value is not Block.get_preview_value
+        )
+        has_global_template = template_is_overridden(
+            self.DEFAULT_PREVIEW_TEMPLATE,
+            "templates",
+        )
+        return has_specific_template or (has_preview_value and has_global_template)
+
+    def get_description(self):
+        """
+        Return the description of the block to be shown to editors as part of the preview.
+        For :ref:`field block types <field_block_types>`, it will fall back to
+        ``help_text`` if not provided.
+        """
+        return getattr(self.meta, "description", "")
+
     def get_api_representation(self, value, context=None):
         """
         Can be used to customise the API response and defaults to the value returned by get_prep_value.
@@ -249,6 +387,21 @@ class Block(metaclass=BaseBlock):
         """
         return []
 
+    def extract_references(self, value):
+        return []
+
+    def get_block_by_content_path(self, value, path_elements):
+        """
+        Given a list of elements from a content path, retrieve the block at that path
+        as a BoundBlock object, or None if the path does not correspond to a valid block.
+        """
+        # In the base case, where a block has no concept of children, the only valid path is
+        # the empty one (which refers to the current block).
+        if path_elements:
+            return None
+        else:
+            return self.bind(value)
+
     def check(self, **kwargs):
         """
         Hook for the Django system checks framework -
@@ -333,7 +486,11 @@ class Block(metaclass=BaseBlock):
         """
         return False
 
-    def deconstruct(self):
+    @cached_property
+    def canonical_module_path(self):
+        """
+        Return the module path string that should be used to refer to this block in migrations.
+        """
         # adapted from django.utils.deconstruct.deconstructible
         module_name = self.__module__
         name = self.__class__.__name__
@@ -351,16 +508,29 @@ class Block(metaclass=BaseBlock):
         # if the module defines a DECONSTRUCT_ALIASES dictionary, see if the class has an entry in there;
         # if so, use that instead of the real path
         try:
-            path = module.DECONSTRUCT_ALIASES[self.__class__]
+            return module.DECONSTRUCT_ALIASES[self.__class__]
         except (AttributeError, KeyError):
-            path = "%s.%s" % (module_name, name)
+            return f"{module_name}.{name}"
 
+    def deconstruct(self):
         return (
-            path,
+            self.canonical_module_path,
             self._constructor_args[0],
             self._constructor_args[1],
         )
 
+    def deconstruct_with_lookup(self, lookup):
+        """
+        Like `deconstruct`, but with a `wagtail.blocks.definition_lookup.BlockDefinitionLookupBuilder`
+        object available so that any block instances within the definition can be added to the lookup
+        table to obtain an ID (potentially shared with other matching block definitions, thus reducing
+        the overall definition size) to be used in place of the block. The resulting deconstructed form
+        returned here can then be restored into a block object using `Block.construct_from_lookup`.
+        """
+        # In the base implementation, no substitutions happen, so we ignore the lookup and just call
+        # deconstruct
+        return self.deconstruct()
+
     def __eq__(self, other):
         """
         Implement equality on block objects so that two blocks with matching definitions are considered
@@ -368,14 +538,9 @@ class Block(metaclass=BaseBlock):
         attributes identified in MUTABLE_META_ATTRIBUTES, so checking these along with the result of
         deconstruct (which captures the constructor arguments) is sufficient to identify (valid) differences.
 
-        This was originally necessary as a workaround for https://code.djangoproject.com/ticket/24340
-        in Django <1.9; the deep_deconstruct function used to detect changes for migrations did not
-        recurse into the block lists, and left them as Block instances. This __eq__ method therefore
-        came into play when identifying changes within migrations.
-
-        As of Django >=1.9, this *probably* isn't required any more. However, it may be useful in
-        future as a way of identifying blocks that can be re-used within StreamField definitions
-        (https://github.com/wagtail/wagtail/issues/4298#issuecomment-367656028).
+        This was implemented as a workaround for a Django <1.9 bug and is quite possibly not used by Wagtail
+        any more, but has been retained as it provides a sensible definition of equality (and there's no
+        reason to break it).
         """
 
         if not isinstance(other, Block):
@@ -446,7 +611,7 @@ class BoundBlock:
         return self.block.render(self.value)
 
     def __repr__(self):
-        return "<block %s: %r>" % (
+        return "<block {}: {!r}>".format(
             self.block.name or type(self.block).__name__,
             self.value,
         )
@@ -470,9 +635,7 @@ class DeclarativeSubBlocksMetaclass(BaseBlock):
         current_blocks.sort(key=lambda x: x[1].creation_counter)
         attrs["declared_blocks"] = collections.OrderedDict(current_blocks)
 
-        new_class = super(DeclarativeSubBlocksMetaclass, mcs).__new__(
-            mcs, name, bases, attrs
-        )
+        new_class = super().__new__(mcs, name, bases, attrs)
 
         # Walk through the MRO, collecting all inherited sub-blocks, to make
         # the combined `base_blocks`.
@@ -503,10 +666,14 @@ class BlockWidget(forms.Widget):
         super().__init__(attrs=attrs)
         self.block_def = block_def
         self._js_context = None
+        self._block_json = None
 
     def _build_block_json(self):
-        self._js_context = JSContext()
-        self._block_json = json.dumps(self._js_context.pack(self.block_def))
+        try:
+            self._js_context = JSContext()
+            self._block_json = json.dumps(self._js_context.pack(self.block_def))
+        except Exception as e:  # noqa: BLE001
+            raise ValueError("Error while serializing block definition: %s" % e) from e
 
     @property
     def js_context(self):
@@ -517,7 +684,7 @@ class BlockWidget(forms.Widget):
 
     @property
     def block_json(self):
-        if self._js_context is None:
+        if self._block_json is None:
             self._build_block_json()
 
         return self._block_json
@@ -531,21 +698,20 @@ class BlockWidget(forms.Widget):
         value_json = json.dumps(self.block_def.get_form_state(value))
 
         if errors:
-            errors_json = json.dumps(self.js_context.pack(errors.as_data()))
+            # errors is expected to be an ErrorList consisting of a single validation error
+            error = errors.as_data()[0]
+            error_json = json.dumps(get_error_json_data(error))
         else:
-            errors_json = "[]"
+            error_json = json.dumps(None)
 
         return format_html(
             """
-                <div id="{id}" data-block="{block_json}" data-value="{value_json}" data-errors="{errors_json}"></div>
-                <script>
-                    initBlockWidget('{id}');
-                </script>
+                <div id="{id}" data-block data-controller="w-block" data-w-block-data-value="{block_json}" data-w-block-arguments-value="[{value_json},{error_json}]"></div>
             """,
             id=name,
             block_json=self.block_json,
             value_json=value_json,
-            errors_json=errors_json,
+            error_json=error_json,
         )
 
     def render(self, name, value, attrs=None, renderer=None):
@@ -598,13 +764,41 @@ class BlockField(forms.Field):
         )
 
 
-@lru_cache(maxsize=1)
+@lru_cache(maxsize=None)
 def get_help_icon():
     return render_to_string(
-        "wagtailadmin/shared/icon.html", {"name": "help", "class_name": "default"}
+        "wagtailadmin/shared/icon.html", {"name": "help", "classname": "default"}
     )
 
 
+def get_error_json_data(error):
+    """
+    Translate a ValidationError instance raised against a block (which may potentially be a
+    ValidationError subclass specialised for a particular block type) into a JSON-serialisable dict
+    consisting of one or both of:
+    messages: a list of error message strings to be displayed against the block
+    blockErrors: a structure specific to the block type, containing further error objects in this
+        format to be displayed against this block's children
+    """
+    if hasattr(error, "as_json_data"):
+        return error.as_json_data()
+    else:
+        return {"messages": error.messages}
+
+
+def get_error_list_json_data(error_list):
+    """
+    Flatten an ErrorList instance containing any number of ValidationErrors
+    (which may themselves contain multiple messages) into a list of error message strings.
+    This does not consider any other properties of ValidationError other than `message`,
+    so should not be used where ValidationError subclasses with nested block errors may be
+    present.
+    (In terms of StreamBlockValidationError et al: it's valid for use on non_block_errors
+    but not block_errors)
+    """
+    return list(itertools.chain(*(err.messages for err in error_list.as_data())))
+
+
 DECONSTRUCT_ALIASES = {
     Block: "wagtail.blocks.Block",
 }
