@@ -1,4 +1,4 @@
 from wagtail.contrib.redirects.models import Redirect
-from wagtail.core.permission_policies import ModelPermissionPolicy
+from wagtail.permission_policies import ModelPermissionPolicy
 
 permission_policy = ModelPermissionPolicy(Redirect)
