@@ -1,23 +1,22 @@
 import os
 
-from django.core.exceptions import PermissionDenied, SuspiciousOperation
-from django.core.paginator import Paginator
+from django.core.exceptions import SuspiciousOperation
 from django.db import transaction
-from django.db.models import Q
-from django.shortcuts import get_object_or_404, redirect, render
-from django.template.response import TemplateResponse
+from django.shortcuts import redirect, render
 from django.urls import reverse
 from django.utils.encoding import force_str
+from django.utils.functional import cached_property
 from django.utils.translation import gettext as _
-from django.utils.translation import ngettext
+from django.utils.translation import gettext_lazy, ngettext
 from django.views.decorators.http import require_http_methods
-from django.views.decorators.vary import vary_on_headers
 
 from wagtail.admin import messages
 from wagtail.admin.auth import PermissionPolicyChecker
 from wagtail.admin.forms.search import SearchForm
-from wagtail.admin.views.reports import ReportView
-from wagtail.contrib.redirects import models
+from wagtail.admin.ui.tables import Column, StatusTagColumn, TitleColumn
+from wagtail.admin.views import generic
+from wagtail.admin.widgets.button import Button
+from wagtail.contrib.frontend_cache.utils import PurgeBatch, purge_urls_from_cache
 from wagtail.contrib.redirects.filters import RedirectsReportFilterSet
 from wagtail.contrib.redirects.forms import (
     ConfirmImportForm,
@@ -25,6 +24,7 @@ from wagtail.contrib.redirects.forms import (
     ImportForm,
     RedirectForm,
 )
+from wagtail.contrib.redirects.models import Redirect
 from wagtail.contrib.redirects.permissions import permission_policy
 from wagtail.contrib.redirects.utils import (
     get_file_storage,
@@ -34,171 +34,177 @@ from wagtail.contrib.redirects.utils import (
     write_to_file_storage,
 )
 from wagtail.log_actions import log
+from wagtail.models import Site
 
 permission_checker = PermissionPolicyChecker(permission_policy)
 
 
-@permission_checker.require_any("add", "change", "delete")
-@vary_on_headers("X-Requested-With")
-def index(request):
-    query_string = request.GET.get("q", "")
-    ordering = request.GET.get("ordering", "old_path")
+class RedirectTargetColumn(Column):
+    cell_template_name = "wagtailredirects/redirect_target_cell.html"
+    url_name = "wagtailadmin_pages:edit"
 
-    redirects = models.Redirect.objects.prefetch_related("redirect_page", "site")
+    def get_value(self, instance):
+        if instance.redirect_page_id:
+            return instance.redirect_page.get_admin_display_title()
+        return instance.redirect_link
 
-    # Search
-    if query_string:
-        redirects = redirects.filter(
-            Q(old_path__icontains=query_string)
-            | Q(redirect_page__url_path__icontains=query_string)
-            | Q(redirect_link__icontains=query_string)
-        )
+    def get_url(self, instance):
+        if instance.redirect_page_id:
+            return reverse(self.url_name, args=[instance.redirect_page_id])
+        return None
 
-    # Ordering (A bit useless at the moment as only 'old_path' is allowed)
-    if ordering not in ["old_path"]:
-        ordering = "old_path"
-
-    redirects = redirects.order_by(ordering)
-
-    # Pagination
-    paginator = Paginator(redirects, per_page=20)
-    redirects = paginator.get_page(request.GET.get("p"))
-
-    # Render template
-    if request.headers.get("x-requested-with") == "XMLHttpRequest":
-        return TemplateResponse(
-            request,
-            "wagtailredirects/results.html",
-            {
-                "ordering": ordering,
-                "redirects": redirects,
-                "query_string": query_string,
-            },
-        )
-    else:
-        return TemplateResponse(
-            request,
-            "wagtailredirects/index.html",
-            {
-                "ordering": ordering,
-                "redirects": redirects,
-                "query_string": query_string,
-                "search_form": SearchForm(
-                    data={"q": query_string} if query_string else None,
-                    placeholder=_("Search redirects"),
-                ),
-                "user_can_add": permission_policy.user_has_permission(
-                    request.user, "add"
-                ),
-            },
-        )
+    def get_cell_context_data(self, instance, parent_context):
+        context = super().get_cell_context_data(instance, parent_context)
+        context["url"] = self.get_url(instance)
+        return context
 
 
-@permission_checker.require("change")
-def edit(request, redirect_id):
-    theredirect = get_object_or_404(models.Redirect, id=redirect_id)
+class IndexView(generic.IndexView):
+    template_name = "wagtailredirects/index.html"
+    results_template_name = "wagtailredirects/index_results.html"
+    permission_policy = permission_policy
+    model = Redirect
+    header_icon = "redirect"
+    add_item_label = gettext_lazy("Add redirect")
+    context_object_name = "redirects"
+    index_url_name = "wagtailredirects:index"
+    index_results_url_name = "wagtailredirects:index_results"
+    add_url_name = "wagtailredirects:add"
+    edit_url_name = "wagtailredirects:edit"
+    delete_url_name = "wagtailredirects:delete"
+    default_ordering = "old_path"
+    paginate_by = 20
+    page_title = gettext_lazy("Redirects")
+    search_fields = ["old_path", "redirect_page__url_path", "redirect_link"]
+    columns = [
+        TitleColumn(
+            "old_path",
+            label=gettext_lazy("From"),
+            url_name="wagtailredirects:edit",
+            sort_key="old_path",
+        ),
+        Column(
+            "site",
+            label=gettext_lazy("Site"),
+            width="25%",
+            sort_key="site__site_name",
+        ),
+        RedirectTargetColumn(
+            "redirect_page",
+            label=gettext_lazy("To"),
+            width="30%",
+        ),
+        StatusTagColumn(
+            "is_permanent",
+            accessor="get_is_permanent_display",
+            label=gettext_lazy("Type"),
+            width="10%",
+            sort_key="is_permanent",
+            primary=lambda r: r.is_permanent,
+        ),
+    ]
+    filterset_class = RedirectsReportFilterSet
+    list_export = [
+        "old_path",
+        "link",
+        "get_is_permanent_display",
+        "site",
+    ]
+    export_headings = {
+        "old_path": _("From"),
+        "site": _("Site"),
+        "link": _("To"),
+        "get_is_permanent_display": _("Type"),
+    }
 
-    if not permission_policy.user_has_permission_for_instance(
-        request.user, "change", theredirect
-    ):
-        raise PermissionDenied
+    def get_base_queryset(self):
+        return super().get_base_queryset().select_related("redirect_page", "site")
 
-    if request.method == "POST":
-        form = RedirectForm(request.POST, request.FILES, instance=theredirect)
-        if form.is_valid():
-            with transaction.atomic():
-                form.save()
-                log(instance=theredirect, action="wagtail.edit")
-            messages.success(
-                request,
-                _("Redirect '{0}' updated.").format(theredirect.title),
-                buttons=[
-                    messages.button(
-                        reverse("wagtailredirects:edit", args=(theredirect.id,)),
-                        _("Edit"),
-                    )
-                ],
+    @cached_property
+    def header_more_buttons(self) -> list[Button]:
+        buttons = super().header_more_buttons.copy()
+        buttons.append(
+            Button(
+                _("Import redirects"),
+                url=reverse("wagtailredirects:start_import"),
+                icon_name="doc-full-inverse",
+                priority=50,
             )
-            return redirect("wagtailredirects:index")
-        else:
-            messages.error(request, _("The redirect could not be saved due to errors."))
-    else:
-        form = RedirectForm(instance=theredirect)
-
-    return TemplateResponse(
-        request,
-        "wagtailredirects/edit.html",
-        {
-            "redirect": theredirect,
-            "form": form,
-            "user_can_delete": permission_policy.user_has_permission(
-                request.user, "delete"
-            ),
-        },
-    )
-
-
-@permission_checker.require("delete")
-def delete(request, redirect_id):
-    theredirect = get_object_or_404(models.Redirect, id=redirect_id)
-
-    if not permission_policy.user_has_permission_for_instance(
-        request.user, "delete", theredirect
-    ):
-        raise PermissionDenied
-
-    if request.method == "POST":
-        with transaction.atomic():
-            log(instance=theredirect, action="wagtail.delete")
-            theredirect.delete()
-        messages.success(
-            request, _("Redirect '{0}' deleted.").format(theredirect.title)
         )
-        return redirect("wagtailredirects:index")
-
-    return TemplateResponse(
-        request,
-        "wagtailredirects/confirm_delete.html",
-        {
-            "redirect": theredirect,
-        },
-    )
+        return buttons
 
 
-@permission_checker.require("add")
-def add(request):
-    if request.method == "POST":
-        form = RedirectForm(request.POST, request.FILES)
-        if form.is_valid():
-            with transaction.atomic():
-                theredirect = form.save()
-                log(instance=theredirect, action="wagtail.edit")
+class EditView(generic.EditView):
+    model = Redirect
+    form_class = RedirectForm
+    permission_policy = permission_policy
+    template_name = "wagtailredirects/edit.html"
+    index_url_name = "wagtailredirects:index"
+    edit_url_name = "wagtailredirects:edit"
+    delete_url_name = "wagtailredirects:delete"
+    pk_url_kwarg = "redirect_id"
+    error_message = gettext_lazy("The redirect could not be saved due to errors.")
+    header_icon = "redirect"
 
-            messages.success(
-                request,
-                _("Redirect '{0}' added.").format(theredirect.title),
-                buttons=[
-                    messages.button(
-                        reverse("wagtailredirects:edit", args=(theredirect.id,)),
-                        _("Edit"),
-                    )
-                ],
-            )
-            return redirect("wagtailredirects:index")
-        else:
-            messages.error(
-                request, _("The redirect could not be created due to errors.")
-            )
-    else:
-        form = RedirectForm()
+    def get_success_message(self):
+        return _("Redirect '%(redirect_title)s' updated.") % {
+            "redirect_title": self.object.title
+        }
 
-    return TemplateResponse(
-        request,
-        "wagtailredirects/add.html",
-        {
-            "form": form,
-        },
-    )
+    def save_instance(self):
+        purger = PurgeBatch()
+
+        root_paths = Site.get_site_root_paths()
+        purger.add_urls(self.object.old_links(root_paths))
+
+        instance = super().save_instance()
+
+        purger.add_urls(self.object.old_links(root_paths))
+
+        purger.purge()
+
+        return instance
+
+
+class DeleteView(generic.DeleteView):
+    model = Redirect
+    pk_url_kwarg = "redirect_id"
+    permission_policy = permission_policy
+    template_name = "wagtailredirects/confirm_delete.html"
+    index_url_name = "wagtailredirects:index"
+    delete_url_name = "wagtailredirects:delete"
+    header_icon = "redirect"
+
+    def delete_action(self):
+        super().delete_action()
+        purge_urls_from_cache(self.object.old_links())
+
+    def get_success_message(self):
+        return _("Redirect '%(redirect_title)s' deleted.") % {
+            "redirect_title": self.object.title
+        }
+
+
+class CreateView(generic.CreateView):
+    model = Redirect
+    form_class = RedirectForm
+    permission_policy = permission_policy
+    template_name = "wagtailredirects/add.html"
+    add_url_name = "wagtailredirects:add"
+    index_url_name = "wagtailredirects:index"
+    edit_url_name = "wagtailredirects:edit"
+    error_message = gettext_lazy("The redirect could not be created due to errors.")
+    header_icon = "redirect"
+
+    def get_success_message(self, instance):
+        return _("Redirect '%(redirect_title)s' added.") % {
+            "redirect_title": instance.title
+        }
+
+    def save_instance(self):
+        instance = super().save_instance()
+        purge_urls_from_cache(instance.old_links())
+        return instance
 
 
 @permission_checker.require_any("add")
@@ -239,7 +245,9 @@ def start_import(request):
 
     if extension not in supported_extensions:
         messages.error(
-            request, _('File format of type "{}" is not supported').format(extension)
+            request,
+            _('File format of type "%(extension)s" is not supported')
+            % {"extension": extension},
         )
         return redirect("wagtailredirects:start_import")
 
@@ -253,9 +261,13 @@ def start_import(request):
             data = force_str(data, from_encoding)
         dataset = input_format.create_dataset(data)
     except UnicodeDecodeError as e:
-        messages.error(request, _("Imported file has a wrong encoding: %s") % e)
+        messages.error(
+            request,
+            _("Imported file has a wrong encoding: %(error_message)s")
+            % {"error_message": e},
+        )
         return redirect("wagtailredirects:start_import")
-    except Exception as e:  # pragma: no cover
+    except Exception as e:  # noqa: BLE001; pragma: no cover
         messages.error(
             request,
             _("%(error)s encountered while trying to read file: %(filename)s")
@@ -401,27 +413,3 @@ def to_readable_errors(error):
     errors = [x.lstrip("* ") for x in errors]
     errors = ", ".join(errors)
     return errors
-
-
-class RedirectsReportView(ReportView):
-    header_icon = "redirect"
-    title = _("Export Redirects")
-    template_name = "wagtailredirects/reports/redirects_report.html"
-    filterset_class = RedirectsReportFilterSet
-
-    list_export = [
-        "old_path",
-        "site",
-        "link",
-        "get_is_permanent_display",
-    ]
-
-    export_headings = {
-        "old_path": _("From"),
-        "site": _("Site"),
-        "link": _("To"),
-        "get_is_permanent_display": _("Type"),
-    }
-
-    def get_queryset(self):
-        return models.Redirect.objects.all().order_by("old_path")
