@@ -1,3 +1,5 @@
+from warnings import warn
+
 from django.apps import apps
 from django.contrib.auth.models import Permission
 from django.urls import reverse
@@ -10,23 +12,19 @@ from wagtail.admin.admin_url_finder import (
 )
 from wagtail.admin.menu import MenuItem
 from wagtail.permission_policies import ModelPermissionPolicy
+from wagtail.utils.deprecation import RemovedInWagtail70Warning
 
 from .permissions import user_can_edit_setting_type
 
 
 class SettingMenuItem(MenuItem):
-    def __init__(self, model, icon="cog", classnames="", **kwargs):
-
-        # Special-case FontAwesome icons to avoid the breaking changes for those customizations.
-        if icon.startswith("fa-"):
-            icon_name = ""
-            icon_classes = "icon icon-" + icon
-            if classnames:
-                classnames += " " + icon_classes
-            else:
-                classnames = icon_classes
-        else:
-            icon_name = icon
+    def __init__(self, model, icon="cog", classname="", classnames="", **kwargs):
+        if classnames:
+            warn(
+                "The `classnames` kwarg for SettingMenuItem is deprecated - use `classname` instead.",
+                category=RemovedInWagtail70Warning,
+            )
+        classname = classname or classnames
 
         self.model = model
         super().__init__(
@@ -35,8 +33,8 @@ class SettingMenuItem(MenuItem):
                 "wagtailsettings:edit",
                 args=[model._meta.app_label, model._meta.model_name],
             ),
-            classnames=classnames,
-            icon_name=icon_name,
+            classname=classname,
+            icon_name=icon,
             **kwargs,
         )
 
@@ -44,7 +42,7 @@ class SettingMenuItem(MenuItem):
         return user_can_edit_setting_type(request.user, self.model)
 
 
-class SettingsAdminURLFinder(ModelAdminURLFinder):
+class SiteSettingAdminURLFinder(ModelAdminURLFinder):
     def construct_edit_url(self, instance):
         return reverse(
             "wagtailsettings:edit",
@@ -56,11 +54,30 @@ class SettingsAdminURLFinder(ModelAdminURLFinder):
         )
 
 
+class GenericSettingAdminURLFinder(ModelAdminURLFinder):
+    def construct_edit_url(self, instance):
+        return reverse(
+            "wagtailsettings:edit",
+            args=[
+                self.model._meta.app_label,
+                self.model._meta.model_name,
+                instance.id,
+            ],
+        )
+
+
 class Registry(list):
-    def register(self, model, **kwargs):
+    def __init__(self):
+        self._model_icons = {}
+
+    def register(self, model, icon="cog", **kwargs):
+        from .models import BaseGenericSetting, BaseSiteSetting
+
         """
         Register a model as a setting, adding it to the wagtail admin menu
         """
+        if icon:
+            self._model_icons[model] = icon
 
         # Don't bother registering this if it is already registered
         if model in self:
@@ -70,33 +87,44 @@ class Registry(list):
         # Register a new menu item in the settings menu
         @hooks.register("register_settings_menu_item")
         def menu_hook():
-            return SettingMenuItem(model, **kwargs)
+            return SettingMenuItem(model, icon=self._model_icons.get(model), **kwargs)
 
         @hooks.register("register_permissions")
         def permissions_hook():
             return Permission.objects.filter(
                 content_type__app_label=model._meta.app_label,
-                codename="change_{}".format(model._meta.model_name),
+                codename=f"change_{model._meta.model_name}",
             )
 
         # Register an admin URL finder
         permission_policy = ModelPermissionPolicy(model)
-        finder_class = type(
-            "_SettingsAdminURLFinder",
-            (SettingsAdminURLFinder,),
-            {"model": model, "permission_policy": permission_policy},
-        )
+
+        if issubclass(model, BaseSiteSetting):
+            finder_class = type(
+                "_SiteSettingAdminURLFinder",
+                (SiteSettingAdminURLFinder,),
+                {"model": model, "permission_policy": permission_policy},
+            )
+        elif issubclass(model, BaseGenericSetting):
+            finder_class = type(
+                "_GenericSettingAdminURLFinder",
+                (GenericSettingAdminURLFinder,),
+                {"model": model, "permission_policy": permission_policy},
+            )
+        else:
+            raise NotImplementedError
+
         register_admin_url_finder(model, finder_class)
 
         return model
 
-    def register_decorator(self, model=None, **kwargs):
+    def register_decorator(self, model=None, icon="cog", **kwargs):
         """
         Register a model as a setting in the Wagtail admin
         """
         if model is None:
-            return lambda model: self.register(model, **kwargs)
-        return self.register(model, **kwargs)
+            return lambda model: self.register(model, icon=icon, **kwargs)
+        return self.register(model, icon=icon, **kwargs)
 
     def get_by_natural_key(self, app_label, model_name):
         """
@@ -109,9 +137,10 @@ class Registry(list):
             Model = apps.get_model(app_label, model_name)
         except LookupError:
             return None
-        if Model not in registry:
-            return None
-        return Model
+        else:
+            if Model not in registry:
+                return None
+            return Model
 
 
 registry = Registry()
