@@ -1,3 +1,4 @@
+from django.contrib.admin.utils import quote
 from django.contrib.auth.models import Group, Permission
 from django.contrib.contenttypes.models import ContentType
 from django.http import Http404
@@ -17,6 +18,7 @@ from wagtail.contrib.simple_translation.views import (
 from wagtail.models import Locale, Page
 from wagtail.test.i18n.models import TestPage
 from wagtail.test.snippets.models import TranslatableSnippet
+from wagtail.test.testapp.models import FullFeaturedSnippet
 from wagtail.test.utils import TestCase, WagtailTestUtils
 
 
@@ -31,6 +33,7 @@ from wagtail.test.utils import TestCase, WagtailTestUtils
         ("fr", "French"),
         ("de", "German"),
     ],
+    WAGTAIL_I18N_ENABLED=True,
 )
 class TestSubmitTranslationView(WagtailTestUtils, TestCase):
     def setUp(self):
@@ -116,6 +119,7 @@ class TestSubmitTranslationView(WagtailTestUtils, TestCase):
         ("fr", "French"),
         ("de", "German"),
     ],
+    WAGTAIL_I18N_ENABLED=True,
 )
 class TestSubmitPageTranslationView(WagtailTestUtils, TestCase):
     def setUp(self):
@@ -220,6 +224,7 @@ class TestSubmitPageTranslationView(WagtailTestUtils, TestCase):
         ("fr", "French"),
         ("de", "German"),
     ],
+    WAGTAIL_I18N_ENABLED=True,
 )
 class TestSubmitSnippetTranslationView(WagtailTestUtils, TestCase):
     def setUp(self):
@@ -256,32 +261,29 @@ class TestSubmitSnippetTranslationView(WagtailTestUtils, TestCase):
         view = SubmitSnippetTranslationView()
         view.object = self.en_snippet
         view.kwargs = {
-            "app_label": "some_app",
-            "model_name": "some_model",
+            "app_label": "snippetstests",
+            "model_name": "translatablesnippet",
             "pk": 99,
         }
         self.assertEqual(
-            view.get_success_url(), "/admin/snippets/some_app/some_model/edit/99/"
+            view.get_success_url(),
+            "/admin/snippets/snippetstests/translatablesnippet/edit/99/",
         )
 
     def test_get_success_url_for_single_locale(self):
         view = SubmitSnippetTranslationView()
         view.object = self.en_snippet
         view.kwargs = {
-            "app_label": "some_app",
-            "model_name": "some_model",
+            "app_label": "snippetstests",
+            "model_name": "translatablesnippet",
             "pk": 99,
         }
 
         self.assertEqual(
             view.get_success_url(view.object),
             reverse(
-                "wagtailsnippets:edit",
-                args=[
-                    "some_app",
-                    "some_model",
-                    view.object.pk,
-                ],
+                "wagtailsnippets_snippetstests_translatablesnippet:edit",
+                args=[view.object.pk],
             ),
         )
 
@@ -294,6 +296,102 @@ class TestSubmitSnippetTranslationView(WagtailTestUtils, TestCase):
         )
 
 
+@override_settings(
+    LANGUAGES=[
+        ("en", "English"),
+        ("fr", "French"),
+        ("de", "German"),
+    ],
+    WAGTAIL_CONTENT_LANGUAGES=[
+        ("en", "English"),
+        ("fr", "French"),
+        ("de", "German"),
+    ],
+    WAGTAIL_I18N_ENABLED=True,
+)
+class TestSubmitSnippetTranslationWithDraftState(WagtailTestUtils, TestCase):
+    def setUp(self):
+        self.login()
+        self.en_locale = Locale.objects.first()
+        self.fr_locale = Locale.objects.create(language_code="fr")
+        self.de_locale = Locale.objects.create(language_code="de")
+
+        self.en_snippet = FullFeaturedSnippet.objects.create(
+            text="Hello world", locale=self.en_locale, live=False
+        )
+        self.en_snippet.save_revision().publish()
+        self.en_snippet.text = "It's edited"
+        self.en_snippet.save_revision()
+        self.en_snippet.refresh_from_db()
+
+        self.model_opts = self.en_snippet._meta
+        self.app_label = self.model_opts.app_label
+        self.model_name = self.model_opts.model_name
+
+    def get_submit_url(self):
+        return reverse(
+            "simple_translation:submit_snippet_translation",
+            args=(self.app_label, self.model_name, quote(self.en_snippet.pk)),
+        )
+
+    def get_snippet_url(self, view, snippet):
+        return reverse(
+            snippet.snippet_viewset.get_url_name(view),
+            args=(quote(snippet.pk),),
+        )
+
+    def test_submit_snippet_translation_view_test_get(self):
+        response = self.client.get(self.get_submit_url())
+        self.assertIsInstance(response.context["form"], SubmitTranslationForm)
+
+    def test_submit_snippet_translation_view_test_post_invalid(self):
+        response = self.client.post(self.get_submit_url(), {})
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(
+            response.context["form"].errors,
+            {"locales": ["This field is required."]},
+        )
+
+    def test_submit_snippet_translation_view_test_post_single_locale(self):
+        data = {"locales": [self.de_locale.id], "include_subtree": True}
+        response = self.client.post(self.get_submit_url(), data, follow=True)
+
+        translated_snippet = self.en_snippet.get_translation(self.de_locale.id)
+        self.assertRedirects(response, self.get_snippet_url("edit", translated_snippet))
+
+        self.assertContains(response, "It's edited", count=1)
+        self.assertContains(response, '<h3 id="status-sidebar-german"', count=1)
+        self.assertContains(
+            response,
+            f'<a href="{self.get_snippet_url("edit", self.en_snippet)}"',
+            count=1,
+        )
+        self.assertNotContains(
+            response,
+            f'<a href="{self.get_snippet_url("edit", translated_snippet)}"',
+        )
+
+        self.assertEqual(
+            [msg.message for msg in response.context["messages"]],
+            ["Successfully created German for full-featured snippet 'It's edited'"],
+        )
+
+    def test_submit_snippet_translation_view_test_post_multiple_locales(self):
+        url = self.get_submit_url()
+        data = {"locales": [self.de_locale.id, self.fr_locale.id]}
+
+        response = self.client.post(url, data)
+
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(response.url, self.get_snippet_url("edit", self.en_snippet))
+
+        response = self.client.get(response.url)  # follow the redirect
+        self.assertEqual(
+            [msg.message for msg in response.context["messages"]],
+            ["Successfully created 2 locales for full-featured snippet 'It's edited'"],
+        )
+
+
 @override_settings(
     LANGUAGES=[
         ("en", "English"),
@@ -306,6 +404,7 @@ class TestSubmitSnippetTranslationView(WagtailTestUtils, TestCase):
         ("de", "German"),
     ],
     WAGTAILSIMPLETRANSLATION_SYNC_PAGE_TREE=True,
+    WAGTAIL_I18N_ENABLED=True,
 )
 class TestPageTreeSync(WagtailTestUtils, TestCase):
     def setUp(self):
@@ -343,3 +442,50 @@ class TestPageTreeSync(WagtailTestUtils, TestCase):
 
         self.assertFalse(en_blog_index.has_translation(self.fr_locale))
         self.assertFalse(en_blog_index.has_translation(self.de_locale))
+
+
+@override_settings(
+    LANGUAGES=[
+        ("en", "English"),
+        ("fr", "French"),
+        ("de", "German"),
+    ],
+    WAGTAIL_CONTENT_LANGUAGES=[
+        ("en", "English"),
+        ("fr", "French"),
+        ("de", "German"),
+    ],
+    WAGTAIL_I18N_ENABLED=True,
+)
+class TestPageListing(WagtailTestUtils, TestCase):
+    @classmethod
+    def setUpTestData(cls):
+        cls.en_locale = Locale.objects.first()
+        cls.fr_locale = Locale.objects.create(language_code="fr")
+        cls.de_locale = Locale.objects.create(language_code="de")
+
+        cls.en_homepage = Page.objects.get(depth=2)
+        cls.pages = []
+        for i in range(10):
+            instance = TestPage(title=f"Foo {i}", slug=f"foo-{i}")
+            cls.pages.append(cls.en_homepage.add_child(instance=instance))
+
+    def setUp(self):
+        self.user = self.login()
+
+    def test_translate_button_displayed(self):
+        url = reverse("wagtailadmin_explore", args=(self.en_homepage.pk,))
+        response = self.client.get(url)
+        with self.assertNumQueries(41):
+            response = self.client.get(url)
+
+        soup = self.get_soup(response.content)
+        page = self.pages[0]
+        translate_url = reverse(
+            "simple_translation:submit_page_translation",
+            args=[page.id],
+        )
+        translate_button = soup.select_one(f'a[href="{translate_url}"]')
+        self.assertIsNotNone(translate_button)
+        self.assertEqual(translate_button.text.strip(), "Translate")
+        self.assertIsNotNone(translate_button.select_one("svg.icon-globe"))
