@@ -3,21 +3,30 @@ import inspect
 import logging
 import re
 import unicodedata
-from typing import TYPE_CHECKING, Any, Dict, Iterable, Union
+from collections.abc import Iterable
+from hashlib import md5
+from typing import TYPE_CHECKING, Any, Union
+from warnings import warn
 
 from anyascii import anyascii
 from django.apps import apps
 from django.conf import settings
 from django.conf.locale import LANG_INFO
+from django.core.cache import cache
+from django.core.cache.utils import make_template_fragment_key
 from django.core.exceptions import ImproperlyConfigured, SuspiciousOperation
 from django.core.signals import setting_changed
 from django.db.models import Model
 from django.db.models.base import ModelBase
 from django.dispatch import receiver
 from django.http import HttpRequest
+from django.test import RequestFactory
 from django.utils.encoding import force_str
-from django.utils.text import slugify
+from django.utils.text import capfirst, slugify
 from django.utils.translation import check_for_language, get_supported_language_variant
+from django.utils.translation import gettext_lazy as _
+
+from wagtail.utils.deprecation import RemovedInWagtail70Warning
 
 if TYPE_CHECKING:
     from wagtail.models import Site
@@ -71,20 +80,18 @@ def resolve_model_string(model_string, default_app=None):
                 model_name = model_string
             else:
                 raise ValueError(
-                    "Can not resolve {0!r} into a model. Model names "
+                    "Can not resolve {!r} into a model. Model names "
                     "should be in the form app_label.model_name".format(model_string),
                     model_string,
                 )
 
         return apps.get_model(app_label, model_name)
 
-    elif isinstance(model_string, type) and issubclass(model_string, Model):
+    elif isinstance(model_string, type):
         return model_string
 
     else:
-        raise ValueError(
-            "Can not resolve {0!r} into a model".format(model_string), model_string
-        )
+        raise ValueError(f"Can not resolve {model_string!r} into a model", model_string)
 
 
 SCRIPT_RE = re.compile(r"<(-*)/script>")
@@ -96,6 +103,10 @@ def escape_script(text):
     accidentally closing it. A '-' character will be inserted for each time it is escaped:
     `<-/script>`, `<--/script>` etc.
     """
+    warn(
+        "The `escape_script` hook is deprecated - use `template` elements instead.",
+        category=RemovedInWagtail70Warning,
+    )
     return SCRIPT_RE.sub(r"<-\1/script>", text)
 
 
@@ -136,7 +147,7 @@ def cautious_slugify(value):
 
 def safe_snake_case(value):
     """
-    Convert a string to ASCII similar to Django's slugify, with catious handling of
+    Convert a string to ASCII similar to Django's slugify, with cautious handling of
     non-ASCII alphanumeric characters. See `cautious_slugify`.
 
     Any inner whitespace, hyphens or dashes will be converted to underscores and
@@ -155,12 +166,15 @@ def get_content_type_label(content_type):
     Return a human-readable label for a content type object, suitable for display in the admin
     in place of the default 'wagtailcore | page' representation
     """
+    if content_type is None:
+        return _("Unknown content type")
+
     model = content_type.model_class()
     if model:
-        return model._meta.verbose_name.capitalize()
+        return str(capfirst(model._meta.verbose_name))
     else:
         # no corresponding model class found; fall back on the name field of the ContentType
-        return content_type.model.capitalize()
+        return capfirst(content_type.model)
 
 
 def accepts_kwarg(func, kwarg):
@@ -243,7 +257,7 @@ def find_available_slug(parent, requested_slug, ignore_page_id=None):
     return slug
 
 
-@functools.lru_cache()
+@functools.cache
 def get_content_languages():
     """
     Cache of settings.WAGTAIL_CONTENT_LANGUAGES in a dictionary for easy lookups by key.
@@ -322,17 +336,21 @@ def get_supported_content_language_variant(lang_code, strict=False):
     raise LookupError(lang_code)
 
 
-@functools.lru_cache()
 def get_locales_display_names() -> dict:
     """
     Cache of the locale id -> locale display name mapping
     """
     from wagtail.models import Locale  # inlined to avoid circular imports
 
-    locales_map = {
-        locale.pk: locale.get_display_name() for locale in Locale.objects.all()
-    }
-    return locales_map
+    cached_map = cache.get("wagtail_locales_display_name")
+
+    if cached_map is None:
+        cached_map = {
+            locale.pk: locale.get_display_name() for locale in Locale.objects.all()
+        }
+        cache.set("wagtail_locales_display_name", cached_map)
+
+    return cached_map
 
 
 @receiver(setting_changed)
@@ -379,14 +397,12 @@ def multigetattr(item, accessor):
                     TypeError,  # unsubscriptable object
                 ):
                     raise AttributeError(
-                        "Failed lookup for key [%s] in %r" % (bit, current)
+                        f"Failed lookup for key [{bit}] in {current!r}"
                     )
 
         if callable(current):
             if getattr(current, "alters_data", False):
-                raise SuspiciousOperation(
-                    "Cannot call %r from multigetattr" % (current,)
-                )
+                raise SuspiciousOperation(f"Cannot call {current!r} from multigetattr")
 
             # if calling without arguments is invalid, let the exception bubble up
             current = current()
@@ -394,7 +410,7 @@ def multigetattr(item, accessor):
     return current
 
 
-def get_dummy_request(path: str = "/", site: "Site" = None) -> HttpRequest:
+def get_dummy_request(*, path: str = "/", site: "Site" = None) -> HttpRequest:
     """
     Return a simple ``HttpRequest`` instance that can be passed to
     ``Page.get_url()`` and other methods to benefit from improved performance
@@ -403,19 +419,32 @@ def get_dummy_request(path: str = "/", site: "Site" = None) -> HttpRequest:
     If ``site`` is provided, the ``HttpRequest`` is made to look like it came
     from that Wagtail ``Site``.
     """
-    request = HttpRequest()
-    request.path = path
-    request.method = "GET"
-    SERVER_PORT = 80
+    server_port = 80
     if site:
-        SERVER_NAME = site.hostname
-        SERVER_PORT = site.port
-    elif settings.ALLOWED_HOSTS == ["*"]:
-        SERVER_NAME = "example.com"
+        server_name = site.hostname
+        server_port = site.port
     else:
-        SERVER_NAME = settings.ALLOWED_HOSTS[0]
-    request.META = {"SERVER_NAME": SERVER_NAME, "SERVER_PORT": SERVER_PORT}
-    return request
+        server_name = settings.ALLOWED_HOSTS[0]
+
+        if server_name == "*":
+            server_name = "example.com"
+
+    # `SERVER_PORT` doesn't work when passed to the constructor
+    return RequestFactory(SERVER_NAME=server_name).get(path, SERVER_PORT=server_port)
+
+
+def safe_md5(data=b"", usedforsecurity=True):
+    """
+    Safely use the MD5 hash algorithm with the given ``data`` and a flag
+    indicating if the purpose of the digest is for security or not.
+
+    On security-restricted systems (such as FIPS systems), insecure hashes
+    like MD5 are disabled by default. But passing ``usedforsecurity`` as
+    ``False`` tells the underlying security implementation we're not trying
+    to use the digest for secure purposes and to please just go ahead and
+    allow it to happen.
+    """
+    return md5(data, usedforsecurity=usedforsecurity)
 
 
 class BatchProcessor:
@@ -517,7 +546,7 @@ class BatchCreator(BatchProcessor):
         if self.max_size and len(self.items) == self.max_size:
             self.process()
 
-    def extend(self, iterable: Iterable[Union[Model, Dict[str, Any]]]) -> None:
+    def extend(self, iterable: Iterable[Union[Model, dict[str, Any]]]) -> None:
         for value in iterable:
             if isinstance(value, self.model):
                 self.add(instance=value)
@@ -539,3 +568,14 @@ class BatchCreator(BatchProcessor):
     def get_summary(self):
         opts = self.model._meta
         return f"{self.created_count}/{self.added_count} {opts.verbose_name_plural} were created successfully."
+
+
+def make_wagtail_template_fragment_key(fragment_name, page, site, vary_on=None):
+    """
+    A modified version of `make_template_fragment_key` which varies on page and
+    site for use with `{% wagtailpagecache %}`.
+    """
+    if vary_on is None:
+        vary_on = []
+    vary_on.extend([page.cache_key, site.id])
+    return make_template_fragment_key(fragment_name, vary_on)
