@@ -1,5 +1,10 @@
 import os.path
 
+from django.urls import reverse
+from django.utils.text import capfirst
+from django.utils.translation import gettext_lazy
+
+from wagtail.admin.views.generic.base import WagtailAdminTemplateMixin
 from wagtail.admin.views.generic.multiple_upload import AddView as BaseAddView
 from wagtail.admin.views.generic.multiple_upload import (
     CreateFromUploadView as BaseCreateFromUploadView,
@@ -9,19 +14,19 @@ from wagtail.admin.views.generic.multiple_upload import (
 )
 from wagtail.admin.views.generic.multiple_upload import DeleteView as BaseDeleteView
 from wagtail.admin.views.generic.multiple_upload import EditView as BaseEditView
-from wagtail.search.backends import get_search_backends
 
 from .. import get_document_model
 from ..forms import get_document_form, get_document_multi_form
-from ..models import UploadedDocument
 from ..permissions import permission_policy
 
 
-class AddView(BaseAddView):
+class AddView(WagtailAdminTemplateMixin, BaseAddView):
     permission_policy = permission_policy
     template_name = "wagtaildocs/multiple/add.html"
-    upload_model = UploadedDocument
+    header_icon = "doc-full-inverse"
+    page_title = gettext_lazy("Add documents")
 
+    index_url_name = "wagtaildocs:index"
     edit_object_url_name = "wagtaildocs:edit_multiple"
     delete_object_url_name = "wagtaildocs:delete_multiple"
     edit_object_form_prefix = "doc"
@@ -32,7 +37,16 @@ class AddView(BaseAddView):
     delete_upload_url_name = "wagtaildocs:delete_upload_multiple"
     edit_upload_form_prefix = "uploaded-document"
     context_upload_name = "uploaded_document"
-    context_upload_id_name = "uploaded_document_id"
+    context_upload_id_name = "uploaded_file_id"
+
+    def get_breadcrumbs_items(self):
+        return self.breadcrumbs_items + [
+            {
+                "url": reverse(self.index_url_name),
+                "label": capfirst(self.model._meta.verbose_name_plural),
+            },
+            {"url": "", "label": self.get_page_title()},
+        ]
 
     def get_model(self):
         return get_document_model()
@@ -46,13 +60,7 @@ class AddView(BaseAddView):
     def save_object(self, form):
         doc = form.save(commit=False)
         doc.uploaded_by_user = self.request.user
-        doc.file_size = doc.file.size
-
-        # Set new document file hash
-        doc.file.seek(0)
-        doc._set_file_hash(doc.file.read())
-        doc.file.seek(0)
-
+        doc._set_document_file_metadata()
         doc.save()
 
         return doc
@@ -84,13 +92,6 @@ class EditView(BaseEditView):
     def get_edit_form_class(self):
         return get_document_multi_form(self.model)
 
-    def save_object(self, form):
-        form.save()
-
-        # Reindex the doc to make sure all tags are indexed
-        for backend in get_search_backends():
-            backend.add(self.object)
-
 
 class DeleteView(BaseDeleteView):
     permission_policy = permission_policy
@@ -104,8 +105,7 @@ class DeleteView(BaseDeleteView):
 class CreateFromUploadedDocumentView(BaseCreateFromUploadView):
     edit_upload_url_name = "wagtaildocs:create_multiple_from_uploaded_document"
     delete_upload_url_name = "wagtaildocs:delete_upload_multiple"
-    upload_model = UploadedDocument
-    upload_pk_url_kwarg = "uploaded_document_id"
+    upload_pk_url_kwarg = "uploaded_file_id"
     edit_upload_form_prefix = "uploaded-document"
     context_object_id_name = "doc_id"
     context_upload_name = "uploaded_document"
@@ -124,18 +124,15 @@ class CreateFromUploadedDocumentView(BaseCreateFromUploadView):
             os.path.basename(self.upload.file.name), self.upload.file.file, save=False
         )
         self.object.uploaded_by_user = self.request.user
-        self.object.file_size = self.object.file.size
-        self.object.file.open()
-        self.object.file.seek(0)
-        self.object._set_file_hash(self.object.file.read())
-        self.object.file.seek(0)
-        form.save()
 
-        # Reindex the document to make sure all tags are indexed
-        for backend in get_search_backends():
-            backend.add(self.object)
+        # form.save() would normally handle writing the image file metadata, but in this case the
+        # file handling happens outside the form, so we need to do that manually
+        self.object._set_document_file_metadata()
+        form.save()
 
 
 class DeleteUploadView(BaseDeleteUploadView):
-    upload_model = UploadedDocument
-    upload_pk_url_kwarg = "uploaded_document_id"
+    upload_pk_url_kwarg = "uploaded_file_id"
+
+    def get_model(self):
+        return get_document_model()
