@@ -1,4 +1,5 @@
 from io import BytesIO
+from pathlib import Path
 from unittest.mock import patch
 
 from django.test import TestCase, override_settings
@@ -11,8 +12,10 @@ from wagtail.images.exceptions import (
 )
 from wagtail.images.image_operations import TransformOperation
 from wagtail.images.models import Filter, Image
+from wagtail.images.shortcuts import get_rendition_or_not_found
 from wagtail.images.tests.utils import (
     get_test_image_file,
+    get_test_image_file_avif,
     get_test_image_file_jpeg,
     get_test_image_file_tiff,
     get_test_image_file_webp,
@@ -96,7 +99,9 @@ class ImageTransformOperationTestCase(TestCase):
             )
 
         test_filter_spec_error.__name__ = str(
-            "test_filter_%s_raises_%s" % (filter_spec, InvalidFilterSpecError.__name__)
+            "test_filter_{}_raises_{}".format(
+                filter_spec, InvalidFilterSpecError.__name__
+            )
         )
         return test_filter_spec_error
 
@@ -616,6 +621,30 @@ class TestUnknownOutputImageFormat(TestCase):
 
 
 class TestFormatFilter(TestCase):
+    def test_avif(self):
+        fil = Filter(spec="width-400|format-avif")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file(),
+        )
+        out = fil.run(image, BytesIO())
+
+        self.assertEqual(out.format_name, "avif")
+
+    def test_avif_lossless(self):
+        fil = Filter(spec="width-400|format-avif-lossless")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file(),
+        )
+
+        f = BytesIO()
+        with patch("PIL.Image.Image.save") as save:
+            fil.run(image, f)
+
+        # quality=80 is default for The Willow and PIL libraries
+        save.assert_called_with(f, "AVIF", quality=-1, chroma=444)
+
     def test_jpeg(self):
         fil = Filter(spec="width-400|format-jpeg")
         image = Image.objects.create(
@@ -656,6 +685,26 @@ class TestFormatFilter(TestCase):
 
         self.assertEqual(out.format_name, "webp")
 
+    def test_ico(self):
+        fil = Filter(spec="width-400|format-ico")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file(),
+        )
+        out = fil.run(image, BytesIO())
+
+        self.assertEqual(out.format_name, "ico")
+
+    def test_ico_rendition(self):
+        fil = Filter(spec="width-400|format-ico")
+        good_image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file(),
+        )
+
+        rendition = get_rendition_or_not_found(good_image, fil)
+        self.assertEqual(Path(rendition.file.name).suffix, ".ico")
+
     def test_webp_lossless(self):
         fil = Filter(spec="width-400|format-webp-lossless")
         image = Image.objects.create(
@@ -667,9 +716,32 @@ class TestFormatFilter(TestCase):
         with patch("PIL.Image.Image.save") as save:
             fil.run(image, f)
 
-        # quality=80 is default for Williw and PIL libs
+        # quality=80 is default for the Willow and PIL libraries
         save.assert_called_with(f, "WEBP", quality=80, lossless=True)
 
+    def test_heic(self):
+        fil = Filter(spec="width-400|format-heic")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file(),
+        )
+        out = fil.run(image, BytesIO())
+
+        self.assertEqual(out.format_name, "heic")
+
+    def test_heic_lossless(self):
+        fil = Filter(spec="width-400|format-heic-lossless")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file(),
+        )
+
+        f = BytesIO()
+        with patch("PIL.Image.Image.save") as save:
+            fil.run(image, f)
+
+        save.assert_called_with(f, "HEIF", quality=-1, chroma=444)
+
     def test_invalid(self):
         fil = Filter(spec="width-400|format-foo")
         image = Image.objects.create(
@@ -679,6 +751,86 @@ class TestFormatFilter(TestCase):
         self.assertRaises(InvalidFilterSpecError, fil.run, image, BytesIO())
 
 
+class TestAvifQualityFilter(TestCase):
+    def test_default_quality(self):
+        fil = Filter(spec="width-400|format-avif")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file_avif(),
+        )
+
+        f = BytesIO()
+        with patch("PIL.Image.Image.save") as save:
+            fil.run(image, f)
+
+        save.assert_called_with(f, "AVIF", quality=80)
+
+    def test_avif_quality_filter(self):
+        fil = Filter(spec="width-400|avifquality-40|format-avif")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file_jpeg(),
+        )
+
+        f = BytesIO()
+        with patch("PIL.Image.Image.save") as save:
+            fil.run(image, f)
+
+        save.assert_called_with(f, "AVIF", quality=40)
+
+    def test_avif_quality_filter_invalid(self):
+        fil = Filter(spec="width-400|avifquality-abc|format-avif")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file_jpeg(),
+        )
+        self.assertRaises(InvalidFilterSpecError, fil.run, image, BytesIO())
+
+    def test_avif_quality_filter_no_value(self):
+        fil = Filter(spec="width-400|avifquality")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file_jpeg(),
+        )
+        self.assertRaises(InvalidFilterSpecError, fil.run, image, BytesIO())
+
+    def test_avif_quality_filter_too_big(self):
+        fil = Filter(spec="width-400|avifquality-101|format-avif")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file_jpeg(),
+        )
+        self.assertRaises(InvalidFilterSpecError, fil.run, image, BytesIO())
+
+    @override_settings(WAGTAILIMAGES_AVIF_QUALITY=50)
+    def test_avif_quality_setting(self):
+        fil = Filter(spec="width-400|format-avif")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file_jpeg(),
+        )
+
+        f = BytesIO()
+        with patch("PIL.Image.Image.save") as save:
+            fil.run(image, f)
+
+        save.assert_called_with(f, "AVIF", quality=50)
+
+    @override_settings(WAGTAILIMAGES_AVIF_QUALITY=50)
+    def test_avif_quality_filter_overrides_setting(self):
+        fil = Filter(spec="width-400|avifquality-40|format-avif")
+        image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file_jpeg(),
+        )
+
+        f = BytesIO()
+        with patch("PIL.Image.Image.save") as save:
+            fil.run(image, f)
+
+        save.assert_called_with(f, "AVIF", quality=40)
+
+
 class TestJPEGQualityFilter(TestCase):
     def test_default_quality(self):
         fil = Filter(spec="width-400")
@@ -771,7 +923,7 @@ class TestWebPQualityFilter(TestCase):
         with patch("PIL.Image.Image.save") as save:
             fil.run(image, f)
 
-        save.assert_called_with(f, "WEBP", quality=85, lossless=False)
+        save.assert_called_with(f, "WEBP", quality=80, lossless=False)
 
     def test_webp_quality_filter(self):
         fil = Filter(spec="width-400|webpquality-40|format-webp")
@@ -825,7 +977,7 @@ class TestWebPQualityFilter(TestCase):
         save.assert_called_with(f, "WEBP", quality=50, lossless=False)
 
     @override_settings(WAGTAILIMAGES_WEBP_QUALITY=50)
-    def test_jpeg_quality_filter_overrides_setting(self):
+    def test_webp_quality_filter_overrides_setting(self):
         fil = Filter(spec="width-400|webpquality-40|format-webp")
         image = Image.objects.create(
             title="Test image",
@@ -913,3 +1065,38 @@ class TestWebpFormatConversion(TestCase):
         out = fil.run(image, BytesIO())
 
         self.assertEqual(out.format_name, "webp")
+
+
+class TestCheckSize(TestCase):
+    def test_check_size_when_floats_allowed(self):
+        sizes = [
+            (1.5, 1.5),
+            (1.5, 1),
+            (1, 1.5),
+            (1, 1),
+        ]
+        for size in sizes:
+            with self.subTest(size=size):
+                self.assertIsNone(
+                    image_operations.ImageTransform._check_size(
+                        size, allow_floating_point=True
+                    )
+                )
+
+    def test_check_size_when_floats_forbidden(self):
+        fail_sizes = [
+            (1.5, 1.5),
+            (1.5, 1),
+            (1, 1.5),
+        ]
+        for size in fail_sizes:
+            with self.subTest(size=size):
+                with self.assertRaises(TypeError):
+                    image_operations.ImageTransform._check_size(
+                        size, allow_floating_point=False
+                    )
+        self.assertIsNone(
+            image_operations.ImageTransform._check_size(
+                (1, 1), allow_floating_point=False
+            )
+        )
