@@ -1,20 +1,27 @@
 import unittest
 
+from django.conf import settings
 from django.contrib.auth.models import Group, Permission
 from django.core.cache import caches
 from django.core.files import File
-from django.core.files.storage import DefaultStorage, Storage
+from django.core.files.storage import Storage, default_storage, storages
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.db.models import Prefetch
 from django.db.utils import IntegrityError
-from django.test import TestCase
-from django.test.utils import override_settings
+from django.test import SimpleTestCase, TestCase, TransactionTestCase, override_settings
 from django.urls import reverse
 from willow.image import Image as WillowImage
 
-from wagtail.images.models import Rendition, SourceImageIOError, get_rendition_storage
+from wagtail.images.models import (
+    Filter,
+    Picture,
+    Rendition,
+    ResponsiveImage,
+    SourceImageIOError,
+    get_rendition_storage,
+)
 from wagtail.images.rect import Rect
-from wagtail.models import Collection, GroupCollectionPermission, Page
+from wagtail.models import Collection, GroupCollectionPermission, Page, ReferenceIndex
 from wagtail.test.testapp.models import (
     EventPage,
     EventPageCarouselItem,
@@ -22,12 +29,16 @@ from wagtail.test.testapp.models import (
 )
 from wagtail.test.utils import WagtailTestUtils
 
-from .utils import Image, get_test_image_file
+from .utils import (
+    Image,
+    get_test_image_file,
+    get_test_image_file_svg,
+    get_test_image_filename,
+)
 
 
 class CustomStorage(Storage):
-    def __init__(self):
-        super().__init__()
+    pass
 
 
 class TestImage(TestCase):
@@ -48,7 +59,7 @@ class TestImage(TestCase):
         self.assertTrue(self.image.is_landscape())
 
     def test_get_rect(self):
-        self.assertTrue(self.image.get_rect(), Rect(0, 0, 640, 480))
+        self.assertEqual(self.image.get_rect(), Rect(0, 0, 640, 480))
 
     def test_get_focal_point(self):
         self.assertIsNone(self.image.get_focal_point())
@@ -97,7 +108,12 @@ class TestImage(TestCase):
         self.assertTrue(self.image.is_stored_locally())
 
     @override_settings(
-        DEFAULT_FILE_STORAGE="wagtail.test.dummy_external_storage.DummyExternalStorage"
+        STORAGES={
+            **settings.STORAGES,
+            "default": {
+                "BACKEND": "wagtail.test.dummy_external_storage.DummyExternalStorage"
+            },
+        },
     )
     def test_is_stored_locally_with_external_storage(self):
         self.assertFalse(self.image.is_stored_locally())
@@ -112,8 +128,44 @@ class TestImage(TestCase):
         with self.assertRaises(SourceImageIOError):
             self.image.get_file_size()
 
+    def test_file_hash(self):
+        self.assertEqual(
+            self.image.get_file_hash(), "4dd0211870e130b7e1690d2ec53c499a54a48fef"
+        )
+
+    def test_get_suggested_focal_point_svg(self):
+        """
+        Feature detection should not be run on SVGs.
+
+        https://github.com/wagtail/wagtail/issues/11172
+        """
+        image = Image.objects.create(
+            title="Test SVG",
+            file=get_test_image_file_svg(),
+        )
+        self.assertIsNone(image.get_suggested_focal_point())
+
+    def test_default_with_description(self):
+        # Primary default should be description
+        image = Image.objects.create(
+            title="Test Image",
+            description="This is a test description",
+            file=get_test_image_file(),
+        )
+        self.assertEqual(image.default_alt_text, image.description)
+
+    def test_default_without_description(self):
+        # Secondary default should be title
+        image = Image.objects.create(
+            title="Test Image",
+            file=get_test_image_file(),
+        )
+        self.assertEqual(image.default_alt_text, image.title)
+
+
+class TestImageQuerySet(TransactionTestCase):
+    fixtures = ["test_empty.json"]
 
-class TestImageQuerySet(TestCase):
     def test_search_method(self):
         # Create an image for running tests on
         image = Image.objects.create(
@@ -152,9 +204,13 @@ class TestImageQuerySet(TestCase):
             file=get_test_image_file(),
         )
 
-        results = Image.objects.order_by("title").search("Test")
+        results = Image.objects.order_by("title").search(
+            "Test", order_by_relevance=False
+        )
         self.assertEqual(list(results), [aaa_image, zzz_image])
-        results = Image.objects.order_by("-title").search("Test")
+        results = Image.objects.order_by("-title").search(
+            "Test", order_by_relevance=False
+        )
         self.assertEqual(list(results), [zzz_image, aaa_image])
 
     def test_search_indexing_prefetches_tags(self):
@@ -173,7 +229,7 @@ class TestImageQuerySet(TestCase):
             self.assertIn("aardvark", results["Test image 0"])
 
 
-class TestImagePermissions(TestCase, WagtailTestUtils):
+class TestImagePermissions(WagtailTestUtils, TestCase):
     def setUp(self):
         # Create some user accounts for testing permissions
         self.user = self.create_user(
@@ -221,7 +277,239 @@ class TestImagePermissions(TestCase, WagtailTestUtils):
         self.assertFalse(self.image.is_editable_by_user(self.user))
 
 
+class TestFilters(SimpleTestCase):
+    def test_expand_spec_single(self):
+        self.assertEqual(Filter.expand_spec("width-100"), ["width-100"])
+
+    def test_expand_spec_flat(self):
+        self.assertEqual(
+            Filter.expand_spec("width-100 jpegquality-20"), ["width-100|jpegquality-20"]
+        )
+
+    def test_expand_spec_pipe(self):
+        self.assertEqual(
+            Filter.expand_spec("width-100|jpegquality-20"), ["width-100|jpegquality-20"]
+        )
+
+    def test_expand_spec_list(self):
+        self.assertEqual(
+            Filter.expand_spec(["width-100", "jpegquality-20"]),
+            ["width-100|jpegquality-20"],
+        )
+
+    def test_expand_spec_braced(self):
+        self.assertEqual(
+            Filter.expand_spec("width-{100,200}"), ["width-100", "width-200"]
+        )
+
+    def test_expand_spec_mixed(self):
+        self.assertEqual(
+            Filter.expand_spec("width-{100,200} jpegquality-40"),
+            ["width-100|jpegquality-40", "width-200|jpegquality-40"],
+        )
+
+    def test_expand_spec_mixed_pipe(self):
+        self.assertEqual(
+            Filter.expand_spec("width-{100,200}|jpegquality-40"),
+            ["width-100|jpegquality-40", "width-200|jpegquality-40"],
+        )
+
+    def test_expand_spec_multiple_braces(self):
+        self.assertEqual(
+            Filter.expand_spec("width-{100,200} jpegquality-{40,80} grayscale"),
+            [
+                "width-100|jpegquality-40|grayscale",
+                "width-100|jpegquality-80|grayscale",
+                "width-200|jpegquality-40|grayscale",
+                "width-200|jpegquality-80|grayscale",
+            ],
+        )
+
+
+class TestResponsiveImage(TestCase):
+    def setUp(self):
+        # Create an image for running tests on
+        self.image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file(),
+        )
+        self.rendition_10 = self.image.get_rendition("width-10")
+
+    def test_construct_empty(self):
+        img = ResponsiveImage({})
+        self.assertEqual(img.renditions, [])
+        self.assertEqual(img.attrs, None)
+
+    def test_construct_with_renditions(self):
+        renditions = {"a": self.rendition_10}
+        img = ResponsiveImage(renditions)
+        self.assertEqual(img.renditions, [self.rendition_10])
+
+    def test_evaluate_value(self):
+        self.assertFalse(ResponsiveImage({}))
+        self.assertFalse(ResponsiveImage({}, {"sizes": "100vw"}))
+
+        renditions = {"a": self.rendition_10}
+        self.assertTrue(ResponsiveImage(renditions))
+
+    def test_compare_value(self):
+        renditions = {"a": self.rendition_10}
+        value1 = ResponsiveImage(renditions)
+        value2 = ResponsiveImage(renditions)
+        value3 = ResponsiveImage({"a": self.image.get_rendition("width-15")})
+        value4 = ResponsiveImage(renditions, {"sizes": "100vw"})
+        self.assertNotEqual(value1, value3)
+        self.assertNotEqual(value1, 12345)
+        self.assertEqual(value1, value2)
+        self.assertNotEqual(value1, value4)
+
+    def test_get_width_srcset(self):
+        renditions = {
+            "width-10": self.rendition_10,
+            "width-90": self.image.get_rendition("width-90"),
+        }
+        filenames = [
+            get_test_image_filename(self.image, "width-10"),
+            get_test_image_filename(self.image, "width-90"),
+        ]
+        self.assertEqual(
+            ResponsiveImage.get_width_srcset(list(renditions.values())),
+            f"{filenames[0]} 10w, {filenames[1]} 90w",
+        )
+
+    def test_get_width_srcset_single_rendition(self):
+        renditions = {"width-10": self.rendition_10}
+        self.assertEqual(
+            ResponsiveImage.get_width_srcset(list(renditions.values())),
+            get_test_image_filename(self.image, "width-10"),
+        )
+
+    def test_render(self):
+        renditions = {
+            "width-10": self.rendition_10,
+            "width-90": self.image.get_rendition("width-90"),
+        }
+        img = ResponsiveImage(renditions)
+        filenames = [
+            get_test_image_filename(self.image, "width-10"),
+            get_test_image_filename(self.image, "width-90"),
+        ]
+        self.assertHTMLEqual(
+            img.__html__(),
+            f"""
+                <img
+                    alt="Test image"
+                    src="{filenames[0]}"
+                    srcset="{filenames[0]} 10w, {filenames[1]} 90w"
+                    width="10"
+                    height="7"
+                >
+            """,
+        )
+
+    def test_render_single_image_same_as_img_tag(self):
+        img = ResponsiveImage({"width-10": self.rendition_10})
+        self.assertHTMLEqual(img.__html__(), self.rendition_10.img_tag())
+
+
+class TestPicture(TestCase):
+    def setUp(self):
+        # Create an image for running tests on
+        self.image = Image.objects.create(
+            title="Test image",
+            file=get_test_image_file(),
+        )
+        self.rendition_10 = self.image.get_rendition("width-10")
+
+    def test_formats(self):
+        renditions = {
+            "format-jpeg": self.rendition_10,
+            "format-webp": self.rendition_10,
+        }
+        img = Picture(renditions)
+        self.assertEqual(
+            img.formats, {"jpeg": [self.rendition_10], "webp": [self.rendition_10]}
+        )
+
+    def test_single_format(self):
+        renditions = {"format-jpeg": self.rendition_10}
+        img = Picture(renditions)
+        self.assertEqual(img.formats, {})
+
+    def test_mixed_format(self):
+        renditions = {
+            "format-jpeg": self.rendition_10,
+            "format-webp": self.rendition_10,
+            "format-webp-lossless": self.rendition_10,
+        }
+        img = Picture(renditions)
+        self.assertEqual(
+            img.formats,
+            {
+                "jpeg": [self.rendition_10],
+                "webp": [self.rendition_10, self.rendition_10],
+            },
+        )
+
+    def test_fallback_format(self):
+        avif = {"format-avif": self.rendition_10}
+        webp = {"format-webp": self.rendition_10}
+        jpeg = {"format-jpeg": self.rendition_10}
+        png = {"format-png": self.rendition_10}
+        gif = {"format-gif": self.rendition_10}
+        fallbacks = {
+            "gif": {**avif, **webp, **jpeg, **png, **gif},
+            "png": {**avif, **webp, **jpeg, **png},
+            "jpeg": {**avif, **webp, **jpeg},
+            "webp": {**avif, **webp},
+        }
+        for fmt, renditions in fallbacks.items():
+            self.assertEqual(Picture(renditions).get_fallback_format(), fmt)
+
+    def test_render_multi_format_sizes(self):
+        renditions = {
+            "format-jpeg|width-10": self.image.get_rendition("format-jpeg|width-10"),
+            "format-jpeg|width-90": self.image.get_rendition("format-jpeg|width-90"),
+            "format-webp|width-10": self.image.get_rendition("format-webp|width-10"),
+            "format-webp|width-90": self.image.get_rendition("format-webp|width-90"),
+        }
+        img = Picture(renditions, {"sizes": "100vw"})
+        filenames = [
+            get_test_image_filename(self.image, "format-jpeg.width-10"),
+            get_test_image_filename(self.image, "format-jpeg.width-90"),
+            get_test_image_filename(self.image, "format-webp.width-10"),
+            get_test_image_filename(self.image, "format-webp.width-90"),
+        ]
+        self.assertHTMLEqual(
+            img.__html__(),
+            f"""
+                <picture>
+                    <source srcset="{filenames[2]} 10w, {filenames[3]} 90w" sizes="100vw" type="image/webp">
+                    <img
+                        alt="Test image"
+                        sizes="100vw"
+                        src="{filenames[0]}"
+                        srcset="{filenames[0]} 10w, {filenames[1]} 90w"
+                        width="10"
+                        height="7"
+                    >
+                </picture>
+            """,
+        )
+
+    def test_render_single_image_same_as_img_tag(self):
+        img = Picture({"width-10": self.rendition_10})
+        self.assertHTMLEqual(
+            img.__html__(), f"<picture>{self.rendition_10.img_tag()}</picture>"
+        )
+
+
+@override_settings(
+    CACHES={"default": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"}}
+)
 class TestRenditions(TestCase):
+    SPECS = ("height-66", "width-100", "width-400")
+
     def setUp(self):
         # Create an image for running tests on
         self.image = Image.objects.create(
@@ -272,6 +560,14 @@ class TestRenditions(TestCase):
         # Check that they are the same object
         self.assertEqual(first_rendition, second_rendition)
 
+    def test_get_with_filter_instance(self):
+        # Get two renditions with the same filter
+        first_rendition = self.image.get_rendition("width-400")
+        second_rendition = self.image.get_rendition(Filter("width-400"))
+
+        # Check that they are the same object
+        self.assertEqual(first_rendition, second_rendition)
+
     def test_prefetched_rendition_found(self):
         # Request a rendition that does not exist yet
         with self.assertNumQueries(5):
@@ -309,8 +605,8 @@ class TestRenditions(TestCase):
 
         # Request a different rendition from this object
         with self.assertNumQueries(4):
-            # The number of queries is fewer than before, because the check for
-            # an existing rendition is skipped
+            # The number of queries is fewer than before, because checks for
+            # an existing rendition (in cache and db) are skipped
             second_rendition = image.get_rendition("height-66")
 
         # The renditions should NOT match
@@ -344,15 +640,55 @@ class TestRenditions(TestCase):
 
         self.assertIs(second_rendition, third_rendition)
 
-    def test_alt_attribute(self):
-        rendition = self.image.get_rendition("width-400")
-        self.assertEqual(rendition.alt, "Test image")
+    def test_get_renditions_with_filter_instance(self):
+        # Get two renditions with the same filter
+        first = list(self.image.get_renditions("width-400").values())
+        second = list(self.image.get_renditions(Filter("width-400")).values())
 
-    def test_full_url(self):
-        ren_img = self.image.get_rendition("original")
-        full_url = ren_img.full_url
-        img_name = ren_img.file.name.split("/")[1]
-        self.assertEqual(full_url, "http://testserver/media/images/{}".format(img_name))
+        # Check that they are the same object
+        self.assertEqual(first[0], second[0])
+
+    def test_get_renditions_key_order(self):
+        # Fetch one of the renditions so it exists before the other two.
+        self.image.get_rendition("width-40")
+        specs = ["width-30", "width-40", "width-50"]
+        renditions_keys = list(self.image.get_renditions(*specs).keys())
+        self.assertEqual(renditions_keys, specs)
+
+    def _test_get_renditions_performance(
+        self,
+        db_queries_expected: int,
+        prefetch_restricted: bool = False,
+        prefetch_all: bool = False,
+    ):
+        queryset = Image.objects.all()
+        if prefetch_all:
+            queryset = queryset.prefetch_related("renditions")
+        elif prefetch_restricted:
+            queryset = queryset.prefetch_renditions(*self.SPECS)
+
+        image = queryset.get(id=self.image.id)
+        with self.assertNumQueries(db_queries_expected):
+            image.get_renditions(*self.SPECS)
+
+    def test_get_renditions_performance_with_rendition_caching_disabled(self):
+        # ATTEMPT 1
+        # 1) An initial lookup for rendition from the DB
+        # 2) A check for clashes before bulk saving new renditions
+        # 3) A bulk_create() to save new renditions
+        self._test_get_renditions_performance(3)
+
+        # ATTEMPT 2
+        # With all renditions already created, we should just see
+        # 1) An initial lookup for rendition from the DB
+        self._test_get_renditions_performance(1)
+
+        # ATTEMPT 3
+        # If the existing renditions are prefetched, no further queries should
+        # be needed, whether that's with prefetch_related("renditions") or
+        # prefetch_renditions()
+        self._test_get_renditions_performance(0, prefetch_all=True)
+        self._test_get_renditions_performance(0, prefetch_restricted=True)
 
     @override_settings(
         CACHES={
@@ -361,30 +697,115 @@ class TestRenditions(TestCase):
             },
         },
     )
-    def test_renditions_cache_backend(self):
-        cache = caches["renditions"]
-        rendition = self.image.get_rendition("width-500")
-        rendition_cache_key = "image-{}-{}-{}".format(
-            rendition.image.id, rendition.focal_point_key, rendition.filter_spec
+    def test_get_renditions_performance_with_rendition_caching_enabled(self):
+        # ATTEMPT 1
+        # 1) An initial lookup for rendition from the DB
+        # 2) A check for clashes before bulk saving new renditions
+        # 3) A bulk_create() to save new renditions
+        self._test_get_renditions_performance(3)
+
+        # ATTEMPT 2
+        # Any renditions created in the first attempt should have
+        # been added to the cache, so a second request should bypass
+        # the database completely
+        self._test_get_renditions_performance(0)
+
+        # ATTEMPT 3
+        # Prefetching renditions should mean that no queries are
+        # required, and no cache hits are made
+        self._test_get_renditions_performance(0, prefetch_all=True)
+
+    def test_create_renditions(self):
+        filter_list = [Filter(spec) for spec in self.SPECS]
+        # When no renditions exist, there should be one query for
+        # 'clash identification', and another for bulk creation
+        with self.assertNumQueries(2):
+            result = self.image.create_renditions(*filter_list)
+
+        # Renditions should match the filters
+        for filter, rendition in result.items():
+            self.assertEqual(filter.spec, rendition.filter_spec)
+
+        # Filter specs should match the filters that were provided as arguments
+        self.assertEqual(
+            {filter.spec for filter in result.keys()},
+            {filter.spec for filter in filter_list},
         )
 
+        # When the renditions already exist, there should be one query
+        # for 'clash identification', but that is all
+        with self.assertNumQueries(1):
+            result = self.image.create_renditions(*filter_list)
+
+        # Renditions should match the filters
+        for filter, rendition in result.items():
+            self.assertEqual(filter.spec, rendition.filter_spec)
+
+        # Filter specs should match the filters that were provided as arguments
+        self.assertEqual(
+            {filter.spec for filter in result.keys()},
+            {filter.spec for filter in filter_list},
+        )
+
+        # Another request should give us an equal result
+        with self.assertNumQueries(1):
+            second_result = self.image.create_renditions(*filter_list)
+        self.assertEqual(result, second_result)
+
+        # When only some renditions exist, we should see the create query
+        # once again
+        self.image.renditions.filter(filter_spec=self.SPECS[0]).delete()
+        with self.assertNumQueries(2):
+            third_result = self.image.create_renditions(*filter_list)
+
+        # Equality check should fail, as newly created rendition has a different pk
+        self.assertNotEqual(third_result, result)
+
+        # But, we should see equality on the keys
+        self.assertEqual(third_result.keys(), result.keys())
+
+    def test_alt_attribute(self):
+        rendition = self.image.get_rendition("width-400")
+        self.assertEqual(rendition.alt, "Test image")
+
+    def test_full_url(self):
+        ren_img = self.image.get_rendition("original")
+        full_url = ren_img.full_url
+        img_name = ren_img.file.name.split("/")[1]
+        self.assertEqual(full_url, f"http://testserver/media/images/{img_name}")
+
+    @override_settings(
+        CACHES={
+            "renditions": {
+                "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
+            },
+        },
+    )
+    def test_renditions_cache(self):
+        cache = Rendition.cache_backend
+        rendition = self.image.get_rendition("width-500")
+        rendition_cache_key = rendition.get_cache_key()
+
         # Check rendition is saved to cache
         self.assertEqual(cache.get(rendition_cache_key), rendition)
 
         # Mark a rendition to check it comes from cache
-        rendition._from_cache = "original"
+        rendition._mark = "original"
         cache.set(rendition_cache_key, rendition)
 
         # Check if get_rendition returns the rendition from cache
         with self.assertNumQueries(0):
             new_rendition = self.image.get_rendition("width-500")
-        self.assertEqual(new_rendition._from_cache, "original")
+        self.assertEqual(new_rendition._mark, "original")
 
         # But, not if the rendition has been prefetched
         fresh_image = Image.objects.prefetch_related("renditions").get(pk=self.image.pk)
         with self.assertNumQueries(0):
             prefetched_rendition = fresh_image.get_rendition("width-500")
-        self.assertFalse(hasattr(prefetched_rendition, "_from_cache"))
+        self.assertFalse(hasattr(prefetched_rendition, "_mark"))
+
+        # Check that the image instance is the same as the retrieved rendition
+        self.assertIs(new_rendition.image, self.image)
 
         # changing the image file should invalidate the cache
         self.image.file = get_test_image_file(colour="green")
@@ -393,7 +814,7 @@ class TestRenditions(TestCase):
         # we're bypassing that here, so have to do it manually
         self.image.renditions.all().delete()
         new_rendition = self.image.get_rendition("width-500")
-        self.assertFalse(hasattr(new_rendition, "_from_cache"))
+        self.assertFalse(hasattr(new_rendition, "_mark"))
 
         # changing it back should also generate a new rendition and not re-use
         # the original one (because that file has now been deleted in the change)
@@ -401,7 +822,30 @@ class TestRenditions(TestCase):
         self.image.save()
         self.image.renditions.all().delete()
         new_rendition = self.image.get_rendition("width-500")
-        self.assertFalse(hasattr(new_rendition, "_from_cache"))
+        self.assertFalse(hasattr(new_rendition, "_mark"))
+
+    def test_prefers_rendition_cache_backend(self):
+        with override_settings(
+            CACHES={
+                "default": {
+                    "BACKEND": "django.core.cache.backends.dummy.DummyCache",
+                },
+                "renditions": {
+                    "BACKEND": "django.core.cache.backends.dummy.DummyCache",
+                },
+            }
+        ):
+            self.assertEqual(Rendition.cache_backend, caches["renditions"])
+
+    def test_uses_default_cache_when_no_renditions_cache(self):
+        with override_settings(
+            CACHES={
+                "default": {
+                    "BACKEND": "django.core.cache.backends.dummy.DummyCache",
+                }
+            }
+        ):
+            self.assertEqual(Rendition.cache_backend, caches["default"])
 
     def test_focal_point(self):
         self.image.focal_point_x = 100
@@ -431,41 +875,47 @@ class TestRenditions(TestCase):
             rendition.background_position_style, "background-position: 50% 50%;"
         )
 
-    def test_custom_rendition_backend_setting(self):
-        """
-        Test the usage of WAGTAILIMAGES_RENDITION_STORAGE setting.
-        """
-        # when setting is not set, instance.get_storage() returns DefaultStorage
-        from django.conf import settings
-
-        bkp = settings
-
+    @override_settings()
+    def test_rendition_storage_setting_absent(self):
+        del settings.WAGTAILIMAGES_RENDITION_STORAGE
         self.assertFalse(hasattr(settings, "WAGTAILIMAGES_RENDITION_STORAGE"))
-        rendition1 = self.image.get_rendition("min-120x120")
-        self.assertIsInstance(rendition1.image.file.storage, DefaultStorage)
+        self.assertEqual(get_rendition_storage(), default_storage)
 
-        # when setting is set to a path
-        setattr(
-            settings,
-            "WAGTAILIMAGES_RENDITION_STORAGE",
-            "wagtail.images.tests.test_models.CustomStorage",
+    @override_settings(
+        WAGTAILIMAGES_RENDITION_STORAGE="wagtail.images.tests.test_models.CustomStorage"
+    )
+    def test_rendition_storage_setting_given_dotted_path(self):
+        self.assertIsInstance(get_rendition_storage(), CustomStorage)
+
+    @override_settings(WAGTAILIMAGES_RENDITION_STORAGE=CustomStorage())
+    def test_rendition_storage_setting_given_storage_instance(self):
+        self.assertEqual(
+            get_rendition_storage(), settings.WAGTAILIMAGES_RENDITION_STORAGE
         )
-        backend = get_rendition_storage()
-        self.assertIsInstance(backend, CustomStorage)
 
-        # when setting is set directly, get_rendition_storage() returns the custom storage backend
-        class CustomStorage2(Storage):
-            def __init__(self):
-                super().__init__()
-
-        setattr(settings, "WAGTAILIMAGES_RENDITION_STORAGE", CustomStorage2())
-        backend = get_rendition_storage()
-        self.assertIsInstance(backend, CustomStorage2)
-
-        # clean up
-        settings = bkp
+    @override_settings(
+        STORAGES={
+            "default": {
+                "BACKEND": "django.core.files.storage.FileSystemStorage",
+            },
+            "staticfiles": {
+                "BACKEND": "django.contrib.staticfiles.storage.StaticFilesStorage",
+            },
+            "custom_storage": {
+                "BACKEND": "wagtail.images.tests.test_models.CustomStorage",
+            },
+        },
+        WAGTAILIMAGES_RENDITION_STORAGE="custom_storage",
+    )
+    def test_rendition_storage_setting_given_storage_alias(self):
+        self.assertEqual(
+            get_rendition_storage(), storages[settings.WAGTAILIMAGES_RENDITION_STORAGE]
+        )
 
 
+@override_settings(
+    CACHES={"default": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"}}
+)
 class TestPrefetchRenditions(TestCase):
     fixtures = ["test.json"]
 
@@ -530,17 +980,16 @@ class TestUsageCount(TestCase):
             file=get_test_image_file(),
         )
 
-    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
     def test_unused_image_usage_count(self):
         self.assertEqual(self.image.get_usage().count(), 0)
 
-    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
     def test_used_image_document_usage_count(self):
-        page = EventPage.objects.get(id=4)
-        event_page_carousel_item = EventPageCarouselItem()
-        event_page_carousel_item.page = page
-        event_page_carousel_item.image = self.image
-        event_page_carousel_item.save()
+        with self.captureOnCommitCallbacks(execute=True):
+            page = EventPage.objects.get(id=4)
+            event_page_carousel_item = EventPageCarouselItem()
+            event_page_carousel_item.page = page
+            event_page_carousel_item.image = self.image
+            event_page_carousel_item.save()
         self.assertEqual(self.image.get_usage().count(), 1)
 
 
@@ -553,21 +1002,21 @@ class TestGetUsage(TestCase):
             file=get_test_image_file(),
         )
 
-    def test_image_get_usage_not_enabled(self):
-        self.assertEqual(list(self.image.get_usage()), [])
-
-    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
     def test_unused_image_get_usage(self):
         self.assertEqual(list(self.image.get_usage()), [])
 
-    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
     def test_used_image_document_get_usage(self):
-        page = EventPage.objects.get(id=4)
-        event_page_carousel_item = EventPageCarouselItem()
-        event_page_carousel_item.page = page
-        event_page_carousel_item.image = self.image
-        event_page_carousel_item.save()
-        self.assertTrue(issubclass(Page, type(self.image.get_usage()[0])))
+        with self.captureOnCommitCallbacks(execute=True):
+            page = EventPage.objects.get(id=4)
+            event_page_carousel_item = EventPageCarouselItem()
+            event_page_carousel_item.page = page
+            event_page_carousel_item.image = self.image
+            event_page_carousel_item.save()
+
+        self.assertIsInstance(self.image.get_usage()[0], tuple)
+        self.assertIsInstance(self.image.get_usage()[0][0], Page)
+        self.assertIsInstance(self.image.get_usage()[0][1], list)
+        self.assertIsInstance(self.image.get_usage()[0][1][0], ReferenceIndex)
 
 
 class TestGetWillowImage(TestCase):
@@ -634,7 +1083,7 @@ class TestIssue573(TestCase):
         image = Image.objects.create(
             title="Test image",
             file=get_test_image_file(
-                "thisisaverylongfilename-abcdefghijklmnopqrstuvwxyz-supercalifragilisticexpialidocious.png"
+                "thisisaverylongfilename-cdefghijklmnopqrstuvwxyzab-supercalifragilisticexpialidocious.png"
             ),
             focal_point_x=1000,
             focal_point_y=1000,
@@ -648,7 +1097,7 @@ class TestIssue573(TestCase):
 
 
 @override_settings(_WAGTAILSEARCH_FORCE_AUTO_UPDATE=["elasticsearch"])
-class TestIssue613(TestCase, WagtailTestUtils):
+class TestIssue613(WagtailTestUtils, TestCase):
     def get_elasticsearch_backend(self):
         from django.conf import settings
 
@@ -780,14 +1229,14 @@ class TestFilenameReduction(TestCase):
         image = Image.objects.create(
             title="Test image",
             file=get_test_image_file(
-                "thisisaverylongfilename-abcdefghijklmnopqrstuvwxyz-supercalifragilisticexpialidocioussuperlong"
+                "thisisaverylongfilename-defghijklmnopqrstuvwxyzabc-supercalifragilisticexpialidocioussuperlong"
             ),
         )
 
         # Saving file will result in infinite loop when bug is present
         image.save()
         self.assertEqual(
-            "original_images/thisisaverylongfilename-abcdefghijklmnopqrstuvwxyz-supercalifragilisticexpiali",
+            "original_images/thisisaverylongfilename-defghijklmnopqrstuvwxyzabc-supercalifragilisticexpiali",
             image.file.name,
         )
 
@@ -797,13 +1246,13 @@ class TestFilenameReduction(TestCase):
         image = Image.objects.create(
             title="Test image",
             file=get_test_image_file(
-                "thisisaverylongfilename-abcdefghijklmnopqrstuvwxyz-supercalifragilisticexpialidocioussuperlong.png"
+                "thisisaverylongfilename-efghijklmnopqrstuvwxyzabcd-supercalifragilisticexpialidocioussuperlong.png"
             ),
         )
 
         image.save()
         self.assertEqual(
-            "original_images/thisisaverylongfilename-abcdefghijklmnopqrstuvwxyz-supercalifragilisticexp.png",
+            "original_images/thisisaverylongfilename-efghijklmnopqrstuvwxyzabcd-supercalifragilisticexp.png",
             image.file.name,
         )
 
