@@ -1,8 +1,6 @@
-from django.urls import include, path, reverse
-from django.utils.html import format_html
-from django.utils.translation import gettext
+from django.urls import include, path, reverse, reverse_lazy
+from django.utils.translation import gettext, ngettext
 from django.utils.translation import gettext_lazy as _
-from django.utils.translation import ngettext
 
 import wagtail.admin.rich_text.editors.draftail.features as draftail_features
 from wagtail import hooks
@@ -26,6 +24,7 @@ from wagtail.images.views.bulk_actions import (
     AddToCollectionBulkAction,
     DeleteBulkAction,
 )
+from wagtail.images.views.chooser import viewset as chooser_viewset
 
 
 @hooks.register("register_admin_urls")
@@ -58,18 +57,6 @@ def register_images_menu_item():
     )
 
 
-@hooks.register("insert_editor_js")
-def editor_js():
-    return format_html(
-        """
-        <script>
-            window.chooserUrls.imageChooser = '{0}';
-        </script>
-        """,
-        reverse("wagtailimages:chooser"),
-    )
-
-
 @hooks.register("register_rich_text_features")
 def register_image_feature(features):
     # define a handler for converting <embed embedtype="image"> tags into frontend HTML
@@ -94,9 +81,12 @@ def register_image_feature(features):
                 # Keep only the attributes Wagtail needs.
                 "attributes": ["id", "src", "alt", "format"],
                 # Keep only images which are from Wagtail.
-                "whitelist": {
+                "allowlist": {
                     "id": True,
                 },
+                "chooserUrls": {
+                    "imageChooser": reverse_lazy("wagtailimages_chooser:choose")
+                },
             },
             js=[
                 "wagtailimages/js/image-chooser-modal.js",
@@ -126,6 +116,7 @@ def register_image_operations():
         ("scale", image_operations.ScaleOperation),
         ("jpegquality", image_operations.JPEGQualityOperation),
         ("webpquality", image_operations.WebPQualityOperation),
+        ("avifquality", image_operations.AvifQualityOperation),
         ("format", image_operations.FormatOperation),
         ("bgcolor", image_operations.BackgroundColorOperation),
     ]
@@ -139,7 +130,9 @@ class ImagesSummaryItem(SummaryItem):
         site_name = get_site_for_user(self.request.user)["site_name"]
 
         return {
-            "total_images": get_image_model().objects.count(),
+            "total_images": permission_policy.instances_user_has_any_permission_for(
+                self.request.user, {"add", "change", "delete", "choose"}
+            ).count(),
             "site_name": site_name,
         }
 
@@ -200,3 +193,8 @@ register_admin_url_finder(get_image_model(), ImageAdminURLFinder)
 
 for action_class in [AddTagsBulkAction, AddToCollectionBulkAction, DeleteBulkAction]:
     hooks.register("register_bulk_action", action_class)
+
+
+@hooks.register("register_admin_viewset")
+def register_image_chooser_viewset():
+    return chooser_viewset
