@@ -63,7 +63,7 @@ def get_site_for_hostname(hostname, port):
 
 class SiteManager(models.Manager):
     def get_queryset(self):
-        return super(SiteManager, self).get_queryset().order_by(Lower("hostname"))
+        return super().get_queryset().order_by(Lower("hostname"))
 
     def get_by_natural_key(self, hostname, port):
         return self.get(hostname=hostname, port=port)
@@ -71,6 +71,10 @@ class SiteManager(models.Manager):
 
 SiteRootPath = namedtuple("SiteRootPath", "site_id root_path root_url language_code")
 
+SITE_ROOT_PATHS_CACHE_KEY = "wagtail_site_root_paths"
+# Increase the cache version whenever the structure SiteRootPath tuple changes
+SITE_ROOT_PATHS_CACHE_VERSION = 2
+
 
 class Site(models.Model):
     hostname = models.CharField(
@@ -125,6 +129,9 @@ class Site(models.Model):
                 + (default_suffix if self.is_default_site else "")
             )
 
+    def clean(self):
+        self.hostname = self.hostname.lower()
+
     @staticmethod
     def find_for_request(request):
         """
@@ -153,7 +160,8 @@ class Site(models.Model):
 
     @staticmethod
     def _find_for_request(request):
-        hostname = split_domain_port(request.get_host())[0]
+        # Use `_get_raw_host` to avoid ALLOWED_HOSTS checks
+        hostname = split_domain_port(request._get_raw_host())[0]
         port = request.get_port()
         site = None
         try:
@@ -199,23 +207,21 @@ class Site(models.Model):
     def get_site_root_paths():
         """
         Return a list of `SiteRootPath` instances, most specific path
-        first - used to translate url_paths into actual URLs with hostnames
+        first - used to translate url_paths into actual URLs with hostnames.
 
         Each root path is an instance of the `SiteRootPath` named tuple,
         and have the following attributes:
 
-        - `site_id` - The ID of the Site record
-        - `root_path` - The internal URL path of the site's home page (for example '/home/')
-        - `root_url` - The scheme/domain name of the site (for example 'https://www.example.com/')
-        - `language_code` - The language code of the site (for example 'en')
+        - ``site_id`` - The ID of the Site record
+        - ``root_path`` - The internal URL path of the site's home page (for example '/home/')
+        - ``root_url`` - The scheme/domain name of the site (for example 'https://www.example.com/')
+        - ``language_code`` - The language code of the site (for example 'en')
         """
-        result = cache.get("wagtail_site_root_paths")
+        result = cache.get(
+            SITE_ROOT_PATHS_CACHE_KEY, version=SITE_ROOT_PATHS_CACHE_VERSION
+        )
 
-        # Wagtail 2.11 changed the way site root paths were stored. This can cause an upgraded 2.11
-        # site to break when loading cached site root paths that were cached with 2.10.2 or older
-        # versions of Wagtail. The line below checks if the any of the cached site urls is consistent
-        # with an older version of Wagtail and invalidates the cache.
-        if result is None or any(len(site_record) == 3 for site_record in result):
+        if result is None:
             result = []
 
             for site in Site.objects.select_related(
@@ -245,6 +251,20 @@ class Site(models.Model):
                         )
                     )
 
-            cache.set("wagtail_site_root_paths", result, 3600)
+            cache.set(
+                SITE_ROOT_PATHS_CACHE_KEY,
+                result,
+                3600,
+                version=SITE_ROOT_PATHS_CACHE_VERSION,
+            )
+
+        else:
+            # Convert the cache result to a list of SiteRootPath tuples, as some
+            # cache backends (e.g. Redis) don't support named tuples.
+            result = [SiteRootPath(*result) for result in result]
 
         return result
+
+    @staticmethod
+    def clear_site_root_paths_cache():
+        cache.delete(SITE_ROOT_PATHS_CACHE_KEY, version=SITE_ROOT_PATHS_CACHE_VERSION)
