@@ -14,19 +14,22 @@ class InvalidSearchBackendError(ImproperlyConfigured):
 
 
 def get_search_backend_config():
-    search_backends = getattr(settings, 'WAGTAILSEARCH_BACKENDS', {})
+    search_backends = getattr(settings, "WAGTAILSEARCH_BACKENDS", {})
 
     # Make sure the default backend is always defined
-    search_backends.setdefault('default', {
-        'BACKEND': 'wagtail.search.backends.db',
-    })
+    search_backends.setdefault(
+        "default",
+        {
+            "BACKEND": "wagtail.search.backends.database",
+        },
+    )
 
     return search_backends
 
 
 def import_backend(dotted_path):
     """
-    Theres two formats for the dotted_path.
+    There's two formats for the dotted_path.
     One with the backend class (old) and one without (new)
     eg:
       old: wagtail.search.backends.elasticsearch.ElasticsearchSearchBackend
@@ -47,7 +50,7 @@ def import_backend(dotted_path):
             raise ImportError from e
 
 
-def get_search_backend(backend='default', **kwargs):
+def get_search_backend(backend="default", **kwargs):
     search_backends = get_search_backend_config()
 
     # Try to find the backend
@@ -59,34 +62,32 @@ def get_search_backend(backend='default', **kwargs):
             # Trying to import the given backend, in case it's a dotted path
             import_backend(backend)
         except ImportError as e:
-            raise InvalidSearchBackendError("Could not find backend '%s': %s" % (
-                backend, e))
+            raise InvalidSearchBackendError(f"Could not find backend '{backend}': {e}")
         params = kwargs
     else:
         # Backend is a conf entry
         params = conf.copy()
         params.update(kwargs)
-        backend = params.pop('BACKEND')
+        backend = params.pop("BACKEND")
 
     # Try to import the backend
     try:
         backend_cls = import_backend(backend)
     except ImportError as e:
-        raise InvalidSearchBackendError("Could not find backend '%s': %s" % (
-            backend, e))
+        raise InvalidSearchBackendError(f"Could not find backend '{backend}': {e}")
 
     # Create backend
     return backend_cls(params)
 
 
 def _backend_requires_auto_update(backend_name, params):
-    if params.get('AUTO_UPDATE', True):
+    if params.get("AUTO_UPDATE", True):
         return True
 
     # _WAGTAILSEARCH_FORCE_AUTO_UPDATE is only used by Wagtail tests. It allows
     # us to test AUTO_UPDATE behaviour against Elasticsearch without having to
     # have AUTO_UPDATE enabed for every test.
-    force_auto_update = getattr(settings, '_WAGTAILSEARCH_FORCE_AUTO_UPDATE', [])
+    force_auto_update = getattr(settings, "_WAGTAILSEARCH_FORCE_AUTO_UPDATE", [])
     if backend_name in force_auto_update:
         return True
 
@@ -104,4 +105,9 @@ def get_search_backends_with_name(with_auto_update=False):
 
 def get_search_backends(with_auto_update=False):
     # For backwards compatibility
-    return (backend for _, backend in get_search_backends_with_name(with_auto_update=with_auto_update))
+    return (
+        backend
+        for _, backend in get_search_backends_with_name(
+            with_auto_update=with_auto_update
+        )
+    )
