@@ -1,5 +1,3 @@
-from __future__ import absolute_import, unicode_literals
-
 #
 # Base classes
 #
@@ -48,7 +46,21 @@ class Phrase(SearchQuery):
         self.query_string = query_string
 
     def __repr__(self):
-        return "<Phrase {}>".format(repr(self.query_string))
+        return f"<Phrase {repr(self.query_string)}>"
+
+
+class Fuzzy(SearchQuery):
+    OPERATORS = ["and", "or"]
+    DEFAULT_OPERATOR = "or"
+
+    def __init__(self, query_string: str, operator: str = DEFAULT_OPERATOR):
+        self.query_string = query_string
+        self.operator = operator.lower()
+        if self.operator not in self.OPERATORS:
+            raise ValueError("`operator` must be either 'or' or 'and'.")
+
+    def __repr__(self):
+        return f"<Fuzzy {repr(self.query_string)} operator={repr(self.operator)}>"
 
 
 class MatchAll(SearchQuery):
@@ -62,7 +74,7 @@ class Boost(SearchQuery):
         self.boost = boost
 
     def __repr__(self):
-        return "<Boost {} boost={}>".format(repr(self.subquery), repr(self.boost))
+        return f"<Boost {repr(self.subquery)} boost={repr(self.boost)}>"
 
 
 #
@@ -95,7 +107,7 @@ class Not(SearchQuery):
         self.subquery = subquery
 
     def __repr__(self):
-        return "<Not {}>".format(repr(self.subquery))
+        return f"<Not {repr(self.subquery)}>"
 
 
 MATCH_ALL = MatchAll()
