@@ -1,5 +1,14 @@
 from django.dispatch import Signal
 
+# Generic object signals
+
+# provides args: instance, revision
+published = Signal()
+
+# provides args: instance
+unpublished = Signal()
+
+
 # Page signals
 
 # provides args: instance, revision
@@ -55,3 +64,7 @@ task_cancelled = Signal()
 # Required as a workaround for https://code.djangoproject.com/ticket/6870
 # provides args: sender, instance
 pre_validate_delete = Signal()
+
+# Translation signals
+# provides args: sender, source_obj, target_obj
+copy_for_translation_done = Signal()
