@@ -1,10 +1,11 @@
 import datetime
 
-from django.test import TestCase
+from django.test import TestCase, override_settings
 from django.urls import reverse
 from django.utils import timezone
 from freezegun import freeze_time
 
+from wagtail.admin.staticfiles import versioned_static
 from wagtail.admin.views.generic.preview import PreviewOnEdit
 from wagtail.test.testapp.models import (
     EventCategory,
@@ -14,9 +15,10 @@ from wagtail.test.testapp.models import (
     RevisableModel,
 )
 from wagtail.test.utils import WagtailTestUtils
+from wagtail.utils.deprecation import RemovedInWagtail70Warning
 
 
-class TestPreview(TestCase, WagtailTestUtils):
+class TestPreview(WagtailTestUtils, TestCase):
     def setUp(self):
         self.user = self.login()
 
@@ -50,7 +52,7 @@ class TestPreview(TestCase, WagtailTestUtils):
         self.assertTemplateUsed(response, "wagtailadmin/generic/preview_error.html")
         self.assertContains(
             response,
-            "<title>Wagtail - Preview not available</title>",
+            "<title>Preview not available - Wagtail</title>",
             html=True,
         )
         self.assertContains(
@@ -58,6 +60,7 @@ class TestPreview(TestCase, WagtailTestUtils):
             '<h1 class="preview-error__title">Preview not available</h1>',
             html=True,
         )
+        self.assertNotContains(response, versioned_static("wagtailadmin/js/icons.js"))
 
     def test_preview_on_create_with_invalid_data(self):
         self.assertNotIn(self.session_key_prefix, self.client.session)
@@ -81,7 +84,7 @@ class TestPreview(TestCase, WagtailTestUtils):
         self.assertTemplateUsed(response, "wagtailadmin/generic/preview_error.html")
         self.assertContains(
             response,
-            "<title>Wagtail - Preview not available</title>",
+            "<title>Preview not available - Wagtail</title>",
             html=True,
         )
         self.assertContains(
@@ -89,6 +92,7 @@ class TestPreview(TestCase, WagtailTestUtils):
             '<h1 class="preview-error__title">Preview not available</h1>',
             html=True,
         )
+        self.assertNotContains(response, versioned_static("wagtailadmin/js/icons.js"))
 
     def test_preview_on_create_with_m2m_field(self):
         response = self.client.post(self.preview_on_add_url, self.post_data)
@@ -220,7 +224,7 @@ class TestPreview(TestCase, WagtailTestUtils):
         self.assertTemplateUsed(response, "wagtailadmin/generic/preview_error.html")
         self.assertContains(
             response,
-            "<title>Wagtail - Preview not available</title>",
+            "<title>Preview not available - Wagtail</title>",
             html=True,
         )
         self.assertContains(
@@ -228,6 +232,7 @@ class TestPreview(TestCase, WagtailTestUtils):
             '<h1 class="preview-error__title">Preview not available</h1>',
             html=True,
         )
+        self.assertNotContains(response, versioned_static("wagtailadmin/js/icons.js"))
 
     def test_preview_on_edit_clear_preview_data(self):
         # Set a fake preview session data for the page
@@ -250,7 +255,7 @@ class TestPreview(TestCase, WagtailTestUtils):
         self.assertTemplateUsed(response, "wagtailadmin/generic/preview_error.html")
         self.assertContains(
             response,
-            "<title>Wagtail - Preview not available</title>",
+            "<title>Preview not available - Wagtail</title>",
             html=True,
         )
         self.assertContains(
@@ -258,6 +263,7 @@ class TestPreview(TestCase, WagtailTestUtils):
             '<h1 class="preview-error__title">Preview not available</h1>',
             html=True,
         )
+        self.assertNotContains(response, versioned_static("wagtailadmin/js/icons.js"))
 
     def test_preview_revision(self):
         snippet = MultiPreviewModesModel.objects.create(text="Multiple modes")
@@ -276,7 +282,7 @@ class TestPreview(TestCase, WagtailTestUtils):
         self.assertContains(response, "Multiple modes (Alternate Preview)")
 
 
-class TestEnablePreview(TestCase, WagtailTestUtils):
+class TestEnablePreview(WagtailTestUtils, TestCase):
     def setUp(self):
         self.user = self.login()
         self.single = PreviewableModel.objects.create(text="Single preview mode")
@@ -291,32 +297,43 @@ class TestEnablePreview(TestCase, WagtailTestUtils):
     def test_show_preview_panel_on_create_with_single_mode(self):
         create_url = self.get_url(self.single, "add")
         preview_url = self.get_url(self.single, "preview_on_add")
-        iframe_url = preview_url + "?in_preview_panel=true&mode="
+        new_tab_url = preview_url + "?mode="
         response = self.client.get(create_url)
 
         self.assertEqual(response.status_code, 200)
 
-        # Should show the preview panel
-        self.assertContains(response, 'data-side-panel-toggle="preview"')
-        self.assertContains(response, 'data-side-panel="preview"')
-        self.assertContains(response, 'data-action="%s"' % preview_url)
+        # Should have the preview side panel toggle button
+        soup = self.get_soup(response.content)
+        self.assertIsNotNone(soup.select_one('[data-side-panel="preview"]'))
+        toggle_button = soup.find("button", {"data-side-panel-toggle": "preview"})
+        self.assertIsNotNone(toggle_button)
+        self.assertEqual("w-tooltip w-kbd", toggle_button["data-controller"])
+        self.assertEqual("mod+p", toggle_button["data-w-kbd-key-value"])
+
+        # Should set the preview URL value on the controller
+        controller = soup.select_one('[data-controller="w-preview"]')
+        self.assertIsNotNone(controller)
+        self.assertEqual(controller.get("data-w-preview-url-value"), preview_url)
 
         # Should show the iframe
-        self.assertContains(
-            response,
-            f'<iframe title="Preview" class="preview-panel__iframe" data-preview-iframe src="{iframe_url}" aria-describedby="preview-panel-error-banner">',
-        )
+        iframe = controller.select_one("#w-preview-iframe")
+        self.assertIsNotNone(iframe)
+        self.assertEqual(iframe.get("data-w-preview-target"), "iframe")
+
+        # Should show the new tab button with the default mode set
+        new_tab_button = controller.select_one('a[data-w-preview-target="newTab"]')
+        self.assertIsNotNone(new_tab_button)
+        self.assertEqual(new_tab_button["href"], new_tab_url)
+        self.assertEqual(new_tab_button["target"], "_blank")
 
         # Should not show the preview mode selection
-        self.assertNotContains(
-            response,
-            '<select id="id_preview_mode" name="preview_mode" class="preview-panel__mode-select" data-preview-mode-select>',
-        )
+        mode_select = controller.select_one('[data-w-preview-target="mode"]')
+        self.assertIsNone(mode_select)
 
     def test_show_preview_panel_on_create_with_multiple_modes(self):
         create_url = self.get_url(self.multiple, "add")
         preview_url = self.get_url(self.multiple, "preview_on_add")
-        iframe_url = preview_url + "?in_preview_panel=true&mode=alt%231"
+        new_tab_url = preview_url + "?mode=alt%231"
         response = self.client.get(create_url)
 
         self.assertEqual(response.status_code, 200)
@@ -324,59 +341,80 @@ class TestEnablePreview(TestCase, WagtailTestUtils):
         # Should show the preview panel
         self.assertContains(response, 'data-side-panel-toggle="preview"')
         self.assertContains(response, 'data-side-panel="preview"')
-        self.assertContains(response, 'data-action="%s"' % preview_url)
 
-        # Should show the iframe with the default mode set and correctly quoted
-        self.assertContains(
-            response,
-            f'<iframe title="Preview" class="preview-panel__iframe" data-preview-iframe src="{iframe_url}" aria-describedby="preview-panel-error-banner">',
-        )
+        # Should set the preview URL value on the controller
+        soup = self.get_soup(response.content)
+        controller = soup.select_one('[data-controller="w-preview"]')
+        self.assertIsNotNone(controller)
+        self.assertEqual(controller.get("data-w-preview-url-value"), preview_url)
 
-        # should show the preview mode selection
-        self.assertContains(
-            response,
-            '<select id="id_preview_mode" name="preview_mode" class="preview-panel__mode-select" data-preview-mode-select>',
-        )
-        self.assertContains(response, '<option value="">Normal</option>')
+        # Should show the iframe
+        iframe = controller.select_one("#w-preview-iframe")
+        self.assertIsNotNone(iframe)
+        self.assertEqual(iframe.get("data-w-preview-target"), "iframe")
 
-        # Should respect the default_preview_mode
-        self.assertContains(
-            response, '<option value="alt#1" selected>Alternate</option>'
-        )
+        # Should show the new tab button with the default mode set and correctly quoted
+        new_tab_button = controller.select_one('a[data-w-preview-target="newTab"]')
+        self.assertIsNotNone(new_tab_button)
+        self.assertEqual(new_tab_button["href"], new_tab_url)
+        self.assertEqual(new_tab_button["target"], "_blank")
+
+        # should show the preview mode selection with the default mode selected
+        mode_select = controller.select_one('[data-w-preview-target="mode"]')
+        self.assertIsNotNone(mode_select)
+        self.assertEqual(mode_select["id"], "id_preview_mode")
+        default_option = mode_select.select_one('option[value="alt#1"]')
+        self.assertIsNotNone(default_option)
+        self.assertIsNotNone(default_option.get("selected"))
+        other_option = mode_select.select_one('option[value=""]')
+        self.assertIsNotNone(other_option)
+        self.assertEqual(other_option.text.strip(), "Normal")
+        self.assertIsNone(other_option.get("selected"))
 
     def test_show_preview_panel_on_edit_with_single_mode(self):
         edit_url = self.get_url(self.single, "edit", args=(self.single.pk,))
         preview_url = self.get_url(
             self.single, "preview_on_edit", args=(self.multiple.pk,)
         )
-        iframe_url = preview_url + "?in_preview_panel=true&mode="
+        new_tab_url = preview_url + "?mode="
         response = self.client.get(edit_url)
 
         self.assertEqual(response.status_code, 200)
 
-        # Should show the preview panel
-        self.assertContains(response, 'data-side-panel-toggle="preview"')
-        self.assertContains(response, 'data-side-panel="preview"')
-        self.assertContains(response, 'data-action="%s"' % preview_url)
+        # Should have the preview side panel toggle button
+        soup = self.get_soup(response.content)
+        self.assertIsNotNone(soup.select_one('[data-side-panel="preview"]'))
+        toggle_button = soup.find("button", {"data-side-panel-toggle": "preview"})
+        self.assertIsNotNone(toggle_button)
+        self.assertEqual("w-tooltip w-kbd", toggle_button["data-controller"])
+        self.assertEqual("mod+p", toggle_button["data-w-kbd-key-value"])
+
+        # Should set the preview URL value on the controller
+        controller = soup.select_one('[data-controller="w-preview"]')
+        self.assertIsNotNone(controller)
+        self.assertEqual(controller.get("data-w-preview-url-value"), preview_url)
 
         # Should show the iframe
-        self.assertContains(
-            response,
-            f'<iframe title="Preview" class="preview-panel__iframe" data-preview-iframe src="{iframe_url}" aria-describedby="preview-panel-error-banner">',
-        )
+        iframe = controller.select_one("#w-preview-iframe")
+        self.assertIsNotNone(iframe)
+        self.assertEqual(iframe.get("data-w-preview-target"), "iframe")
+
+        # Should show the new tab button with the default mode set
+        new_tab_button = controller.select_one('a[data-w-preview-target="newTab"]')
+        self.assertIsNotNone(new_tab_button)
+        self.assertEqual(new_tab_button["href"], new_tab_url)
+        self.assertEqual(new_tab_button["target"], "_blank")
 
         # Should not show the preview mode selection
-        self.assertNotContains(
-            response,
-            '<select id="id_preview_mode" name="preview_mode" class="preview-panel__mode-select" data-preview-mode-select>',
-        )
+        mode_select = controller.select_one('[data-w-preview-target="mode"]')
+        self.assertIsNone(mode_select)
 
     def test_show_preview_panel_on_edit_with_multiple_modes(self):
         edit_url = self.get_url(self.multiple, "edit", args=(self.multiple.pk,))
         preview_url = self.get_url(
             self.multiple, "preview_on_edit", args=(self.multiple.pk,)
         )
-        iframe_url = preview_url + "?in_preview_panel=true&mode=alt%231"
+        new_tab_url = preview_url + "?mode=alt%231"
         response = self.client.get(edit_url)
 
         self.assertEqual(response.status_code, 200)
@@ -384,25 +422,140 @@ class TestEnablePreview(TestCase, WagtailTestUtils):
         # Should show the preview panel
         self.assertContains(response, 'data-side-panel-toggle="preview"')
         self.assertContains(response, 'data-side-panel="preview"')
-        self.assertContains(response, 'data-action="%s"' % preview_url)
 
-        # Should show the iframe with the default mode set and correctly quoted
-        self.assertContains(
-            response,
-            f'<iframe title="Preview" class="preview-panel__iframe" data-preview-iframe src="{iframe_url}" aria-describedby="preview-panel-error-banner">',
-        )
+        # Should set the preview URL value on the controller
+        soup = self.get_soup(response.content)
+        controller = soup.select_one('[data-controller="w-preview"]')
+        self.assertIsNotNone(controller)
+        self.assertEqual(controller.get("data-w-preview-url-value"), preview_url)
 
-        # should show the preview mode selection
-        self.assertContains(
-            response,
-            '<select id="id_preview_mode" name="preview_mode" class="preview-panel__mode-select" data-preview-mode-select>',
-        )
-        self.assertContains(response, '<option value="">Normal</option>')
+        # Should have a default interval of 500ms and should render the hidden spinner
+        interval_value = controller.get("data-w-preview-auto-update-interval-value")
+        self.assertEqual(interval_value, "500")
+        spinner = controller.select_one('[data-w-preview-target="spinner"]')
+        self.assertIsNotNone(spinner)
+        self.assertIsNotNone(spinner.get("hidden"))
+        self.assertIsNotNone(spinner.select_one("svg.icon-spinner"))
 
-        # Should respect the default_preview_mode
-        self.assertContains(
-            response, '<option value="alt#1" selected>Alternate</option>'
+        # Should not render any buttons (the refresh button in particular)
+        refresh_button = controller.select_one("button")
+        self.assertIsNone(refresh_button)
+
+        # Should show the iframe
+        iframe = controller.select_one("#w-preview-iframe")
+        self.assertIsNotNone(iframe)
+        self.assertEqual(iframe.get("data-w-preview-target"), "iframe")
+
+        # Should show the new tab button with the default mode set and correctly quoted
+        new_tab_button = controller.select_one('a[data-w-preview-target="newTab"]')
+        self.assertIsNotNone(new_tab_button)
+        self.assertEqual(new_tab_button["href"], new_tab_url)
+        self.assertEqual(new_tab_button["target"], "_blank")
+
+        # should show the preview mode selection with the default mode selected
+        mode_select = controller.select_one('[data-w-preview-target="mode"]')
+        self.assertIsNotNone(mode_select)
+        self.assertEqual(mode_select["id"], "id_preview_mode")
+        default_option = mode_select.select_one('option[value="alt#1"]')
+        self.assertIsNotNone(default_option)
+        self.assertIsNotNone(default_option.get("selected"))
+        other_option = mode_select.select_one('option[value=""]')
+        self.assertIsNotNone(other_option)
+        self.assertEqual(other_option.text.strip(), "Normal")
+        self.assertIsNone(other_option.get("selected"))
+
+    @override_settings(WAGTAIL_AUTO_UPDATE_PREVIEW_INTERVAL=12345)
+    def test_custom_auto_update_interval(self):
+        edit_url = self.get_url(self.single, "edit", args=(self.single.pk,))
+        preview_url = self.get_url(
+            self.single, "preview_on_edit", args=(self.multiple.pk,)
         )
+        response = self.client.get(edit_url)
+
+        self.assertEqual(response.status_code, 200)
+
+        soup = self.get_soup(response.content)
+
+        # Should set the custom interval value on the controller
+        controller = soup.select_one('[data-controller="w-preview"]')
+        self.assertIsNotNone(controller)
+        self.assertEqual(controller.get("data-w-preview-url-value"), preview_url)
+        interval_value = controller.get("data-w-preview-auto-update-interval-value")
+        self.assertEqual(interval_value, "12345")
+
+        # Should render the spinner
+        spinner = controller.select_one('[data-w-preview-target="spinner"]')
+        self.assertIsNotNone(spinner)
+        self.assertIsNotNone(spinner.get("hidden"))
+        self.assertIsNotNone(spinner.select_one("svg.icon-spinner"))
+
+        # Should not render any buttons (the refresh button in particular)
+        refresh_button = controller.select_one("button")
+        self.assertIsNone(refresh_button)
+
+    @override_settings(WAGTAIL_AUTO_UPDATE_PREVIEW_INTERVAL=0)
+    def test_disable_auto_update_using_zero_interval(self):
+        edit_url = self.get_url(self.single, "edit", args=(self.single.pk,))
+        preview_url = self.get_url(
+            self.single, "preview_on_edit", args=(self.multiple.pk,)
+        )
+        response = self.client.get(edit_url)
+
+        self.assertEqual(response.status_code, 200)
+
+        soup = self.get_soup(response.content)
+
+        # Should set the interval value on the controller
+        controller = soup.select_one('[data-controller="w-preview"]')
+        self.assertIsNotNone(controller)
+        self.assertEqual(controller.get("data-w-preview-url-value"), preview_url)
+        interval_value = controller.get("data-w-preview-auto-update-interval-value")
+        self.assertEqual(interval_value, "0")
+
+        # Should not render the spinner
+        spinner = controller.select_one('[data-w-preview-target="spinner"]')
+        self.assertIsNone(spinner)
+
+        # Should render the refresh button with the w-progress controller
+        refresh_button = controller.select_one("button")
+        self.assertIsNotNone(refresh_button)
+        self.assertEqual(refresh_button.get("data-controller"), "w-progress")
+        self.assertEqual(refresh_button.text.strip(), "Refresh")
+
+    @override_settings(WAGTAIL_AUTO_UPDATE_PREVIEW=False)
+    def test_disable_auto_update_using_deprecated_setting(self):
+        edit_url = self.get_url(self.single, "edit", args=(self.single.pk,))
+        preview_url = self.get_url(
+            self.single, "preview_on_edit", args=(self.multiple.pk,)
+        )
+        with self.assertWarnsMessage(
+            RemovedInWagtail70Warning,
+            "`WAGTAIL_AUTO_UPDATE_PREVIEW` is deprecated. "
+            "Set `WAGTAIL_AUTO_UPDATE_PREVIEW_INTERVAL = 0` to disable auto-update "
+            "for previews.",
+        ):
+            response = self.client.get(edit_url)
+
+        self.assertEqual(response.status_code, 200)
+
+        soup = self.get_soup(response.content)
+
+        # Should set the interval value on the controller
+        controller = soup.select_one('[data-controller="w-preview"]')
+        self.assertIsNotNone(controller)
+        self.assertEqual(controller.get("data-w-preview-url-value"), preview_url)
+        interval_value = controller.get("data-w-preview-auto-update-interval-value")
+        self.assertEqual(interval_value, "0")
+
+        # Should not render the spinner
+        spinner = controller.select_one('[data-w-preview-target="spinner"]')
+        self.assertIsNone(spinner)
+
+        # Should render the refresh button with the w-progress controller
+        refresh_button = controller.select_one("button")
+        self.assertIsNotNone(refresh_button)
+        self.assertEqual(refresh_button.get("data-controller"), "w-progress")
+        self.assertEqual(refresh_button.text.strip(), "Refresh")
 
     def test_show_preview_on_revisions_list(self):
         latest_revision = self.multiple.save_revision(log_action=True)
@@ -418,7 +571,7 @@ class TestEnablePreview(TestCase, WagtailTestUtils):
         self.assertContains(response, preview_url)
 
 
-class TestDisablePreviewWithEmptyModes(TestCase, WagtailTestUtils):
+class TestDisablePreviewWithEmptyModes(WagtailTestUtils, TestCase):
     """
     Preview can be disabled by setting preview_modes to an empty list.
     """
@@ -441,7 +594,8 @@ class TestDisablePreviewWithEmptyModes(TestCase, WagtailTestUtils):
         preview_url = self.get_url("preview_on_add")
         self.assertNotContains(response, 'data-side-panel-toggle="preview"')
         self.assertNotContains(response, 'data-side-panel="preview"')
-        self.assertNotContains(response, 'data-action="%s"' % preview_url)
+        self.assertNotContains(response, 'data-controller="w-preview"')
+        self.assertNotContains(response, preview_url)
 
     def test_disable_preview_on_edit(self):
         response = self.client.get(self.get_url("edit", args=(self.snippet.pk,)))
@@ -450,7 +604,8 @@ class TestDisablePreviewWithEmptyModes(TestCase, WagtailTestUtils):
         preview_url = self.get_url("preview_on_edit", args=(self.snippet.pk,))
         self.assertNotContains(response, 'data-side-panel-toggle="preview"')
         self.assertNotContains(response, 'data-side-panel="preview"')
-        self.assertNotContains(response, 'data-action="%s"' % preview_url)
+        self.assertNotContains(response, 'data-controller="w-preview"')
+        self.assertNotContains(response, preview_url)
 
     def test_disable_preview_on_revisions_list(self):
         latest_revision = self.snippet.save_revision(log_action=True)
@@ -459,9 +614,14 @@ class TestDisablePreviewWithEmptyModes(TestCase, WagtailTestUtils):
         preview_url = self.get_url(
             "revisions_view", args=(self.snippet.pk, latest_revision.id)
         )
-        self.assertNotContains(response, "Preview")
+
         self.assertNotContains(response, preview_url)
 
+        soup = self.get_soup(response.content)
+
+        preview_link = soup.find("a", {"href": preview_url})
+        self.assertIsNone(preview_link)
+
 
 class TestDisablePreviewWithoutMixin(TestDisablePreviewWithEmptyModes):
     """
