@@ -3,6 +3,7 @@ from django.shortcuts import resolve_url
 from django.template.defaulttags import token_kwargs
 from django.template.loader import render_to_string
 from django.utils.encoding import force_str
+from django.utils.functional import Promise
 from django.utils.html import conditional_escape
 
 from wagtail import VERSION, __version__
@@ -23,25 +24,28 @@ def pageurl(context, page, fallback=None):
     if page is None and fallback:
         return resolve_url(fallback)
 
-    if not hasattr(page, "relative_url"):
+    if not isinstance(page, Page):
         raise ValueError("pageurl tag expected a Page object, got %r" % page)
 
-    try:
-        site = Site.find_for_request(context["request"])
-        current_site = site
-    except KeyError:
-        # request not available in the current context; fall back on page.url
-        return page.url
+    return page.get_url(request=context.get("request"))
 
-    if current_site is None:
-        # request does not correspond to a recognised site; fall back on page.url
-        return page.url
 
-    # Pass page.relative_url the request object, which may contain a cached copy of
-    # Site.get_site_root_paths()
-    # This avoids page.relative_url having to make a database/cache fetch for this list
-    # each time it's called.
-    return page.relative_url(current_site, request=context.get("request"))
+@register.simple_tag(takes_context=True)
+def fullpageurl(context, page, fallback=None):
+    """
+    Outputs a page's absolute URL (http://example.com/foo/bar/)
+    If kwargs contains a fallback view name and page is None, the fallback view url will be returned.
+    """
+    if page is None and fallback:
+        fallback_url = resolve_url(fallback)
+        if fallback_url and "request" in context and fallback_url[0] == "/":
+            fallback_url = context["request"].build_absolute_uri(fallback_url)
+        return fallback_url
+
+    if not isinstance(page, Page):
+        raise ValueError("fullpageurl tag expected a Page object, got %r" % page)
+
+    return page.get_full_url(request=context.get("request"))
 
 
 @register.simple_tag(takes_context=True)
@@ -93,6 +97,22 @@ def wagtail_release_notes_path():
     return "%s.html" % get_main_version(VERSION)
 
 
+@register.simple_tag
+def wagtail_feature_release_whats_new_link():
+    major, minor, patch, release, num = VERSION
+    if release == "final":
+        return f"https://guide.wagtail.org/en-{major}.{minor}.x/releases/new-in-wagtail-{major}-{minor}/"
+    return "https://guide.wagtail.org/en-latest/releases/latest/"
+
+
+@register.simple_tag
+def wagtail_feature_release_editor_guide_link():
+    major, minor, patch, release, num = VERSION
+    if release == "final":
+        return f"https://guide.wagtail.org/en-{major}.{minor}.x/"
+    return "https://guide.wagtail.org/"
+
+
 @register.filter
 def richtext(value):
     if isinstance(value, RichText):
@@ -101,6 +121,8 @@ def richtext(value):
     elif value is None:
         html = ""
     else:
+        if isinstance(value, Promise):
+            value = str(value)
         if isinstance(value, str):
             html = expand_db_html(value)
         else:
@@ -175,7 +197,7 @@ def include_block(parser, token):
 
     if tokens:
         raise template.TemplateSyntaxError(
-            "Unexpected argument to %r tag: %r" % (tag_name, tokens[0])
+            f"Unexpected argument to {tag_name!r} tag: {tokens[0]!r}"
         )
 
     return IncludeBlockNode(block_var, extra_context, use_parent_context)
