@@ -1,4 +1,5 @@
 from itertools import groupby
+from warnings import warn
 
 from django import forms
 from django.conf import settings
@@ -14,13 +15,15 @@ from django.utils.html import mark_safe
 from django.utils.translation import gettext_lazy as _
 
 from wagtail import hooks
+from wagtail.admin.forms.formsets import BaseFormSetMixin
 from wagtail.admin.widgets import AdminPageChooser
 from wagtail.models import (
-    PAGE_PERMISSION_TYPE_CHOICES,
+    PAGE_PERMISSION_CODENAMES,
     PAGE_PERMISSION_TYPES,
     GroupPagePermission,
     Page,
 )
+from wagtail.utils.deprecation import RemovedInWagtail70Warning
 
 User = get_user_model()
 
@@ -29,6 +32,11 @@ standard_fields = {"email", "first_name", "last_name", "is_superuser", "groups"}
 # Custom fields
 if hasattr(settings, "WAGTAIL_USER_CUSTOM_FIELDS"):
     custom_fields = set(settings.WAGTAIL_USER_CUSTOM_FIELDS)
+    warn(
+        "The `WAGTAIL_USER_CUSTOM_FIELDS` setting is deprecated. Use a custom "
+        "`UserViewSet` subclass and override `get_form_class()` instead.",
+        RemovedInWagtail70Warning,
+    )
 else:
     custom_fields = set()
 
@@ -89,12 +97,14 @@ class UserForm(UsernameForm):
         required=False,
         widget=forms.PasswordInput(attrs={"autocomplete": "new-password"}),
         help_text=_("Leave blank if not changing."),
+        strip=False,
     )
     password2 = forms.CharField(
         label=_("Password confirmation"),
         required=False,
         widget=forms.PasswordInput(attrs={"autocomplete": "new-password"}),
         help_text=_("Enter the same password as above, for verification."),
+        strip=False,
     )
 
     is_superuser = forms.BooleanField(
@@ -290,17 +300,29 @@ class PagePermissionsForm(forms.Form):
         queryset=Page.objects.all(),
         widget=AdminPageChooser(show_edit_link=False, can_choose_root=True),
     )
-    permission_types = forms.MultipleChoiceField(
-        choices=PAGE_PERMISSION_TYPE_CHOICES,
+    permissions = forms.ModelMultipleChoiceField(
+        queryset=Permission.objects.filter(
+            content_type__app_label="wagtailcore",
+            content_type__model="page",
+            codename__in=PAGE_PERMISSION_CODENAMES,
+        )
+        .select_related("content_type")
+        .order_by("codename"),
+        # Use codename as the field to use for the option values rather than pk,
+        # to minimise the changes needed since we moved to the Permission model
+        # and to ease testing.
+        # Django advises `to_field_name` to be a unique field. While `codename`
+        # is not unique by itself, it is unique together with `content_type`, so
+        # it is unique in the context of the above queryset.
+        to_field_name="codename",
         required=False,
         widget=forms.CheckboxSelectMultiple,
     )
 
 
-class BaseGroupPagePermissionFormSet(forms.BaseFormSet):
-    permission_types = (
-        PAGE_PERMISSION_TYPES  # defined here for easy access from templates
-    )
+class BaseGroupPagePermissionFormSet(BaseFormSetMixin, forms.BaseFormSet):
+    # defined here for easy access from templates
+    permission_types = PAGE_PERMISSION_TYPES
 
     def __init__(self, data=None, files=None, instance=None, prefix="page_permissions"):
         if instance is None:
@@ -310,7 +332,7 @@ class BaseGroupPagePermissionFormSet(forms.BaseFormSet):
             full_page_permissions = []
         else:
             full_page_permissions = instance.page_permissions.select_related(
-                "page"
+                "page", "permission"
             ).order_by("page")
 
         self.instance = instance
@@ -324,19 +346,11 @@ class BaseGroupPagePermissionFormSet(forms.BaseFormSet):
             initial_data.append(
                 {
                     "page": page,
-                    "permission_types": [pp.permission_type for pp in page_permissions],
+                    "permissions": [pp.permission for pp in page_permissions],
                 }
             )
 
         super().__init__(data, files, initial=initial_data, prefix=prefix)
-        for form in self.forms:
-            form.fields["DELETE"].widget = forms.HiddenInput()
-
-    @property
-    def empty_form(self):
-        empty_form = super().empty_form
-        empty_form.fields["DELETE"].widget = forms.HiddenInput()
-        return empty_form
 
     def clean(self):
         """Checks that no two forms refer to the same page object"""
@@ -364,7 +378,7 @@ class BaseGroupPagePermissionFormSet(forms.BaseFormSet):
                 "Cannot save a GroupPagePermissionFormSet for an unsaved group instance"
             )
 
-        # get a set of (page, permission_type) tuples for all ticked permissions
+        # get a set of (page, permission) tuples for all ticked permissions
         forms_to_save = [
             form
             for form in self.forms
@@ -373,10 +387,8 @@ class BaseGroupPagePermissionFormSet(forms.BaseFormSet):
 
         final_permission_records = set()
         for form in forms_to_save:
-            for permission_type in form.cleaned_data["permission_types"]:
-                final_permission_records.add(
-                    (form.cleaned_data["page"], permission_type)
-                )
+            for permission in form.cleaned_data["permissions"]:
+                final_permission_records.add((form.cleaned_data["page"], permission))
 
         # fetch the group's existing page permission records, and from that, build a list
         # of records to be created / deleted
@@ -384,8 +396,8 @@ class BaseGroupPagePermissionFormSet(forms.BaseFormSet):
         permission_records_to_keep = set()
 
         for pp in self.instance.page_permissions.all():
-            if (pp.page, pp.permission_type) in final_permission_records:
-                permission_records_to_keep.add((pp.page, pp.permission_type))
+            if (pp.page, pp.permission) in final_permission_records:
+                permission_records_to_keep.add((pp.page, pp.permission))
             else:
                 permission_ids_to_delete.append(pp.pk)
 
@@ -395,9 +407,11 @@ class BaseGroupPagePermissionFormSet(forms.BaseFormSet):
         GroupPagePermission.objects.bulk_create(
             [
                 GroupPagePermission(
-                    group=self.instance, page=page, permission_type=permission_type
+                    group=self.instance,
+                    page=page,
+                    permission=permission,
                 )
-                for (page, permission_type) in permissions_to_add
+                for (page, permission) in permissions_to_add
             ]
         )
 
