@@ -2,13 +2,10 @@
 # This is based on sendfiles builtin "simple" backend but uses a StreamingHttpResponse
 
 import os
-import re
 import stat
-
 from email.utils import mktime_tz, parsedate_tz
-from wsgiref.util import FileWrapper
 
-from django.http import HttpResponseNotModified, StreamingHttpResponse
+from django.http import FileResponse, HttpResponseNotModified
 from django.utils.http import http_date
 
 
@@ -16,17 +13,19 @@ def sendfile(request, filename, **kwargs):
     # Respect the If-Modified-Since header.
     statobj = os.stat(filename)
 
-    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
-                              statobj[stat.ST_MTIME], statobj[stat.ST_SIZE]):
+    if not was_modified_since(
+        request.headers.get("if-modified-since"),
+        statobj[stat.ST_MTIME],
+    ):
         return HttpResponseNotModified()
 
-    response = StreamingHttpResponse(FileWrapper(open(filename, 'rb')))
+    response = FileResponse(open(filename, "rb"))
 
     response["Last-Modified"] = http_date(statobj[stat.ST_MTIME])
     return response
 
 
-def was_modified_since(header=None, mtime=0, size=0):
+def was_modified_since(header=None, mtime=0):
     """
     Was something modified since the user last downloaded it?
 
@@ -36,24 +35,16 @@ def was_modified_since(header=None, mtime=0, size=0):
 
     mtime
       This is the modification time of the item we're talking about.
-
-    size
-      This is the size of the item we're talking about.
     """
     try:
         if header is None:
             raise ValueError
-        matches = re.match(r"^([^;]+)(; length=([0-9]+))?$", header,
-                           re.IGNORECASE)
-        header_date = parsedate_tz(matches.group(1))
+        header_date = parsedate_tz(header)
         if header_date is None:
             raise ValueError
         header_mtime = mktime_tz(header_date)
-        header_len = matches.group(3)
-        if header_len and int(header_len) != size:
-            raise ValueError
         if mtime > header_mtime:
             raise ValueError
-    except (AttributeError, ValueError, OverflowError):
+    except (ValueError, OverflowError):
         return True
     return False
