@@ -1,4 +1,3 @@
-import io
 import json
 import os
 import subprocess
@@ -15,14 +14,14 @@ class assets_mixin:
         try:
             subprocess.check_call(["npm", "run", "build"])
         except (OSError, subprocess.CalledProcessError) as e:
-            print("Error compiling assets: " + str(e))  # noqa
+            print("Error compiling assets: " + str(e))  # noqa: T201
             raise SystemExit(1)
 
     def publish_assets(self):
         try:
             subprocess.check_call(["npm", "publish", "client"])
         except (OSError, subprocess.CalledProcessError) as e:
-            print("Error publishing front-end assets: " + str(e))  # noqa
+            print("Error publishing front-end assets: " + str(e))  # noqa: T201
             raise SystemExit(1)
 
     def bump_client_version(self):
@@ -30,21 +29,23 @@ class assets_mixin:
         Writes the current Wagtail version number into package.json
         """
         path = os.path.join(".", "client", "package.json")
-        input_file = io.open(path, "r")
 
         try:
-            package = json.loads(input_file.read().decode("utf-8"))
-        except (ValueError) as e:
-            print("Unable to read " + path + " " + e)  # noqa
+            with open(path, "r") as f:
+                package = json.loads(f.read())
+        except ValueError as e:
+            print("Unable to read " + path + " " + e)  # noqa: T201
             raise SystemExit(1)
 
         package["version"] = __semver__
 
         try:
-            with io.open(path, "w", encoding="utf-8") as f:
+            with open(path, "w", encoding="utf-8") as f:
                 f.write(str(json.dumps(package, indent=2, ensure_ascii=False)))
-        except (IOError) as e:
-            print("Error setting the version for front-end assets: " + str(e))  # noqa
+        except OSError as e:
+            print(  # noqa: T201
+                "Error setting the version for front-end assets: " + str(e)
+            )
             raise SystemExit(1)
 
 
@@ -70,20 +71,19 @@ class sdist(base_sdist, assets_mixin):
 
 
 class check_bdist_egg(bdist_egg):
-
     # If this file does not exist, warn the user to compile the assets
     sentinel_dir = "wagtail/wagtailadmin/static/"
 
     def run(self):
         bdist_egg.run(self)
         if not os.path.isdir(self.sentinel_dir):
-            print(
+            print(  # noqa: T201
                 "\n".join(
-                    [  # noqa
+                    [
                         "************************************************************",
                         "The front end assets for Wagtail are missing.",
                         "To generate the assets, please refer to the documentation in",
-                        "docs/contributing/css_guidelines.rst",
+                        "docs/contributing/developing.md",
                         "************************************************************",
                     ]
                 )
