@@ -1,8 +1,18 @@
+from warnings import warn
+
 from django.forms.widgets import Widget
 from django.utils.safestring import mark_safe
 
+from wagtail.utils.deprecation import RemovedInWagtail70Warning
+
 
 class WidgetWithScript(Widget):
+    warn(
+        "The usage of `WidgetWithScript` hook is deprecated. Use external scripts instead.",
+        category=RemovedInWagtail70Warning,
+        stacklevel=3,
+    )
+
     def render_html(self, name, value, attrs):
         """Render the HTML (non-JS) portion of the field markup"""
         return super().render(name, value, attrs)
@@ -19,16 +29,18 @@ class WidgetWithScript(Widget):
         # no point trying to come up with sensible semantics for when 'id' is missing from attrs,
         # so let's make sure it fails early in the process
         try:
-            id_ = attrs['id']
+            id_ = attrs["id"]
         except (KeyError, TypeError):
-            raise TypeError("WidgetWithScript cannot be rendered without an 'id' attribute")
+            raise TypeError(
+                "WidgetWithScript cannot be rendered without an 'id' attribute"
+            )
 
         value_data = self.get_value_data(value)
         widget_html = self.render_html(name, value_data, attrs)
 
         js = self.render_js_init(id_, name, value_data)
-        out = '{0}<script>{1}</script>'.format(widget_html, js)
+        out = f"{widget_html}<script>{js}</script>"
         return mark_safe(out)
 
     def render_js_init(self, id_, name, value):
-        return ''
+        return ""
